// This code conforms with the UFC specification version 2017.1.0
// and was automatically generated by FFC version 2017.1.0.
//
// This code was generated with the option '-l dolfin' and
// contains DOLFIN-specific wrappers that depend on DOLFIN.
// 
// This code was generated with the following parameters:
// 
//   add_tabulate_tensor_timing:     False
//   convert_exceptions_to_warnings: False
//   cpp_optimize:                   True
//   cpp_optimize_flags:             '-O2'
//   epsilon:                        1e-14
//   error_control:                  False
//   form_postfix:                   True
//   format:                         'dolfin'
//   generate_dummy_tabulate_tensor: False
//   max_signature_length:           0
//   optimize:                       True
//   precision:                      None
//   quadrature_degree:              None
//   quadrature_rule:                None
//   representation:                 'auto'
//   split:                          False

#ifndef __BIHARMONIC_H
#define __BIHARMONIC_H
#include <algorithm>
#include <cmath>
#include <stdexcept>
#include <ufc.h>

class biharmonic_finite_element_0: public ufc::finite_element
{
public:

  biharmonic_finite_element_0() : ufc::finite_element()
  {
    // Do nothing
  }

  ~biharmonic_finite_element_0() override
  {
    // Do nothing
  }

  const char * signature() const final override
  {
    return "FiniteElement('Lagrange', triangle, 1)";
  }

  ufc::shape cell_shape() const final override
  {
    return ufc::shape::triangle;
  }

  std::size_t topological_dimension() const final override
  {
    return 2;
  }

  std::size_t geometric_dimension() const final override
  {
    return 2;
  }

  std::size_t space_dimension() const final override
  {
    return 3;
  }

  std::size_t value_rank() const final override
  {
    return 0;
  }

  std::size_t value_dimension(std::size_t i) const final override
  {
    return 1;
  }

  std::size_t value_size() const final override
  {
    return 1;
  }

  std::size_t reference_value_rank() const final override
  {
    return 0;
  }

  std::size_t reference_value_dimension(std::size_t i) const final override
  {
    return 1;
  }

  std::size_t reference_value_size() const final override
  {
    return 1;
  }

  std::size_t degree() const final override
  {
    return 1;
  }

  const char * family() const final override
  {
    return "Lagrange";
  }

  static void _evaluate_basis(std::size_t i,
                              double * values,
                              const double * x,
                              const double * coordinate_dofs,
                              int cell_orientation)
  {
    // Compute Jacobian
    double J[4];
    compute_jacobian_triangle_2d(J, coordinate_dofs);
    
    // Compute Jacobian inverse and determinant
    double K[4];
    double detJ;
    compute_jacobian_inverse_triangle_2d(K, detJ, J);
    
    
    // Compute constants
    const double C0 = coordinate_dofs[2] + coordinate_dofs[4];
    const double C1 = coordinate_dofs[3] + coordinate_dofs[5];
    
    // Get coordinates and map to the reference (FIAT) element
    double X = (J[1]*(C1 - 2.0*x[1]) + J[3]*(2.0*x[0] - C0)) / detJ;
    double Y = (J[0]*(2.0*x[1] - C1) + J[2]*(C0 - 2.0*x[0])) / detJ;
    
    // Reset values
    *values = 0.0;
    switch (i)
    {
    case 0:
      {
        
      // Array of basisvalues
      double basisvalues[3] = {0.0, 0.0, 0.0};
      
      // Declare helper variables
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      
      // Compute basisvalues
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[1] *= std::sqrt(3.0);
      
      // Table(s) of coefficients
      static const double coefficients0[3] = \
      {0.4714045207910317, -0.2886751345948129, -0.1666666666666667};
      
      // Compute value(s)
      for (unsigned int r = 0; r < 3; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      } // end loop over 'r'
        break;
      }
    case 1:
      {
        
      // Array of basisvalues
      double basisvalues[3] = {0.0, 0.0, 0.0};
      
      // Declare helper variables
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      
      // Compute basisvalues
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[1] *= std::sqrt(3.0);
      
      // Table(s) of coefficients
      static const double coefficients0[3] = \
      {0.4714045207910317, 0.2886751345948129, -0.1666666666666667};
      
      // Compute value(s)
      for (unsigned int r = 0; r < 3; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      } // end loop over 'r'
        break;
      }
    case 2:
      {
        
      // Array of basisvalues
      double basisvalues[3] = {0.0, 0.0, 0.0};
      
      // Declare helper variables
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      
      // Compute basisvalues
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[1] *= std::sqrt(3.0);
      
      // Table(s) of coefficients
      static const double coefficients0[3] = \
      {0.4714045207910316, 0.0, 0.3333333333333333};
      
      // Compute value(s)
      for (unsigned int r = 0; r < 3; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      } // end loop over 'r'
        break;
      }
    }
    
  }

  void evaluate_basis(std::size_t i,
                      double * values,
                      const double * x,
                      const double * coordinate_dofs,
                      int cell_orientation) const final override
  {
    _evaluate_basis(i, values, x, coordinate_dofs, cell_orientation);
  }

  static void _evaluate_basis_all(double * values,
                                  const double * x,
                                  const double * coordinate_dofs,
                                  int cell_orientation)
  {
    // Helper variable to hold values of a single dof.
    double dof_values = 0.0;
    
    // Loop dofs and call evaluate_basis
    for (unsigned int r = 0; r < 3; r++)
    {
      _evaluate_basis(r, &dof_values, x, coordinate_dofs, cell_orientation);
      values[r] = dof_values;
    } // end loop over 'r'
  }

  void evaluate_basis_all(double * values,
                          const double * x,
                          const double * coordinate_dofs,
                          int cell_orientation) const final override
  {
    _evaluate_basis_all(values, x, coordinate_dofs, cell_orientation);
  }

  static void _evaluate_basis_derivatives(std::size_t i,
                                          std::size_t n,
                                          double * values,
                                          const double * x,
                                          const double * coordinate_dofs,
                                          int cell_orientation)
  {
    
    // Compute number of derivatives.
    unsigned int num_derivatives = 1;
    for (unsigned int r = 0; r < n; r++)
    {
      num_derivatives *= 2;
    } // end loop over 'r'
    
    // Reset values. Assuming that values is always an array.
    for (unsigned int r = 0; r < num_derivatives; r++)
    {
      values[r] = 0.0;
    } // end loop over 'r'
    
    // Call evaluate_basis if order of derivatives is equal to zero.
    if (n == 0)
    {
      _evaluate_basis(i, values, x, coordinate_dofs, cell_orientation);
      return ;
    }
    
    // If order of derivatives is greater than the maximum polynomial degree, return zeros.
    if (n > 1)
    {
    return ;
    }
    
    // Compute Jacobian
    double J[4];
    compute_jacobian_triangle_2d(J, coordinate_dofs);
    
    // Compute Jacobian inverse and determinant
    double K[4];
    double detJ;
    compute_jacobian_inverse_triangle_2d(K, detJ, J);
    
    
    // Compute constants
    const double C0 = coordinate_dofs[2] + coordinate_dofs[4];
    const double C1 = coordinate_dofs[3] + coordinate_dofs[5];
    
    // Get coordinates and map to the reference (FIAT) element
    double X = (J[1]*(C1 - 2.0*x[1]) + J[3]*(2.0*x[0] - C0)) / detJ;
    double Y = (J[0]*(2.0*x[1] - C1) + J[2]*(C0 - 2.0*x[0])) / detJ;
    
    // Declare two dimensional array that holds combinations of derivatives and initialise
    unsigned int combinations[2][1];
    for (unsigned int row = 0; row < 2; row++)
    {
      for (unsigned int col = 0; col < 1; col++)
        combinations[row][col] = 0;
    }
    
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 1)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[2][2] = {{K[0], K[1]}, {K[2], K[3]}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double transform[2][2];
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    switch (i)
    {
    case 0:
      {
        
      // Array of basisvalues
      double basisvalues[3] = {0.0, 0.0, 0.0};
      
      // Declare helper variables
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      
      // Compute basisvalues
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[1] *= std::sqrt(3.0);
      
      // Table(s) of coefficients
      static const double coefficients0[3] = \
      {0.4714045207910317, -0.2886751345948129, -0.1666666666666667};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[3][3] = \
      {{0.0, 0.0, 0.0},
      {4.89897948556635, 0.0, 0.0},
      {0.0, 0.0, 0.0}};
      
      static const double dmats1[3][3] = \
      {{0.0, 0.0, 0.0},
      {2.449489742783182, 0.0, 0.0},
      {4.242640687119285, 0.0, 0.0}};
      
      // Compute reference derivatives.
      // Declare array of derivatives on FIAT element.
      double derivatives[2];
      for (unsigned int r = 0; r < 2; r++)
      {
        derivatives[r] = 0.0;
      } // end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[3][3] = \
      {{1.0, 0.0, 0.0},
      {0.0, 1.0, 0.0},
      {0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[3][3] = \
      {{1.0, 0.0, 0.0},
      {0.0, 1.0, 0.0},
      {0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 3; t++)
        {
          for (unsigned int u = 0; u < 3; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          } // end loop over 'u'
        } // end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            } // end loop over 'u'
          } // end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              for (unsigned int tu = 0; tu < 3; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              } // end loop over 'tu'
            } // end loop over 'u'
          } // end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              for (unsigned int tu = 0; tu < 3; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              } // end loop over 'tu'
            } // end loop over 'u'
          } // end loop over 't'
          }
          
        } // end loop over 's'
        for (unsigned int s = 0; s < 3; s++)
        {
          for (unsigned int t = 0; t < 3; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          } // end loop over 't'
        } // end loop over 's'
      } // end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        } // end loop over 's'
      } // end loop over 'r'
        break;
      }
    case 1:
      {
        
      // Array of basisvalues
      double basisvalues[3] = {0.0, 0.0, 0.0};
      
      // Declare helper variables
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      
      // Compute basisvalues
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[1] *= std::sqrt(3.0);
      
      // Table(s) of coefficients
      static const double coefficients0[3] = \
      {0.4714045207910317, 0.2886751345948129, -0.1666666666666667};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[3][3] = \
      {{0.0, 0.0, 0.0},
      {4.89897948556635, 0.0, 0.0},
      {0.0, 0.0, 0.0}};
      
      static const double dmats1[3][3] = \
      {{0.0, 0.0, 0.0},
      {2.449489742783182, 0.0, 0.0},
      {4.242640687119285, 0.0, 0.0}};
      
      // Compute reference derivatives.
      // Declare array of derivatives on FIAT element.
      double derivatives[2];
      for (unsigned int r = 0; r < 2; r++)
      {
        derivatives[r] = 0.0;
      } // end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[3][3] = \
      {{1.0, 0.0, 0.0},
      {0.0, 1.0, 0.0},
      {0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[3][3] = \
      {{1.0, 0.0, 0.0},
      {0.0, 1.0, 0.0},
      {0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 3; t++)
        {
          for (unsigned int u = 0; u < 3; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          } // end loop over 'u'
        } // end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            } // end loop over 'u'
          } // end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              for (unsigned int tu = 0; tu < 3; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              } // end loop over 'tu'
            } // end loop over 'u'
          } // end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              for (unsigned int tu = 0; tu < 3; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              } // end loop over 'tu'
            } // end loop over 'u'
          } // end loop over 't'
          }
          
        } // end loop over 's'
        for (unsigned int s = 0; s < 3; s++)
        {
          for (unsigned int t = 0; t < 3; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          } // end loop over 't'
        } // end loop over 's'
      } // end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        } // end loop over 's'
      } // end loop over 'r'
        break;
      }
    case 2:
      {
        
      // Array of basisvalues
      double basisvalues[3] = {0.0, 0.0, 0.0};
      
      // Declare helper variables
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      
      // Compute basisvalues
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[1] *= std::sqrt(3.0);
      
      // Table(s) of coefficients
      static const double coefficients0[3] = \
      {0.4714045207910316, 0.0, 0.3333333333333333};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[3][3] = \
      {{0.0, 0.0, 0.0},
      {4.89897948556635, 0.0, 0.0},
      {0.0, 0.0, 0.0}};
      
      static const double dmats1[3][3] = \
      {{0.0, 0.0, 0.0},
      {2.449489742783182, 0.0, 0.0},
      {4.242640687119285, 0.0, 0.0}};
      
      // Compute reference derivatives.
      // Declare array of derivatives on FIAT element.
      double derivatives[2];
      for (unsigned int r = 0; r < 2; r++)
      {
        derivatives[r] = 0.0;
      } // end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[3][3] = \
      {{1.0, 0.0, 0.0},
      {0.0, 1.0, 0.0},
      {0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[3][3] = \
      {{1.0, 0.0, 0.0},
      {0.0, 1.0, 0.0},
      {0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 3; t++)
        {
          for (unsigned int u = 0; u < 3; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          } // end loop over 'u'
        } // end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            } // end loop over 'u'
          } // end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              for (unsigned int tu = 0; tu < 3; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              } // end loop over 'tu'
            } // end loop over 'u'
          } // end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              for (unsigned int tu = 0; tu < 3; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              } // end loop over 'tu'
            } // end loop over 'u'
          } // end loop over 't'
          }
          
        } // end loop over 's'
        for (unsigned int s = 0; s < 3; s++)
        {
          for (unsigned int t = 0; t < 3; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          } // end loop over 't'
        } // end loop over 's'
      } // end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        } // end loop over 's'
      } // end loop over 'r'
        break;
      }
    }
    
  }

  void evaluate_basis_derivatives(std::size_t i,
                                  std::size_t n,
                                  double * values,
                                  const double * x,
                                  const double * coordinate_dofs,
                                  int cell_orientation) const final override
  {
    _evaluate_basis_derivatives(i, n, values, x, coordinate_dofs, cell_orientation);
  }

  static void _evaluate_basis_derivatives_all(std::size_t n,
                                              double * values,
                                              const double * x,
                                              const double * coordinate_dofs,
                                              int cell_orientation)
  {
    // Call evaluate_basis_all if order of derivatives is equal to zero.
    if (n == 0)
    {
      _evaluate_basis_all(values, x, coordinate_dofs, cell_orientation);
      return ;
    }
    
    // Compute number of derivatives.
    unsigned int num_derivatives = 1;
    for (unsigned int r = 0; r < n; r++)
    {
      num_derivatives *= 2;
    } // end loop over 'r'
    
    // Set values equal to zero.
    for (unsigned int r = 0; r < 3; r++)
    {
      for (unsigned int s = 0; s < num_derivatives; s++)
      {
        values[r*num_derivatives + s] = 0.0;
      } // end loop over 's'
    } // end loop over 'r'
    
    // If order of derivatives is greater than the maximum polynomial degree, return zeros.
    if (n > 1)
    {
      return ;
    }
    
    // Helper variable to hold values of a single dof.
    double dof_values[2];
    for (unsigned int r = 0; r < 2; r++)
    {
      dof_values[r] = 0.0;
    } // end loop over 'r'
    
    // Loop dofs and call evaluate_basis_derivatives.
    for (unsigned int r = 0; r < 3; r++)
    {
      _evaluate_basis_derivatives(r, n, dof_values, x, coordinate_dofs, cell_orientation);
      for (unsigned int s = 0; s < num_derivatives; s++)
      {
        values[r*num_derivatives + s] = dof_values[s];
      } // end loop over 's'
    } // end loop over 'r'
  }

  void evaluate_basis_derivatives_all(std::size_t n,
                                      double * values,
                                      const double * x,
                                      const double * coordinate_dofs,
                                      int cell_orientation) const final override
  {
    _evaluate_basis_derivatives_all(n, values, x, coordinate_dofs, cell_orientation);
  }

  double evaluate_dof(std::size_t i,
                      const ufc::function& f,
                      const double * coordinate_dofs,
                      int cell_orientation,
                      const ufc::cell& c) const final override
  {
    // Declare variables for result of evaluation
    double vals[1];
    
    // Declare variable for physical coordinates
    double y[2];
    switch (i)
    {
    case 0:
      {
        y[0] = coordinate_dofs[0];
      y[1] = coordinate_dofs[1];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 1:
      {
        y[0] = coordinate_dofs[2];
      y[1] = coordinate_dofs[3];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 2:
      {
        y[0] = coordinate_dofs[4];
      y[1] = coordinate_dofs[5];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    }
    
    return 0.0;
  }

  void evaluate_dofs(double * values,
                             const ufc::function& f,
                             const double * coordinate_dofs,
                             int cell_orientation,
                             const ufc::cell& c) const final override
  {
    // Declare variables for result of evaluation
    double vals[1];
    
    // Declare variable for physical coordinates
    double y[2];
    y[0] = coordinate_dofs[0];
    y[1] = coordinate_dofs[1];
    f.evaluate(vals, y, c);
    values[0] = vals[0];
    y[0] = coordinate_dofs[2];
    y[1] = coordinate_dofs[3];
    f.evaluate(vals, y, c);
    values[1] = vals[0];
    y[0] = coordinate_dofs[4];
    y[1] = coordinate_dofs[5];
    f.evaluate(vals, y, c);
    values[2] = vals[0];
  }

  void interpolate_vertex_values(double * vertex_values,
                                 const double * dof_values,
                                 const double * coordinate_dofs,
                                 int cell_orientation,
                                 const ufc::cell& c) const final override
  {
    // Evaluate function and change variables
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
  }

  void tabulate_dof_coordinates(double * dof_coordinates,
                                const double * coordinate_dofs) const final override
  {
    dof_coordinates[0] = coordinate_dofs[0];
    dof_coordinates[1] = coordinate_dofs[1];
    dof_coordinates[2] = coordinate_dofs[2];
    dof_coordinates[3] = coordinate_dofs[3];
    dof_coordinates[4] = coordinate_dofs[4];
    dof_coordinates[5] = coordinate_dofs[5];
  }

  std::size_t num_sub_elements() const final override
  {
    return 0;
  }

  ufc::finite_element * create_sub_element(std::size_t i) const final override
  {
    return 0;
  }

  ufc::finite_element * create() const final override
  {
    return new biharmonic_finite_element_0();
  }

};


class biharmonic_finite_element_1: public ufc::finite_element
{
public:

  biharmonic_finite_element_1() : ufc::finite_element()
  {
    // Do nothing
  }

  ~biharmonic_finite_element_1() override
  {
    // Do nothing
  }

  const char * signature() const final override
  {
    return "VectorElement(FiniteElement('Lagrange', triangle, 1), dim=2)";
  }

  ufc::shape cell_shape() const final override
  {
    return ufc::shape::triangle;
  }

  std::size_t topological_dimension() const final override
  {
    return 2;
  }

  std::size_t geometric_dimension() const final override
  {
    return 2;
  }

  std::size_t space_dimension() const final override
  {
    return 6;
  }

  std::size_t value_rank() const final override
  {
    return 1;
  }

  std::size_t value_dimension(std::size_t i) const final override
  {
    switch (i)
    {
    case 0:
      {
        return 2;
        break;
      }
    }
    
    return 0;
  }

  std::size_t value_size() const final override
  {
    return 2;
  }

  std::size_t reference_value_rank() const final override
  {
    return 1;
  }

  std::size_t reference_value_dimension(std::size_t i) const final override
  {
    switch (i)
    {
    case 0:
      {
        return 2;
        break;
      }
    }
    
    return 0;
  }

  std::size_t reference_value_size() const final override
  {
    return 2;
  }

  std::size_t degree() const final override
  {
    return 1;
  }

  const char * family() const final override
  {
    return "Lagrange";
  }

  static void _evaluate_basis(std::size_t i,
                              double * values,
                              const double * x,
                              const double * coordinate_dofs,
                              int cell_orientation)
  {
    // Compute Jacobian
    double J[4];
    compute_jacobian_triangle_2d(J, coordinate_dofs);
    
    // Compute Jacobian inverse and determinant
    double K[4];
    double detJ;
    compute_jacobian_inverse_triangle_2d(K, detJ, J);
    
    
    // Compute constants
    const double C0 = coordinate_dofs[2] + coordinate_dofs[4];
    const double C1 = coordinate_dofs[3] + coordinate_dofs[5];
    
    // Get coordinates and map to the reference (FIAT) element
    double X = (J[1]*(C1 - 2.0*x[1]) + J[3]*(2.0*x[0] - C0)) / detJ;
    double Y = (J[0]*(2.0*x[1] - C1) + J[2]*(C0 - 2.0*x[0])) / detJ;
    
    // Reset values
    values[0] = 0.0;
    values[1] = 0.0;
    switch (i)
    {
    case 0:
      {
        
      // Array of basisvalues
      double basisvalues[3] = {0.0, 0.0, 0.0};
      
      // Declare helper variables
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      
      // Compute basisvalues
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[1] *= std::sqrt(3.0);
      
      // Table(s) of coefficients
      static const double coefficients0[3] = \
      {0.4714045207910317, -0.2886751345948129, -0.1666666666666667};
      
      // Compute value(s)
      for (unsigned int r = 0; r < 3; r++)
      {
        values[0] += coefficients0[r]*basisvalues[r];
      } // end loop over 'r'
        break;
      }
    case 1:
      {
        
      // Array of basisvalues
      double basisvalues[3] = {0.0, 0.0, 0.0};
      
      // Declare helper variables
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      
      // Compute basisvalues
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[1] *= std::sqrt(3.0);
      
      // Table(s) of coefficients
      static const double coefficients0[3] = \
      {0.4714045207910317, 0.2886751345948129, -0.1666666666666667};
      
      // Compute value(s)
      for (unsigned int r = 0; r < 3; r++)
      {
        values[0] += coefficients0[r]*basisvalues[r];
      } // end loop over 'r'
        break;
      }
    case 2:
      {
        
      // Array of basisvalues
      double basisvalues[3] = {0.0, 0.0, 0.0};
      
      // Declare helper variables
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      
      // Compute basisvalues
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[1] *= std::sqrt(3.0);
      
      // Table(s) of coefficients
      static const double coefficients0[3] = \
      {0.4714045207910316, 0.0, 0.3333333333333333};
      
      // Compute value(s)
      for (unsigned int r = 0; r < 3; r++)
      {
        values[0] += coefficients0[r]*basisvalues[r];
      } // end loop over 'r'
        break;
      }
    case 3:
      {
        
      // Array of basisvalues
      double basisvalues[3] = {0.0, 0.0, 0.0};
      
      // Declare helper variables
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      
      // Compute basisvalues
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[1] *= std::sqrt(3.0);
      
      // Table(s) of coefficients
      static const double coefficients0[3] = \
      {0.4714045207910317, -0.2886751345948129, -0.1666666666666667};
      
      // Compute value(s)
      for (unsigned int r = 0; r < 3; r++)
      {
        values[1] += coefficients0[r]*basisvalues[r];
      } // end loop over 'r'
        break;
      }
    case 4:
      {
        
      // Array of basisvalues
      double basisvalues[3] = {0.0, 0.0, 0.0};
      
      // Declare helper variables
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      
      // Compute basisvalues
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[1] *= std::sqrt(3.0);
      
      // Table(s) of coefficients
      static const double coefficients0[3] = \
      {0.4714045207910317, 0.2886751345948129, -0.1666666666666667};
      
      // Compute value(s)
      for (unsigned int r = 0; r < 3; r++)
      {
        values[1] += coefficients0[r]*basisvalues[r];
      } // end loop over 'r'
        break;
      }
    case 5:
      {
        
      // Array of basisvalues
      double basisvalues[3] = {0.0, 0.0, 0.0};
      
      // Declare helper variables
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      
      // Compute basisvalues
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[1] *= std::sqrt(3.0);
      
      // Table(s) of coefficients
      static const double coefficients0[3] = \
      {0.4714045207910316, 0.0, 0.3333333333333333};
      
      // Compute value(s)
      for (unsigned int r = 0; r < 3; r++)
      {
        values[1] += coefficients0[r]*basisvalues[r];
      } // end loop over 'r'
        break;
      }
    }
    
  }

  void evaluate_basis(std::size_t i,
                      double * values,
                      const double * x,
                      const double * coordinate_dofs,
                      int cell_orientation) const final override
  {
    _evaluate_basis(i, values, x, coordinate_dofs, cell_orientation);
  }

  static void _evaluate_basis_all(double * values,
                                  const double * x,
                                  const double * coordinate_dofs,
                                  int cell_orientation)
  {
    // Helper variable to hold values of a single dof.
    double dof_values[2] = {0.0, 0.0};
    
    // Loop dofs and call evaluate_basis
    for (unsigned int r = 0; r < 6; r++)
    {
      _evaluate_basis(r, dof_values, x, coordinate_dofs, cell_orientation);
      for (unsigned int s = 0; s < 2; s++)
      {
        values[r*2 + s] = dof_values[s];
      } // end loop over 's'
    } // end loop over 'r'
  }

  void evaluate_basis_all(double * values,
                          const double * x,
                          const double * coordinate_dofs,
                          int cell_orientation) const final override
  {
    _evaluate_basis_all(values, x, coordinate_dofs, cell_orientation);
  }

  static void _evaluate_basis_derivatives(std::size_t i,
                                          std::size_t n,
                                          double * values,
                                          const double * x,
                                          const double * coordinate_dofs,
                                          int cell_orientation)
  {
    
    // Compute number of derivatives.
    unsigned int num_derivatives = 1;
    for (unsigned int r = 0; r < n; r++)
    {
      num_derivatives *= 2;
    } // end loop over 'r'
    
    // Reset values. Assuming that values is always an array.
    for (unsigned int r = 0; r < 2*num_derivatives; r++)
    {
      values[r] = 0.0;
    } // end loop over 'r'
    
    // Call evaluate_basis if order of derivatives is equal to zero.
    if (n == 0)
    {
      _evaluate_basis(i, values, x, coordinate_dofs, cell_orientation);
      return ;
    }
    
    // If order of derivatives is greater than the maximum polynomial degree, return zeros.
    if (n > 1)
    {
    return ;
    }
    
    // Compute Jacobian
    double J[4];
    compute_jacobian_triangle_2d(J, coordinate_dofs);
    
    // Compute Jacobian inverse and determinant
    double K[4];
    double detJ;
    compute_jacobian_inverse_triangle_2d(K, detJ, J);
    
    
    // Compute constants
    const double C0 = coordinate_dofs[2] + coordinate_dofs[4];
    const double C1 = coordinate_dofs[3] + coordinate_dofs[5];
    
    // Get coordinates and map to the reference (FIAT) element
    double X = (J[1]*(C1 - 2.0*x[1]) + J[3]*(2.0*x[0] - C0)) / detJ;
    double Y = (J[0]*(2.0*x[1] - C1) + J[2]*(C0 - 2.0*x[0])) / detJ;
    
    // Declare two dimensional array that holds combinations of derivatives and initialise
    unsigned int combinations[2][1];
    for (unsigned int row = 0; row < 2; row++)
    {
      for (unsigned int col = 0; col < 1; col++)
        combinations[row][col] = 0;
    }
    
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 1)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[2][2] = {{K[0], K[1]}, {K[2], K[3]}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double transform[2][2];
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    switch (i)
    {
    case 0:
      {
        
      // Array of basisvalues
      double basisvalues[3] = {0.0, 0.0, 0.0};
      
      // Declare helper variables
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      
      // Compute basisvalues
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[1] *= std::sqrt(3.0);
      
      // Table(s) of coefficients
      static const double coefficients0[3] = \
      {0.4714045207910317, -0.2886751345948129, -0.1666666666666667};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[3][3] = \
      {{0.0, 0.0, 0.0},
      {4.89897948556635, 0.0, 0.0},
      {0.0, 0.0, 0.0}};
      
      static const double dmats1[3][3] = \
      {{0.0, 0.0, 0.0},
      {2.449489742783182, 0.0, 0.0},
      {4.242640687119285, 0.0, 0.0}};
      
      // Compute reference derivatives.
      // Declare array of derivatives on FIAT element.
      double derivatives[2];
      for (unsigned int r = 0; r < 2; r++)
      {
        derivatives[r] = 0.0;
      } // end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[3][3] = \
      {{1.0, 0.0, 0.0},
      {0.0, 1.0, 0.0},
      {0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[3][3] = \
      {{1.0, 0.0, 0.0},
      {0.0, 1.0, 0.0},
      {0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 3; t++)
        {
          for (unsigned int u = 0; u < 3; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          } // end loop over 'u'
        } // end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            } // end loop over 'u'
          } // end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              for (unsigned int tu = 0; tu < 3; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              } // end loop over 'tu'
            } // end loop over 'u'
          } // end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              for (unsigned int tu = 0; tu < 3; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              } // end loop over 'tu'
            } // end loop over 'u'
          } // end loop over 't'
          }
          
        } // end loop over 's'
        for (unsigned int s = 0; s < 3; s++)
        {
          for (unsigned int t = 0; t < 3; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          } // end loop over 't'
        } // end loop over 's'
      } // end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        } // end loop over 's'
      } // end loop over 'r'
        break;
      }
    case 1:
      {
        
      // Array of basisvalues
      double basisvalues[3] = {0.0, 0.0, 0.0};
      
      // Declare helper variables
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      
      // Compute basisvalues
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[1] *= std::sqrt(3.0);
      
      // Table(s) of coefficients
      static const double coefficients0[3] = \
      {0.4714045207910317, 0.2886751345948129, -0.1666666666666667};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[3][3] = \
      {{0.0, 0.0, 0.0},
      {4.89897948556635, 0.0, 0.0},
      {0.0, 0.0, 0.0}};
      
      static const double dmats1[3][3] = \
      {{0.0, 0.0, 0.0},
      {2.449489742783182, 0.0, 0.0},
      {4.242640687119285, 0.0, 0.0}};
      
      // Compute reference derivatives.
      // Declare array of derivatives on FIAT element.
      double derivatives[2];
      for (unsigned int r = 0; r < 2; r++)
      {
        derivatives[r] = 0.0;
      } // end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[3][3] = \
      {{1.0, 0.0, 0.0},
      {0.0, 1.0, 0.0},
      {0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[3][3] = \
      {{1.0, 0.0, 0.0},
      {0.0, 1.0, 0.0},
      {0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 3; t++)
        {
          for (unsigned int u = 0; u < 3; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          } // end loop over 'u'
        } // end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            } // end loop over 'u'
          } // end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              for (unsigned int tu = 0; tu < 3; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              } // end loop over 'tu'
            } // end loop over 'u'
          } // end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              for (unsigned int tu = 0; tu < 3; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              } // end loop over 'tu'
            } // end loop over 'u'
          } // end loop over 't'
          }
          
        } // end loop over 's'
        for (unsigned int s = 0; s < 3; s++)
        {
          for (unsigned int t = 0; t < 3; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          } // end loop over 't'
        } // end loop over 's'
      } // end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        } // end loop over 's'
      } // end loop over 'r'
        break;
      }
    case 2:
      {
        
      // Array of basisvalues
      double basisvalues[3] = {0.0, 0.0, 0.0};
      
      // Declare helper variables
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      
      // Compute basisvalues
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[1] *= std::sqrt(3.0);
      
      // Table(s) of coefficients
      static const double coefficients0[3] = \
      {0.4714045207910316, 0.0, 0.3333333333333333};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[3][3] = \
      {{0.0, 0.0, 0.0},
      {4.89897948556635, 0.0, 0.0},
      {0.0, 0.0, 0.0}};
      
      static const double dmats1[3][3] = \
      {{0.0, 0.0, 0.0},
      {2.449489742783182, 0.0, 0.0},
      {4.242640687119285, 0.0, 0.0}};
      
      // Compute reference derivatives.
      // Declare array of derivatives on FIAT element.
      double derivatives[2];
      for (unsigned int r = 0; r < 2; r++)
      {
        derivatives[r] = 0.0;
      } // end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[3][3] = \
      {{1.0, 0.0, 0.0},
      {0.0, 1.0, 0.0},
      {0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[3][3] = \
      {{1.0, 0.0, 0.0},
      {0.0, 1.0, 0.0},
      {0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 3; t++)
        {
          for (unsigned int u = 0; u < 3; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          } // end loop over 'u'
        } // end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            } // end loop over 'u'
          } // end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              for (unsigned int tu = 0; tu < 3; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              } // end loop over 'tu'
            } // end loop over 'u'
          } // end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              for (unsigned int tu = 0; tu < 3; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              } // end loop over 'tu'
            } // end loop over 'u'
          } // end loop over 't'
          }
          
        } // end loop over 's'
        for (unsigned int s = 0; s < 3; s++)
        {
          for (unsigned int t = 0; t < 3; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          } // end loop over 't'
        } // end loop over 's'
      } // end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        } // end loop over 's'
      } // end loop over 'r'
        break;
      }
    case 3:
      {
        
      // Array of basisvalues
      double basisvalues[3] = {0.0, 0.0, 0.0};
      
      // Declare helper variables
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      
      // Compute basisvalues
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[1] *= std::sqrt(3.0);
      
      // Table(s) of coefficients
      static const double coefficients0[3] = \
      {0.4714045207910317, -0.2886751345948129, -0.1666666666666667};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[3][3] = \
      {{0.0, 0.0, 0.0},
      {4.89897948556635, 0.0, 0.0},
      {0.0, 0.0, 0.0}};
      
      static const double dmats1[3][3] = \
      {{0.0, 0.0, 0.0},
      {2.449489742783182, 0.0, 0.0},
      {4.242640687119285, 0.0, 0.0}};
      
      // Compute reference derivatives.
      // Declare array of derivatives on FIAT element.
      double derivatives[2];
      for (unsigned int r = 0; r < 2; r++)
      {
        derivatives[r] = 0.0;
      } // end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[3][3] = \
      {{1.0, 0.0, 0.0},
      {0.0, 1.0, 0.0},
      {0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[3][3] = \
      {{1.0, 0.0, 0.0},
      {0.0, 1.0, 0.0},
      {0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 3; t++)
        {
          for (unsigned int u = 0; u < 3; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          } // end loop over 'u'
        } // end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            } // end loop over 'u'
          } // end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              for (unsigned int tu = 0; tu < 3; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              } // end loop over 'tu'
            } // end loop over 'u'
          } // end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              for (unsigned int tu = 0; tu < 3; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              } // end loop over 'tu'
            } // end loop over 'u'
          } // end loop over 't'
          }
          
        } // end loop over 's'
        for (unsigned int s = 0; s < 3; s++)
        {
          for (unsigned int t = 0; t < 3; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          } // end loop over 't'
        } // end loop over 's'
      } // end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[num_derivatives + r] += transform[r][s]*derivatives[s];
        } // end loop over 's'
      } // end loop over 'r'
        break;
      }
    case 4:
      {
        
      // Array of basisvalues
      double basisvalues[3] = {0.0, 0.0, 0.0};
      
      // Declare helper variables
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      
      // Compute basisvalues
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[1] *= std::sqrt(3.0);
      
      // Table(s) of coefficients
      static const double coefficients0[3] = \
      {0.4714045207910317, 0.2886751345948129, -0.1666666666666667};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[3][3] = \
      {{0.0, 0.0, 0.0},
      {4.89897948556635, 0.0, 0.0},
      {0.0, 0.0, 0.0}};
      
      static const double dmats1[3][3] = \
      {{0.0, 0.0, 0.0},
      {2.449489742783182, 0.0, 0.0},
      {4.242640687119285, 0.0, 0.0}};
      
      // Compute reference derivatives.
      // Declare array of derivatives on FIAT element.
      double derivatives[2];
      for (unsigned int r = 0; r < 2; r++)
      {
        derivatives[r] = 0.0;
      } // end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[3][3] = \
      {{1.0, 0.0, 0.0},
      {0.0, 1.0, 0.0},
      {0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[3][3] = \
      {{1.0, 0.0, 0.0},
      {0.0, 1.0, 0.0},
      {0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 3; t++)
        {
          for (unsigned int u = 0; u < 3; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          } // end loop over 'u'
        } // end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            } // end loop over 'u'
          } // end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              for (unsigned int tu = 0; tu < 3; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              } // end loop over 'tu'
            } // end loop over 'u'
          } // end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              for (unsigned int tu = 0; tu < 3; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              } // end loop over 'tu'
            } // end loop over 'u'
          } // end loop over 't'
          }
          
        } // end loop over 's'
        for (unsigned int s = 0; s < 3; s++)
        {
          for (unsigned int t = 0; t < 3; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          } // end loop over 't'
        } // end loop over 's'
      } // end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[num_derivatives + r] += transform[r][s]*derivatives[s];
        } // end loop over 's'
      } // end loop over 'r'
        break;
      }
    case 5:
      {
        
      // Array of basisvalues
      double basisvalues[3] = {0.0, 0.0, 0.0};
      
      // Declare helper variables
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      
      // Compute basisvalues
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[1] *= std::sqrt(3.0);
      
      // Table(s) of coefficients
      static const double coefficients0[3] = \
      {0.4714045207910316, 0.0, 0.3333333333333333};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[3][3] = \
      {{0.0, 0.0, 0.0},
      {4.89897948556635, 0.0, 0.0},
      {0.0, 0.0, 0.0}};
      
      static const double dmats1[3][3] = \
      {{0.0, 0.0, 0.0},
      {2.449489742783182, 0.0, 0.0},
      {4.242640687119285, 0.0, 0.0}};
      
      // Compute reference derivatives.
      // Declare array of derivatives on FIAT element.
      double derivatives[2];
      for (unsigned int r = 0; r < 2; r++)
      {
        derivatives[r] = 0.0;
      } // end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[3][3] = \
      {{1.0, 0.0, 0.0},
      {0.0, 1.0, 0.0},
      {0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[3][3] = \
      {{1.0, 0.0, 0.0},
      {0.0, 1.0, 0.0},
      {0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 3; t++)
        {
          for (unsigned int u = 0; u < 3; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          } // end loop over 'u'
        } // end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            } // end loop over 'u'
          } // end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              for (unsigned int tu = 0; tu < 3; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              } // end loop over 'tu'
            } // end loop over 'u'
          } // end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              for (unsigned int tu = 0; tu < 3; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              } // end loop over 'tu'
            } // end loop over 'u'
          } // end loop over 't'
          }
          
        } // end loop over 's'
        for (unsigned int s = 0; s < 3; s++)
        {
          for (unsigned int t = 0; t < 3; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          } // end loop over 't'
        } // end loop over 's'
      } // end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[num_derivatives + r] += transform[r][s]*derivatives[s];
        } // end loop over 's'
      } // end loop over 'r'
        break;
      }
    }
    
  }

  void evaluate_basis_derivatives(std::size_t i,
                                  std::size_t n,
                                  double * values,
                                  const double * x,
                                  const double * coordinate_dofs,
                                  int cell_orientation) const final override
  {
    _evaluate_basis_derivatives(i, n, values, x, coordinate_dofs, cell_orientation);
  }

  static void _evaluate_basis_derivatives_all(std::size_t n,
                                              double * values,
                                              const double * x,
                                              const double * coordinate_dofs,
                                              int cell_orientation)
  {
    // Call evaluate_basis_all if order of derivatives is equal to zero.
    if (n == 0)
    {
      _evaluate_basis_all(values, x, coordinate_dofs, cell_orientation);
      return ;
    }
    
    // Compute number of derivatives.
    unsigned int num_derivatives = 1;
    for (unsigned int r = 0; r < n; r++)
    {
      num_derivatives *= 2;
    } // end loop over 'r'
    
    // Set values equal to zero.
    for (unsigned int r = 0; r < 6; r++)
    {
      for (unsigned int s = 0; s < 2*num_derivatives; s++)
      {
        values[r*2*num_derivatives + s] = 0.0;
      } // end loop over 's'
    } // end loop over 'r'
    
    // If order of derivatives is greater than the maximum polynomial degree, return zeros.
    if (n > 1)
    {
      return ;
    }
    
    // Helper variable to hold values of a single dof.
    double dof_values[4];
    for (unsigned int r = 0; r < 4; r++)
    {
      dof_values[r] = 0.0;
    } // end loop over 'r'
    
    // Loop dofs and call evaluate_basis_derivatives.
    for (unsigned int r = 0; r < 6; r++)
    {
      _evaluate_basis_derivatives(r, n, dof_values, x, coordinate_dofs, cell_orientation);
      for (unsigned int s = 0; s < 2*num_derivatives; s++)
      {
        values[r*2*num_derivatives + s] = dof_values[s];
      } // end loop over 's'
    } // end loop over 'r'
  }

  void evaluate_basis_derivatives_all(std::size_t n,
                                      double * values,
                                      const double * x,
                                      const double * coordinate_dofs,
                                      int cell_orientation) const final override
  {
    _evaluate_basis_derivatives_all(n, values, x, coordinate_dofs, cell_orientation);
  }

  double evaluate_dof(std::size_t i,
                      const ufc::function& f,
                      const double * coordinate_dofs,
                      int cell_orientation,
                      const ufc::cell& c) const final override
  {
    // Declare variables for result of evaluation
    double vals[2];
    
    // Declare variable for physical coordinates
    double y[2];
    switch (i)
    {
    case 0:
      {
        y[0] = coordinate_dofs[0];
      y[1] = coordinate_dofs[1];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 1:
      {
        y[0] = coordinate_dofs[2];
      y[1] = coordinate_dofs[3];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 2:
      {
        y[0] = coordinate_dofs[4];
      y[1] = coordinate_dofs[5];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 3:
      {
        y[0] = coordinate_dofs[0];
      y[1] = coordinate_dofs[1];
      f.evaluate(vals, y, c);
      return vals[1];
        break;
      }
    case 4:
      {
        y[0] = coordinate_dofs[2];
      y[1] = coordinate_dofs[3];
      f.evaluate(vals, y, c);
      return vals[1];
        break;
      }
    case 5:
      {
        y[0] = coordinate_dofs[4];
      y[1] = coordinate_dofs[5];
      f.evaluate(vals, y, c);
      return vals[1];
        break;
      }
    }
    
    return 0.0;
  }

  void evaluate_dofs(double * values,
                             const ufc::function& f,
                             const double * coordinate_dofs,
                             int cell_orientation,
                             const ufc::cell& c) const final override
  {
    // Declare variables for result of evaluation
    double vals[2];
    
    // Declare variable for physical coordinates
    double y[2];
    y[0] = coordinate_dofs[0];
    y[1] = coordinate_dofs[1];
    f.evaluate(vals, y, c);
    values[0] = vals[0];
    values[3] = vals[1];
    y[0] = coordinate_dofs[2];
    y[1] = coordinate_dofs[3];
    f.evaluate(vals, y, c);
    values[1] = vals[0];
    values[4] = vals[1];
    y[0] = coordinate_dofs[4];
    y[1] = coordinate_dofs[5];
    f.evaluate(vals, y, c);
    values[2] = vals[0];
    values[5] = vals[1];
  }

  void interpolate_vertex_values(double * vertex_values,
                                 const double * dof_values,
                                 const double * coordinate_dofs,
                                 int cell_orientation,
                                 const ufc::cell& c) const final override
  {
    // Evaluate function and change variables
    vertex_values[0] = dof_values[0];
    vertex_values[2] = dof_values[1];
    vertex_values[4] = dof_values[2];
    // Evaluate function and change variables
    vertex_values[1] = dof_values[3];
    vertex_values[3] = dof_values[4];
    vertex_values[5] = dof_values[5];
  }

  void tabulate_dof_coordinates(double * dof_coordinates,
                                const double * coordinate_dofs) const final override
  {
    dof_coordinates[0] = coordinate_dofs[0];
    dof_coordinates[1] = coordinate_dofs[1];
    dof_coordinates[2] = coordinate_dofs[2];
    dof_coordinates[3] = coordinate_dofs[3];
    dof_coordinates[4] = coordinate_dofs[4];
    dof_coordinates[5] = coordinate_dofs[5];
    dof_coordinates[6] = coordinate_dofs[0];
    dof_coordinates[7] = coordinate_dofs[1];
    dof_coordinates[8] = coordinate_dofs[2];
    dof_coordinates[9] = coordinate_dofs[3];
    dof_coordinates[10] = coordinate_dofs[4];
    dof_coordinates[11] = coordinate_dofs[5];
  }

  std::size_t num_sub_elements() const final override
  {
    return 2;
  }

  ufc::finite_element * create_sub_element(std::size_t i) const final override
  {
    switch (i)
    {
    case 0:
      {
        return new biharmonic_finite_element_0();
        break;
      }
    case 1:
      {
        return new biharmonic_finite_element_0();
        break;
      }
    }
    
    return 0;
  }

  ufc::finite_element * create() const final override
  {
    return new biharmonic_finite_element_1();
  }

};


class biharmonic_finite_element_2: public ufc::finite_element
{
public:

  biharmonic_finite_element_2() : ufc::finite_element()
  {
    // Do nothing
  }

  ~biharmonic_finite_element_2() override
  {
    // Do nothing
  }

  const char * signature() const final override
  {
    return "FiniteElement('Real', triangle, 0)";
  }

  ufc::shape cell_shape() const final override
  {
    return ufc::shape::triangle;
  }

  std::size_t topological_dimension() const final override
  {
    return 2;
  }

  std::size_t geometric_dimension() const final override
  {
    return 2;
  }

  std::size_t space_dimension() const final override
  {
    return 1;
  }

  std::size_t value_rank() const final override
  {
    return 0;
  }

  std::size_t value_dimension(std::size_t i) const final override
  {
    return 1;
  }

  std::size_t value_size() const final override
  {
    return 1;
  }

  std::size_t reference_value_rank() const final override
  {
    return 0;
  }

  std::size_t reference_value_dimension(std::size_t i) const final override
  {
    return 1;
  }

  std::size_t reference_value_size() const final override
  {
    return 1;
  }

  std::size_t degree() const final override
  {
    return 0;
  }

  const char * family() const final override
  {
    return "Real";
  }

  static void _evaluate_basis(std::size_t i,
                              double * values,
                              const double * x,
                              const double * coordinate_dofs,
                              int cell_orientation)
  {
    // Compute Jacobian
    double J[4];
    compute_jacobian_triangle_2d(J, coordinate_dofs);
    
    // Compute Jacobian inverse and determinant
    double K[4];
    double detJ;
    compute_jacobian_inverse_triangle_2d(K, detJ, J);
    
    
    // Compute constants
    
    // Get coordinates and map to the reference (FIAT) element
    
    // Reset values
    *values = 0.0;
    
    // Array of basisvalues
    double basisvalues[1] = {0.0};
    
    // Declare helper variables
    
    // Compute basisvalues
    basisvalues[0] = 1.0;
    
    // Table(s) of coefficients
    static const double coefficients0[1] = \
    {1.0};
    
    // Compute value(s)
    for (unsigned int r = 0; r < 1; r++)
    {
      *values += coefficients0[r]*basisvalues[r];
    } // end loop over 'r'
  }

  void evaluate_basis(std::size_t i,
                      double * values,
                      const double * x,
                      const double * coordinate_dofs,
                      int cell_orientation) const final override
  {
    _evaluate_basis(i, values, x, coordinate_dofs, cell_orientation);
  }

  static void _evaluate_basis_all(double * values,
                                  const double * x,
                                  const double * coordinate_dofs,
                                  int cell_orientation)
  {
    // Element is constant, calling evaluate_basis.
    _evaluate_basis(0, values, x, coordinate_dofs, cell_orientation);
  }

  void evaluate_basis_all(double * values,
                          const double * x,
                          const double * coordinate_dofs,
                          int cell_orientation) const final override
  {
    _evaluate_basis_all(values, x, coordinate_dofs, cell_orientation);
  }

  static void _evaluate_basis_derivatives(std::size_t i,
                                          std::size_t n,
                                          double * values,
                                          const double * x,
                                          const double * coordinate_dofs,
                                          int cell_orientation)
  {
    
    // Compute number of derivatives.
    unsigned int num_derivatives = 1;
    for (unsigned int r = 0; r < n; r++)
    {
      num_derivatives *= 2;
    } // end loop over 'r'
    
    // Reset values. Assuming that values is always an array.
    for (unsigned int r = 0; r < num_derivatives; r++)
    {
      values[r] = 0.0;
    } // end loop over 'r'
    
    // Call evaluate_basis if order of derivatives is equal to zero.
    if (n == 0)
    {
      _evaluate_basis(i, values, x, coordinate_dofs, cell_orientation);
      return ;
    }
    
    // If order of derivatives is greater than the maximum polynomial degree, return zeros.
    if (n > 0)
    {
    return ;
    }
    
  }

  void evaluate_basis_derivatives(std::size_t i,
                                  std::size_t n,
                                  double * values,
                                  const double * x,
                                  const double * coordinate_dofs,
                                  int cell_orientation) const final override
  {
    _evaluate_basis_derivatives(i, n, values, x, coordinate_dofs, cell_orientation);
  }

  static void _evaluate_basis_derivatives_all(std::size_t n,
                                              double * values,
                                              const double * x,
                                              const double * coordinate_dofs,
                                              int cell_orientation)
  {
    // Element is constant, calling evaluate_basis_derivatives.
    _evaluate_basis_derivatives(0, n, values, x, coordinate_dofs, cell_orientation);
  }

  void evaluate_basis_derivatives_all(std::size_t n,
                                      double * values,
                                      const double * x,
                                      const double * coordinate_dofs,
                                      int cell_orientation) const final override
  {
    _evaluate_basis_derivatives_all(n, values, x, coordinate_dofs, cell_orientation);
  }

  double evaluate_dof(std::size_t i,
                      const ufc::function& f,
                      const double * coordinate_dofs,
                      int cell_orientation,
                      const ufc::cell& c) const final override
  {
    // Declare variables for result of evaluation
    double vals[1];
    
    // Declare variable for physical coordinates
    double y[2];
    switch (i)
    {
    case 0:
      {
        y[0] = 0.3333333333333334*coordinate_dofs[0] + 0.3333333333333333*coordinate_dofs[2] + 0.3333333333333333*coordinate_dofs[4];
      y[1] = 0.3333333333333334*coordinate_dofs[1] + 0.3333333333333333*coordinate_dofs[3] + 0.3333333333333333*coordinate_dofs[5];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    }
    
    return 0.0;
  }

  void evaluate_dofs(double * values,
                             const ufc::function& f,
                             const double * coordinate_dofs,
                             int cell_orientation,
                             const ufc::cell& c) const final override
  {
    // Declare variables for result of evaluation
    double vals[1];
    
    // Declare variable for physical coordinates
    double y[2];
    y[0] = 0.3333333333333334*coordinate_dofs[0] + 0.3333333333333333*coordinate_dofs[2] + 0.3333333333333333*coordinate_dofs[4];
    y[1] = 0.3333333333333334*coordinate_dofs[1] + 0.3333333333333333*coordinate_dofs[3] + 0.3333333333333333*coordinate_dofs[5];
    f.evaluate(vals, y, c);
    values[0] = vals[0];
  }

  void interpolate_vertex_values(double * vertex_values,
                                 const double * dof_values,
                                 const double * coordinate_dofs,
                                 int cell_orientation,
                                 const ufc::cell& c) const final override
  {
    // Evaluate function and change variables
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
  }

  void tabulate_dof_coordinates(double * dof_coordinates,
                                const double * coordinate_dofs) const final override
  {
    dof_coordinates[0] = 0.3333333333333334*coordinate_dofs[0] + 0.3333333333333333*coordinate_dofs[2] + 0.3333333333333333*coordinate_dofs[4];
    dof_coordinates[1] = 0.3333333333333334*coordinate_dofs[1] + 0.3333333333333333*coordinate_dofs[3] + 0.3333333333333333*coordinate_dofs[5];
  }

  std::size_t num_sub_elements() const final override
  {
    return 0;
  }

  ufc::finite_element * create_sub_element(std::size_t i) const final override
  {
    return 0;
  }

  ufc::finite_element * create() const final override
  {
    return new biharmonic_finite_element_2();
  }

};


class biharmonic_finite_element_3: public ufc::finite_element
{
public:

  biharmonic_finite_element_3() : ufc::finite_element()
  {
    // Do nothing
  }

  ~biharmonic_finite_element_3() override
  {
    // Do nothing
  }

  const char * signature() const final override
  {
    return "FiniteElement('Lagrange', triangle, 2)";
  }

  ufc::shape cell_shape() const final override
  {
    return ufc::shape::triangle;
  }

  std::size_t topological_dimension() const final override
  {
    return 2;
  }

  std::size_t geometric_dimension() const final override
  {
    return 2;
  }

  std::size_t space_dimension() const final override
  {
    return 6;
  }

  std::size_t value_rank() const final override
  {
    return 0;
  }

  std::size_t value_dimension(std::size_t i) const final override
  {
    return 1;
  }

  std::size_t value_size() const final override
  {
    return 1;
  }

  std::size_t reference_value_rank() const final override
  {
    return 0;
  }

  std::size_t reference_value_dimension(std::size_t i) const final override
  {
    return 1;
  }

  std::size_t reference_value_size() const final override
  {
    return 1;
  }

  std::size_t degree() const final override
  {
    return 2;
  }

  const char * family() const final override
  {
    return "Lagrange";
  }

  static void _evaluate_basis(std::size_t i,
                              double * values,
                              const double * x,
                              const double * coordinate_dofs,
                              int cell_orientation)
  {
    // Compute Jacobian
    double J[4];
    compute_jacobian_triangle_2d(J, coordinate_dofs);
    
    // Compute Jacobian inverse and determinant
    double K[4];
    double detJ;
    compute_jacobian_inverse_triangle_2d(K, detJ, J);
    
    
    // Compute constants
    const double C0 = coordinate_dofs[2] + coordinate_dofs[4];
    const double C1 = coordinate_dofs[3] + coordinate_dofs[5];
    
    // Get coordinates and map to the reference (FIAT) element
    double X = (J[1]*(C1 - 2.0*x[1]) + J[3]*(2.0*x[0] - C0)) / detJ;
    double Y = (J[0]*(2.0*x[1] - C1) + J[2]*(C0 - 2.0*x[0])) / detJ;
    
    // Reset values
    *values = 0.0;
    switch (i)
    {
    case 0:
      {
        
      // Array of basisvalues
      double basisvalues[6] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      double tmp1 = (1.0 - Y)/2.0;
      double tmp2 = tmp1*tmp1;
      
      // Compute basisvalues
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[3] = basisvalues[1]*1.5*tmp0 - basisvalues[0]*0.5*tmp2;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[4] = basisvalues[1]*(1.5 + 2.5*Y);
      basisvalues[5] = basisvalues[2]*(0.1111111111111111 + Y*1.666666666666667) - basisvalues[0]*0.5555555555555556;
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[5] *= std::sqrt(1.5);
      basisvalues[1] *= std::sqrt(3.0);
      basisvalues[4] *= std::sqrt(4.5);
      basisvalues[3] *= std::sqrt(7.5);
      
      // Table(s) of coefficients
      static const double coefficients0[6] = \
      {0.0, -0.1732050807568878, -0.1, 0.1217161238900369, 0.09428090415820634, 0.05443310539518172};
      
      // Compute value(s)
      for (unsigned int r = 0; r < 6; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      } // end loop over 'r'
        break;
      }
    case 1:
      {
        
      // Array of basisvalues
      double basisvalues[6] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      double tmp1 = (1.0 - Y)/2.0;
      double tmp2 = tmp1*tmp1;
      
      // Compute basisvalues
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[3] = basisvalues[1]*1.5*tmp0 - basisvalues[0]*0.5*tmp2;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[4] = basisvalues[1]*(1.5 + 2.5*Y);
      basisvalues[5] = basisvalues[2]*(0.1111111111111111 + Y*1.666666666666667) - basisvalues[0]*0.5555555555555556;
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[5] *= std::sqrt(1.5);
      basisvalues[1] *= std::sqrt(3.0);
      basisvalues[4] *= std::sqrt(4.5);
      basisvalues[3] *= std::sqrt(7.5);
      
      // Table(s) of coefficients
      static const double coefficients0[6] = \
      {0.0, 0.1732050807568877, -0.1, 0.1217161238900369, -0.09428090415820636, 0.05443310539518175};
      
      // Compute value(s)
      for (unsigned int r = 0; r < 6; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      } // end loop over 'r'
        break;
      }
    case 2:
      {
        
      // Array of basisvalues
      double basisvalues[6] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      double tmp1 = (1.0 - Y)/2.0;
      double tmp2 = tmp1*tmp1;
      
      // Compute basisvalues
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[3] = basisvalues[1]*1.5*tmp0 - basisvalues[0]*0.5*tmp2;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[4] = basisvalues[1]*(1.5 + 2.5*Y);
      basisvalues[5] = basisvalues[2]*(0.1111111111111111 + Y*1.666666666666667) - basisvalues[0]*0.5555555555555556;
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[5] *= std::sqrt(1.5);
      basisvalues[1] *= std::sqrt(3.0);
      basisvalues[4] *= std::sqrt(4.5);
      basisvalues[3] *= std::sqrt(7.5);
      
      // Table(s) of coefficients
      static const double coefficients0[6] = \
      {0.0, 0.0, 0.2, 0.0, 0.0, 0.1632993161855452};
      
      // Compute value(s)
      for (unsigned int r = 0; r < 6; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      } // end loop over 'r'
        break;
      }
    case 3:
      {
        
      // Array of basisvalues
      double basisvalues[6] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      double tmp1 = (1.0 - Y)/2.0;
      double tmp2 = tmp1*tmp1;
      
      // Compute basisvalues
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[3] = basisvalues[1]*1.5*tmp0 - basisvalues[0]*0.5*tmp2;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[4] = basisvalues[1]*(1.5 + 2.5*Y);
      basisvalues[5] = basisvalues[2]*(0.1111111111111111 + Y*1.666666666666667) - basisvalues[0]*0.5555555555555556;
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[5] *= std::sqrt(1.5);
      basisvalues[1] *= std::sqrt(3.0);
      basisvalues[4] *= std::sqrt(4.5);
      basisvalues[3] *= std::sqrt(7.5);
      
      // Table(s) of coefficients
      static const double coefficients0[6] = \
      {0.4714045207910316, 0.2309401076758503, 0.1333333333333333, 0.0, 0.1885618083164126, -0.1632993161855452};
      
      // Compute value(s)
      for (unsigned int r = 0; r < 6; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      } // end loop over 'r'
        break;
      }
    case 4:
      {
        
      // Array of basisvalues
      double basisvalues[6] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      double tmp1 = (1.0 - Y)/2.0;
      double tmp2 = tmp1*tmp1;
      
      // Compute basisvalues
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[3] = basisvalues[1]*1.5*tmp0 - basisvalues[0]*0.5*tmp2;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[4] = basisvalues[1]*(1.5 + 2.5*Y);
      basisvalues[5] = basisvalues[2]*(0.1111111111111111 + Y*1.666666666666667) - basisvalues[0]*0.5555555555555556;
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[5] *= std::sqrt(1.5);
      basisvalues[1] *= std::sqrt(3.0);
      basisvalues[4] *= std::sqrt(4.5);
      basisvalues[3] *= std::sqrt(7.5);
      
      // Table(s) of coefficients
      static const double coefficients0[6] = \
      {0.4714045207910316, -0.2309401076758503, 0.1333333333333333, 0.0, -0.1885618083164126, -0.1632993161855452};
      
      // Compute value(s)
      for (unsigned int r = 0; r < 6; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      } // end loop over 'r'
        break;
      }
    case 5:
      {
        
      // Array of basisvalues
      double basisvalues[6] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      double tmp1 = (1.0 - Y)/2.0;
      double tmp2 = tmp1*tmp1;
      
      // Compute basisvalues
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[3] = basisvalues[1]*1.5*tmp0 - basisvalues[0]*0.5*tmp2;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[4] = basisvalues[1]*(1.5 + 2.5*Y);
      basisvalues[5] = basisvalues[2]*(0.1111111111111111 + Y*1.666666666666667) - basisvalues[0]*0.5555555555555556;
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[5] *= std::sqrt(1.5);
      basisvalues[1] *= std::sqrt(3.0);
      basisvalues[4] *= std::sqrt(4.5);
      basisvalues[3] *= std::sqrt(7.5);
      
      // Table(s) of coefficients
      static const double coefficients0[6] = \
      {0.4714045207910316, 0.0, -0.2666666666666667, -0.2434322477800738, 0.0, 0.05443310539518172};
      
      // Compute value(s)
      for (unsigned int r = 0; r < 6; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      } // end loop over 'r'
        break;
      }
    }
    
  }

  void evaluate_basis(std::size_t i,
                      double * values,
                      const double * x,
                      const double * coordinate_dofs,
                      int cell_orientation) const final override
  {
    _evaluate_basis(i, values, x, coordinate_dofs, cell_orientation);
  }

  static void _evaluate_basis_all(double * values,
                                  const double * x,
                                  const double * coordinate_dofs,
                                  int cell_orientation)
  {
    // Helper variable to hold values of a single dof.
    double dof_values = 0.0;
    
    // Loop dofs and call evaluate_basis
    for (unsigned int r = 0; r < 6; r++)
    {
      _evaluate_basis(r, &dof_values, x, coordinate_dofs, cell_orientation);
      values[r] = dof_values;
    } // end loop over 'r'
  }

  void evaluate_basis_all(double * values,
                          const double * x,
                          const double * coordinate_dofs,
                          int cell_orientation) const final override
  {
    _evaluate_basis_all(values, x, coordinate_dofs, cell_orientation);
  }

  static void _evaluate_basis_derivatives(std::size_t i,
                                          std::size_t n,
                                          double * values,
                                          const double * x,
                                          const double * coordinate_dofs,
                                          int cell_orientation)
  {
    
    // Compute number of derivatives.
    unsigned int num_derivatives = 1;
    for (unsigned int r = 0; r < n; r++)
    {
      num_derivatives *= 2;
    } // end loop over 'r'
    
    // Reset values. Assuming that values is always an array.
    for (unsigned int r = 0; r < num_derivatives; r++)
    {
      values[r] = 0.0;
    } // end loop over 'r'
    
    // Call evaluate_basis if order of derivatives is equal to zero.
    if (n == 0)
    {
      _evaluate_basis(i, values, x, coordinate_dofs, cell_orientation);
      return ;
    }
    
    // If order of derivatives is greater than the maximum polynomial degree, return zeros.
    if (n > 2)
    {
    return ;
    }
    
    // Compute Jacobian
    double J[4];
    compute_jacobian_triangle_2d(J, coordinate_dofs);
    
    // Compute Jacobian inverse and determinant
    double K[4];
    double detJ;
    compute_jacobian_inverse_triangle_2d(K, detJ, J);
    
    
    // Compute constants
    const double C0 = coordinate_dofs[2] + coordinate_dofs[4];
    const double C1 = coordinate_dofs[3] + coordinate_dofs[5];
    
    // Get coordinates and map to the reference (FIAT) element
    double X = (J[1]*(C1 - 2.0*x[1]) + J[3]*(2.0*x[0] - C0)) / detJ;
    double Y = (J[0]*(2.0*x[1] - C1) + J[2]*(C0 - 2.0*x[0])) / detJ;
    
    // Declare two dimensional array that holds combinations of derivatives and initialise
    unsigned int combinations[4][2];
    for (unsigned int row = 0; row < 4; row++)
    {
      for (unsigned int col = 0; col < 2; col++)
        combinations[row][col] = 0;
    }
    
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 1)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[2][2] = {{K[0], K[1]}, {K[2], K[3]}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double transform[4][4];
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    switch (i)
    {
    case 0:
      {
        
      // Array of basisvalues
      double basisvalues[6] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      double tmp1 = (1.0 - Y)/2.0;
      double tmp2 = tmp1*tmp1;
      
      // Compute basisvalues
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[3] = basisvalues[1]*1.5*tmp0 - basisvalues[0]*0.5*tmp2;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[4] = basisvalues[1]*(1.5 + 2.5*Y);
      basisvalues[5] = basisvalues[2]*(0.1111111111111111 + Y*1.666666666666667) - basisvalues[0]*0.5555555555555556;
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[5] *= std::sqrt(1.5);
      basisvalues[1] *= std::sqrt(3.0);
      basisvalues[4] *= std::sqrt(4.5);
      basisvalues[3] *= std::sqrt(7.5);
      
      // Table(s) of coefficients
      static const double coefficients0[6] = \
      {0.0, -0.1732050807568878, -0.1, 0.1217161238900369, 0.09428090415820634, 0.05443310539518172};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[6][6] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.898979485566347, 0.0, 0.0, 0.0, 0.0, -1.332267629550188e-15},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 9.486832980505149, 0.0, -1.77635683940025e-15, 2.220446049250313e-15, 0.0},
      {3.999999999999994, 0.0, 7.071067811865468, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[6][6] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.449489742783181, 0.0, 0.0, 0.0, 0.0, -1.110223024625157e-15},
      {4.242640687119282, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.581988897471618, 4.743416490252574, -0.91287092917528, 0.0, 1.332267629550188e-15, -1.77635683940025e-15},
      {1.999999999999915, 6.123724356957939, 3.535533905932734, 0.0, 0.0, 0.0},
      {-2.309401076758489, 0.0, 8.164965809277266, 0.0, 0.0, 0.0}};
      
      // Compute reference derivatives.
      // Declare array of derivatives on FIAT element.
      double derivatives[4];
      for (unsigned int r = 0; r < 4; r++)
      {
        derivatives[r] = 0.0;
      } // end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[6][6] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[6][6] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 6; t++)
        {
          for (unsigned int u = 0; u < 6; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          } // end loop over 'u'
        } // end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 6; t++)
          {
            for (unsigned int u = 0; u < 6; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            } // end loop over 'u'
          } // end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 6; t++)
          {
            for (unsigned int u = 0; u < 6; u++)
            {
              for (unsigned int tu = 0; tu < 6; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              } // end loop over 'tu'
            } // end loop over 'u'
          } // end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 6; t++)
          {
            for (unsigned int u = 0; u < 6; u++)
            {
              for (unsigned int tu = 0; tu < 6; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              } // end loop over 'tu'
            } // end loop over 'u'
          } // end loop over 't'
          }
          
        } // end loop over 's'
        for (unsigned int s = 0; s < 6; s++)
        {
          for (unsigned int t = 0; t < 6; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          } // end loop over 't'
        } // end loop over 's'
      } // end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        } // end loop over 's'
      } // end loop over 'r'
        break;
      }
    case 1:
      {
        
      // Array of basisvalues
      double basisvalues[6] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      double tmp1 = (1.0 - Y)/2.0;
      double tmp2 = tmp1*tmp1;
      
      // Compute basisvalues
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[3] = basisvalues[1]*1.5*tmp0 - basisvalues[0]*0.5*tmp2;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[4] = basisvalues[1]*(1.5 + 2.5*Y);
      basisvalues[5] = basisvalues[2]*(0.1111111111111111 + Y*1.666666666666667) - basisvalues[0]*0.5555555555555556;
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[5] *= std::sqrt(1.5);
      basisvalues[1] *= std::sqrt(3.0);
      basisvalues[4] *= std::sqrt(4.5);
      basisvalues[3] *= std::sqrt(7.5);
      
      // Table(s) of coefficients
      static const double coefficients0[6] = \
      {0.0, 0.1732050807568877, -0.1, 0.1217161238900369, -0.09428090415820636, 0.05443310539518175};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[6][6] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.898979485566347, 0.0, 0.0, 0.0, 0.0, -1.332267629550188e-15},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 9.486832980505149, 0.0, -1.77635683940025e-15, 2.220446049250313e-15, 0.0},
      {3.999999999999994, 0.0, 7.071067811865468, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[6][6] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.449489742783181, 0.0, 0.0, 0.0, 0.0, -1.110223024625157e-15},
      {4.242640687119282, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.581988897471618, 4.743416490252574, -0.91287092917528, 0.0, 1.332267629550188e-15, -1.77635683940025e-15},
      {1.999999999999915, 6.123724356957939, 3.535533905932734, 0.0, 0.0, 0.0},
      {-2.309401076758489, 0.0, 8.164965809277266, 0.0, 0.0, 0.0}};
      
      // Compute reference derivatives.
      // Declare array of derivatives on FIAT element.
      double derivatives[4];
      for (unsigned int r = 0; r < 4; r++)
      {
        derivatives[r] = 0.0;
      } // end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[6][6] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[6][6] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 6; t++)
        {
          for (unsigned int u = 0; u < 6; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          } // end loop over 'u'
        } // end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 6; t++)
          {
            for (unsigned int u = 0; u < 6; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            } // end loop over 'u'
          } // end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 6; t++)
          {
            for (unsigned int u = 0; u < 6; u++)
            {
              for (unsigned int tu = 0; tu < 6; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              } // end loop over 'tu'
            } // end loop over 'u'
          } // end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 6; t++)
          {
            for (unsigned int u = 0; u < 6; u++)
            {
              for (unsigned int tu = 0; tu < 6; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              } // end loop over 'tu'
            } // end loop over 'u'
          } // end loop over 't'
          }
          
        } // end loop over 's'
        for (unsigned int s = 0; s < 6; s++)
        {
          for (unsigned int t = 0; t < 6; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          } // end loop over 't'
        } // end loop over 's'
      } // end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        } // end loop over 's'
      } // end loop over 'r'
        break;
      }
    case 2:
      {
        
      // Array of basisvalues
      double basisvalues[6] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      double tmp1 = (1.0 - Y)/2.0;
      double tmp2 = tmp1*tmp1;
      
      // Compute basisvalues
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[3] = basisvalues[1]*1.5*tmp0 - basisvalues[0]*0.5*tmp2;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[4] = basisvalues[1]*(1.5 + 2.5*Y);
      basisvalues[5] = basisvalues[2]*(0.1111111111111111 + Y*1.666666666666667) - basisvalues[0]*0.5555555555555556;
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[5] *= std::sqrt(1.5);
      basisvalues[1] *= std::sqrt(3.0);
      basisvalues[4] *= std::sqrt(4.5);
      basisvalues[3] *= std::sqrt(7.5);
      
      // Table(s) of coefficients
      static const double coefficients0[6] = \
      {0.0, 0.0, 0.2, 0.0, 0.0, 0.1632993161855452};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[6][6] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.898979485566347, 0.0, 0.0, 0.0, 0.0, -1.332267629550188e-15},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 9.486832980505149, 0.0, -1.77635683940025e-15, 2.220446049250313e-15, 0.0},
      {3.999999999999994, 0.0, 7.071067811865468, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[6][6] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.449489742783181, 0.0, 0.0, 0.0, 0.0, -1.110223024625157e-15},
      {4.242640687119282, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.581988897471618, 4.743416490252574, -0.91287092917528, 0.0, 1.332267629550188e-15, -1.77635683940025e-15},
      {1.999999999999915, 6.123724356957939, 3.535533905932734, 0.0, 0.0, 0.0},
      {-2.309401076758489, 0.0, 8.164965809277266, 0.0, 0.0, 0.0}};
      
      // Compute reference derivatives.
      // Declare array of derivatives on FIAT element.
      double derivatives[4];
      for (unsigned int r = 0; r < 4; r++)
      {
        derivatives[r] = 0.0;
      } // end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[6][6] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[6][6] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 6; t++)
        {
          for (unsigned int u = 0; u < 6; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          } // end loop over 'u'
        } // end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 6; t++)
          {
            for (unsigned int u = 0; u < 6; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            } // end loop over 'u'
          } // end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 6; t++)
          {
            for (unsigned int u = 0; u < 6; u++)
            {
              for (unsigned int tu = 0; tu < 6; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              } // end loop over 'tu'
            } // end loop over 'u'
          } // end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 6; t++)
          {
            for (unsigned int u = 0; u < 6; u++)
            {
              for (unsigned int tu = 0; tu < 6; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              } // end loop over 'tu'
            } // end loop over 'u'
          } // end loop over 't'
          }
          
        } // end loop over 's'
        for (unsigned int s = 0; s < 6; s++)
        {
          for (unsigned int t = 0; t < 6; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          } // end loop over 't'
        } // end loop over 's'
      } // end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        } // end loop over 's'
      } // end loop over 'r'
        break;
      }
    case 3:
      {
        
      // Array of basisvalues
      double basisvalues[6] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      double tmp1 = (1.0 - Y)/2.0;
      double tmp2 = tmp1*tmp1;
      
      // Compute basisvalues
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[3] = basisvalues[1]*1.5*tmp0 - basisvalues[0]*0.5*tmp2;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[4] = basisvalues[1]*(1.5 + 2.5*Y);
      basisvalues[5] = basisvalues[2]*(0.1111111111111111 + Y*1.666666666666667) - basisvalues[0]*0.5555555555555556;
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[5] *= std::sqrt(1.5);
      basisvalues[1] *= std::sqrt(3.0);
      basisvalues[4] *= std::sqrt(4.5);
      basisvalues[3] *= std::sqrt(7.5);
      
      // Table(s) of coefficients
      static const double coefficients0[6] = \
      {0.4714045207910316, 0.2309401076758503, 0.1333333333333333, 0.0, 0.1885618083164126, -0.1632993161855452};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[6][6] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.898979485566347, 0.0, 0.0, 0.0, 0.0, -1.332267629550188e-15},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 9.486832980505149, 0.0, -1.77635683940025e-15, 2.220446049250313e-15, 0.0},
      {3.999999999999994, 0.0, 7.071067811865468, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[6][6] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.449489742783181, 0.0, 0.0, 0.0, 0.0, -1.110223024625157e-15},
      {4.242640687119282, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.581988897471618, 4.743416490252574, -0.91287092917528, 0.0, 1.332267629550188e-15, -1.77635683940025e-15},
      {1.999999999999915, 6.123724356957939, 3.535533905932734, 0.0, 0.0, 0.0},
      {-2.309401076758489, 0.0, 8.164965809277266, 0.0, 0.0, 0.0}};
      
      // Compute reference derivatives.
      // Declare array of derivatives on FIAT element.
      double derivatives[4];
      for (unsigned int r = 0; r < 4; r++)
      {
        derivatives[r] = 0.0;
      } // end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[6][6] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[6][6] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 6; t++)
        {
          for (unsigned int u = 0; u < 6; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          } // end loop over 'u'
        } // end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 6; t++)
          {
            for (unsigned int u = 0; u < 6; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            } // end loop over 'u'
          } // end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 6; t++)
          {
            for (unsigned int u = 0; u < 6; u++)
            {
              for (unsigned int tu = 0; tu < 6; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              } // end loop over 'tu'
            } // end loop over 'u'
          } // end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 6; t++)
          {
            for (unsigned int u = 0; u < 6; u++)
            {
              for (unsigned int tu = 0; tu < 6; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              } // end loop over 'tu'
            } // end loop over 'u'
          } // end loop over 't'
          }
          
        } // end loop over 's'
        for (unsigned int s = 0; s < 6; s++)
        {
          for (unsigned int t = 0; t < 6; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          } // end loop over 't'
        } // end loop over 's'
      } // end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        } // end loop over 's'
      } // end loop over 'r'
        break;
      }
    case 4:
      {
        
      // Array of basisvalues
      double basisvalues[6] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      double tmp1 = (1.0 - Y)/2.0;
      double tmp2 = tmp1*tmp1;
      
      // Compute basisvalues
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[3] = basisvalues[1]*1.5*tmp0 - basisvalues[0]*0.5*tmp2;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[4] = basisvalues[1]*(1.5 + 2.5*Y);
      basisvalues[5] = basisvalues[2]*(0.1111111111111111 + Y*1.666666666666667) - basisvalues[0]*0.5555555555555556;
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[5] *= std::sqrt(1.5);
      basisvalues[1] *= std::sqrt(3.0);
      basisvalues[4] *= std::sqrt(4.5);
      basisvalues[3] *= std::sqrt(7.5);
      
      // Table(s) of coefficients
      static const double coefficients0[6] = \
      {0.4714045207910316, -0.2309401076758503, 0.1333333333333333, 0.0, -0.1885618083164126, -0.1632993161855452};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[6][6] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.898979485566347, 0.0, 0.0, 0.0, 0.0, -1.332267629550188e-15},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 9.486832980505149, 0.0, -1.77635683940025e-15, 2.220446049250313e-15, 0.0},
      {3.999999999999994, 0.0, 7.071067811865468, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[6][6] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.449489742783181, 0.0, 0.0, 0.0, 0.0, -1.110223024625157e-15},
      {4.242640687119282, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.581988897471618, 4.743416490252574, -0.91287092917528, 0.0, 1.332267629550188e-15, -1.77635683940025e-15},
      {1.999999999999915, 6.123724356957939, 3.535533905932734, 0.0, 0.0, 0.0},
      {-2.309401076758489, 0.0, 8.164965809277266, 0.0, 0.0, 0.0}};
      
      // Compute reference derivatives.
      // Declare array of derivatives on FIAT element.
      double derivatives[4];
      for (unsigned int r = 0; r < 4; r++)
      {
        derivatives[r] = 0.0;
      } // end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[6][6] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[6][6] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 6; t++)
        {
          for (unsigned int u = 0; u < 6; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          } // end loop over 'u'
        } // end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 6; t++)
          {
            for (unsigned int u = 0; u < 6; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            } // end loop over 'u'
          } // end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 6; t++)
          {
            for (unsigned int u = 0; u < 6; u++)
            {
              for (unsigned int tu = 0; tu < 6; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              } // end loop over 'tu'
            } // end loop over 'u'
          } // end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 6; t++)
          {
            for (unsigned int u = 0; u < 6; u++)
            {
              for (unsigned int tu = 0; tu < 6; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              } // end loop over 'tu'
            } // end loop over 'u'
          } // end loop over 't'
          }
          
        } // end loop over 's'
        for (unsigned int s = 0; s < 6; s++)
        {
          for (unsigned int t = 0; t < 6; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          } // end loop over 't'
        } // end loop over 's'
      } // end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        } // end loop over 's'
      } // end loop over 'r'
        break;
      }
    case 5:
      {
        
      // Array of basisvalues
      double basisvalues[6] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      double tmp1 = (1.0 - Y)/2.0;
      double tmp2 = tmp1*tmp1;
      
      // Compute basisvalues
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[3] = basisvalues[1]*1.5*tmp0 - basisvalues[0]*0.5*tmp2;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[4] = basisvalues[1]*(1.5 + 2.5*Y);
      basisvalues[5] = basisvalues[2]*(0.1111111111111111 + Y*1.666666666666667) - basisvalues[0]*0.5555555555555556;
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[5] *= std::sqrt(1.5);
      basisvalues[1] *= std::sqrt(3.0);
      basisvalues[4] *= std::sqrt(4.5);
      basisvalues[3] *= std::sqrt(7.5);
      
      // Table(s) of coefficients
      static const double coefficients0[6] = \
      {0.4714045207910316, 0.0, -0.2666666666666667, -0.2434322477800738, 0.0, 0.05443310539518172};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[6][6] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.898979485566347, 0.0, 0.0, 0.0, 0.0, -1.332267629550188e-15},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 9.486832980505149, 0.0, -1.77635683940025e-15, 2.220446049250313e-15, 0.0},
      {3.999999999999994, 0.0, 7.071067811865468, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[6][6] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.449489742783181, 0.0, 0.0, 0.0, 0.0, -1.110223024625157e-15},
      {4.242640687119282, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.581988897471618, 4.743416490252574, -0.91287092917528, 0.0, 1.332267629550188e-15, -1.77635683940025e-15},
      {1.999999999999915, 6.123724356957939, 3.535533905932734, 0.0, 0.0, 0.0},
      {-2.309401076758489, 0.0, 8.164965809277266, 0.0, 0.0, 0.0}};
      
      // Compute reference derivatives.
      // Declare array of derivatives on FIAT element.
      double derivatives[4];
      for (unsigned int r = 0; r < 4; r++)
      {
        derivatives[r] = 0.0;
      } // end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[6][6] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[6][6] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 6; t++)
        {
          for (unsigned int u = 0; u < 6; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          } // end loop over 'u'
        } // end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 6; t++)
          {
            for (unsigned int u = 0; u < 6; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            } // end loop over 'u'
          } // end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 6; t++)
          {
            for (unsigned int u = 0; u < 6; u++)
            {
              for (unsigned int tu = 0; tu < 6; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              } // end loop over 'tu'
            } // end loop over 'u'
          } // end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 6; t++)
          {
            for (unsigned int u = 0; u < 6; u++)
            {
              for (unsigned int tu = 0; tu < 6; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              } // end loop over 'tu'
            } // end loop over 'u'
          } // end loop over 't'
          }
          
        } // end loop over 's'
        for (unsigned int s = 0; s < 6; s++)
        {
          for (unsigned int t = 0; t < 6; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          } // end loop over 't'
        } // end loop over 's'
      } // end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        } // end loop over 's'
      } // end loop over 'r'
        break;
      }
    }
    
  }

  void evaluate_basis_derivatives(std::size_t i,
                                  std::size_t n,
                                  double * values,
                                  const double * x,
                                  const double * coordinate_dofs,
                                  int cell_orientation) const final override
  {
    _evaluate_basis_derivatives(i, n, values, x, coordinate_dofs, cell_orientation);
  }

  static void _evaluate_basis_derivatives_all(std::size_t n,
                                              double * values,
                                              const double * x,
                                              const double * coordinate_dofs,
                                              int cell_orientation)
  {
    // Call evaluate_basis_all if order of derivatives is equal to zero.
    if (n == 0)
    {
      _evaluate_basis_all(values, x, coordinate_dofs, cell_orientation);
      return ;
    }
    
    // Compute number of derivatives.
    unsigned int num_derivatives = 1;
    for (unsigned int r = 0; r < n; r++)
    {
      num_derivatives *= 2;
    } // end loop over 'r'
    
    // Set values equal to zero.
    for (unsigned int r = 0; r < 6; r++)
    {
      for (unsigned int s = 0; s < num_derivatives; s++)
      {
        values[r*num_derivatives + s] = 0.0;
      } // end loop over 's'
    } // end loop over 'r'
    
    // If order of derivatives is greater than the maximum polynomial degree, return zeros.
    if (n > 2)
    {
      return ;
    }
    
    // Helper variable to hold values of a single dof.
    double dof_values[4];
    for (unsigned int r = 0; r < 4; r++)
    {
      dof_values[r] = 0.0;
    } // end loop over 'r'
    
    // Loop dofs and call evaluate_basis_derivatives.
    for (unsigned int r = 0; r < 6; r++)
    {
      _evaluate_basis_derivatives(r, n, dof_values, x, coordinate_dofs, cell_orientation);
      for (unsigned int s = 0; s < num_derivatives; s++)
      {
        values[r*num_derivatives + s] = dof_values[s];
      } // end loop over 's'
    } // end loop over 'r'
  }

  void evaluate_basis_derivatives_all(std::size_t n,
                                      double * values,
                                      const double * x,
                                      const double * coordinate_dofs,
                                      int cell_orientation) const final override
  {
    _evaluate_basis_derivatives_all(n, values, x, coordinate_dofs, cell_orientation);
  }

  double evaluate_dof(std::size_t i,
                      const ufc::function& f,
                      const double * coordinate_dofs,
                      int cell_orientation,
                      const ufc::cell& c) const final override
  {
    // Declare variables for result of evaluation
    double vals[1];
    
    // Declare variable for physical coordinates
    double y[2];
    switch (i)
    {
    case 0:
      {
        y[0] = coordinate_dofs[0];
      y[1] = coordinate_dofs[1];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 1:
      {
        y[0] = coordinate_dofs[2];
      y[1] = coordinate_dofs[3];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 2:
      {
        y[0] = coordinate_dofs[4];
      y[1] = coordinate_dofs[5];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 3:
      {
        y[0] = 0.5*coordinate_dofs[2] + 0.5*coordinate_dofs[4];
      y[1] = 0.5*coordinate_dofs[3] + 0.5*coordinate_dofs[5];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 4:
      {
        y[0] = 0.5*coordinate_dofs[0] + 0.5*coordinate_dofs[4];
      y[1] = 0.5*coordinate_dofs[1] + 0.5*coordinate_dofs[5];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 5:
      {
        y[0] = 0.5*coordinate_dofs[0] + 0.5*coordinate_dofs[2];
      y[1] = 0.5*coordinate_dofs[1] + 0.5*coordinate_dofs[3];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    }
    
    return 0.0;
  }

  void evaluate_dofs(double * values,
                             const ufc::function& f,
                             const double * coordinate_dofs,
                             int cell_orientation,
                             const ufc::cell& c) const final override
  {
    // Declare variables for result of evaluation
    double vals[1];
    
    // Declare variable for physical coordinates
    double y[2];
    y[0] = coordinate_dofs[0];
    y[1] = coordinate_dofs[1];
    f.evaluate(vals, y, c);
    values[0] = vals[0];
    y[0] = coordinate_dofs[2];
    y[1] = coordinate_dofs[3];
    f.evaluate(vals, y, c);
    values[1] = vals[0];
    y[0] = coordinate_dofs[4];
    y[1] = coordinate_dofs[5];
    f.evaluate(vals, y, c);
    values[2] = vals[0];
    y[0] = 0.5*coordinate_dofs[2] + 0.5*coordinate_dofs[4];
    y[1] = 0.5*coordinate_dofs[3] + 0.5*coordinate_dofs[5];
    f.evaluate(vals, y, c);
    values[3] = vals[0];
    y[0] = 0.5*coordinate_dofs[0] + 0.5*coordinate_dofs[4];
    y[1] = 0.5*coordinate_dofs[1] + 0.5*coordinate_dofs[5];
    f.evaluate(vals, y, c);
    values[4] = vals[0];
    y[0] = 0.5*coordinate_dofs[0] + 0.5*coordinate_dofs[2];
    y[1] = 0.5*coordinate_dofs[1] + 0.5*coordinate_dofs[3];
    f.evaluate(vals, y, c);
    values[5] = vals[0];
  }

  void interpolate_vertex_values(double * vertex_values,
                                 const double * dof_values,
                                 const double * coordinate_dofs,
                                 int cell_orientation,
                                 const ufc::cell& c) const final override
  {
    // Evaluate function and change variables
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
  }

  void tabulate_dof_coordinates(double * dof_coordinates,
                                const double * coordinate_dofs) const final override
  {
    dof_coordinates[0] = coordinate_dofs[0];
    dof_coordinates[1] = coordinate_dofs[1];
    dof_coordinates[2] = coordinate_dofs[2];
    dof_coordinates[3] = coordinate_dofs[3];
    dof_coordinates[4] = coordinate_dofs[4];
    dof_coordinates[5] = coordinate_dofs[5];
    dof_coordinates[6] = 0.5*coordinate_dofs[2] + 0.5*coordinate_dofs[4];
    dof_coordinates[7] = 0.5*coordinate_dofs[3] + 0.5*coordinate_dofs[5];
    dof_coordinates[8] = 0.5*coordinate_dofs[0] + 0.5*coordinate_dofs[4];
    dof_coordinates[9] = 0.5*coordinate_dofs[1] + 0.5*coordinate_dofs[5];
    dof_coordinates[10] = 0.5*coordinate_dofs[0] + 0.5*coordinate_dofs[2];
    dof_coordinates[11] = 0.5*coordinate_dofs[1] + 0.5*coordinate_dofs[3];
  }

  std::size_t num_sub_elements() const final override
  {
    return 0;
  }

  ufc::finite_element * create_sub_element(std::size_t i) const final override
  {
    return 0;
  }

  ufc::finite_element * create() const final override
  {
    return new biharmonic_finite_element_3();
  }

};


class biharmonic_dofmap_0: public ufc::dofmap
{
public:

  biharmonic_dofmap_0() : ufc::dofmap()
  {
    // Do nothing
  }

  ~biharmonic_dofmap_0() override
  {
    // Do nothing
  }

  const char * signature() const final override
  {
    return "FFC dofmap for FiniteElement('Lagrange', triangle, 1)";
  }

  bool needs_mesh_entities(std::size_t d) const final override
  {
    switch (d)
    {
    case 0:
      {
        return true;
        break;
      }
    case 1:
      {
        return false;
        break;
      }
    case 2:
      {
        return false;
        break;
      }
    }
    
    return false;
  }

  std::size_t topological_dimension() const final override
  {
    return 2;
  }

  std::size_t global_dimension(const std::vector<std::size_t>&
                               num_global_entities) const final override
  {
    return num_global_entities[0];
  }

  std::size_t num_element_dofs() const final override
  {
    return 3;
  }

  std::size_t num_facet_dofs() const final override
  {
    return 2;
  }

  std::size_t num_entity_dofs(std::size_t d) const final override
  {
    switch (d)
    {
    case 0:
      {
        return 1;
        break;
      }
    case 1:
      {
        return 0;
        break;
      }
    case 2:
      {
        return 0;
        break;
      }
    }
    
    return 0;
  }

  std::size_t num_entity_closure_dofs(std::size_t d) const final override
  {
    switch (d)
    {
    case 0:
      {
        return 1;
        break;
      }
    case 1:
      {
        return 2;
        break;
      }
    case 2:
      {
        return 3;
        break;
      }
    }
    
    return 0;
  }

  void tabulate_dofs(std::size_t * dofs,
                     const std::vector<std::size_t>& num_global_entities,
                     const std::vector<std::vector<std::size_t>>& entity_indices) const final override
  {
    dofs[0] = entity_indices[0][0];
    dofs[1] = entity_indices[0][1];
    dofs[2] = entity_indices[0][2];
  }

  void tabulate_facet_dofs(std::size_t * dofs,
                           std::size_t facet) const final override
  {
    switch (facet)
    {
    case 0:
      {
        dofs[0] = 1;
      dofs[1] = 2;
        break;
      }
    case 1:
      {
        dofs[0] = 0;
      dofs[1] = 2;
        break;
      }
    case 2:
      {
        dofs[0] = 0;
      dofs[1] = 1;
        break;
      }
    }
    
  }

  void tabulate_entity_dofs(std::size_t * dofs,
                            std::size_t d, std::size_t i) const final override
  {
    if (d > 2)
    {
    throw std::runtime_error("d is larger than dimension (2)");
    }
    
    switch (d)
    {
    case 0:
      {
        if (i > 2)
      {
      throw std::runtime_error("i is larger than number of entities (2)");
      }
      
      switch (i)
      {
      case 0:
        {
          dofs[0] = 0;
          break;
        }
      case 1:
        {
          dofs[0] = 1;
          break;
        }
      case 2:
        {
          dofs[0] = 2;
          break;
        }
      }
      
        break;
      }
    case 1:
      {
        
        break;
      }
    case 2:
      {
        
        break;
      }
    }
    
  }

  void tabulate_entity_closure_dofs(std::size_t * dofs,
                                    std::size_t d, std::size_t i) const final override
  {
    if (d > 2)
    {
    throw std::runtime_error("d is larger than dimension (2)");
    }
    
    switch (d)
    {
    case 0:
      {
        if (i > 2)
      {
      throw std::runtime_error("i is larger than number of entities (2)");
      }
      
      switch (i)
      {
      case 0:
        {
          dofs[0] = 0;
          break;
        }
      case 1:
        {
          dofs[0] = 1;
          break;
        }
      case 2:
        {
          dofs[0] = 2;
          break;
        }
      }
      
        break;
      }
    case 1:
      {
        if (i > 2)
      {
      throw std::runtime_error("i is larger than number of entities (2)");
      }
      
      switch (i)
      {
      case 0:
        {
          dofs[0] = 1;
        dofs[1] = 2;
          break;
        }
      case 1:
        {
          dofs[0] = 0;
        dofs[1] = 2;
          break;
        }
      case 2:
        {
          dofs[0] = 0;
        dofs[1] = 1;
          break;
        }
      }
      
        break;
      }
    case 2:
      {
        if (i > 0)
      {
      throw std::runtime_error("i is larger than number of entities (0)");
      }
      
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
        break;
      }
    }
    
  }


  std::size_t num_sub_dofmaps() const final override
  {
    return 0;
  }

  ufc::dofmap * create_sub_dofmap(std::size_t i) const final override
  {
    return 0;
  }

  ufc::dofmap * create() const final override
  {
    return new biharmonic_dofmap_0();
  }

};


class biharmonic_dofmap_1: public ufc::dofmap
{
public:

  biharmonic_dofmap_1() : ufc::dofmap()
  {
    // Do nothing
  }

  ~biharmonic_dofmap_1() override
  {
    // Do nothing
  }

  const char * signature() const final override
  {
    return "FFC dofmap for VectorElement(FiniteElement('Lagrange', triangle, 1), dim=2)";
  }

  bool needs_mesh_entities(std::size_t d) const final override
  {
    switch (d)
    {
    case 0:
      {
        return true;
        break;
      }
    case 1:
      {
        return false;
        break;
      }
    case 2:
      {
        return false;
        break;
      }
    }
    
    return false;
  }

  std::size_t topological_dimension() const final override
  {
    return 2;
  }

  std::size_t global_dimension(const std::vector<std::size_t>&
                               num_global_entities) const final override
  {
    return 2*num_global_entities[0];
  }

  std::size_t num_element_dofs() const final override
  {
    return 6;
  }

  std::size_t num_facet_dofs() const final override
  {
    return 4;
  }

  std::size_t num_entity_dofs(std::size_t d) const final override
  {
    switch (d)
    {
    case 0:
      {
        return 2;
        break;
      }
    case 1:
      {
        return 0;
        break;
      }
    case 2:
      {
        return 0;
        break;
      }
    }
    
    return 0;
  }

  std::size_t num_entity_closure_dofs(std::size_t d) const final override
  {
    switch (d)
    {
    case 0:
      {
        return 2;
        break;
      }
    case 1:
      {
        return 4;
        break;
      }
    case 2:
      {
        return 6;
        break;
      }
    }
    
    return 0;
  }

  void tabulate_dofs(std::size_t * dofs,
                     const std::vector<std::size_t>& num_global_entities,
                     const std::vector<std::vector<std::size_t>>& entity_indices) const final override
  {
    unsigned int offset = 0;
    dofs[0] = offset + entity_indices[0][0];
    dofs[1] = offset + entity_indices[0][1];
    dofs[2] = offset + entity_indices[0][2];
    offset += num_global_entities[0];
    dofs[3] = offset + entity_indices[0][0];
    dofs[4] = offset + entity_indices[0][1];
    dofs[5] = offset + entity_indices[0][2];
    offset += num_global_entities[0];
  }

  void tabulate_facet_dofs(std::size_t * dofs,
                           std::size_t facet) const final override
  {
    switch (facet)
    {
    case 0:
      {
        dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 4;
      dofs[3] = 5;
        break;
      }
    case 1:
      {
        dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 5;
        break;
      }
    case 2:
      {
        dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      dofs[3] = 4;
        break;
      }
    }
    
  }

  void tabulate_entity_dofs(std::size_t * dofs,
                            std::size_t d, std::size_t i) const final override
  {
    if (d > 2)
    {
    throw std::runtime_error("d is larger than dimension (2)");
    }
    
    switch (d)
    {
    case 0:
      {
        if (i > 2)
      {
      throw std::runtime_error("i is larger than number of entities (2)");
      }
      
      switch (i)
      {
      case 0:
        {
          dofs[0] = 0;
        dofs[1] = 3;
          break;
        }
      case 1:
        {
          dofs[0] = 1;
        dofs[1] = 4;
          break;
        }
      case 2:
        {
          dofs[0] = 2;
        dofs[1] = 5;
          break;
        }
      }
      
        break;
      }
    case 1:
      {
        
        break;
      }
    case 2:
      {
        
        break;
      }
    }
    
  }

  void tabulate_entity_closure_dofs(std::size_t * dofs,
                                    std::size_t d, std::size_t i) const final override
  {
    if (d > 2)
    {
    throw std::runtime_error("d is larger than dimension (2)");
    }
    
    switch (d)
    {
    case 0:
      {
        if (i > 2)
      {
      throw std::runtime_error("i is larger than number of entities (2)");
      }
      
      switch (i)
      {
      case 0:
        {
          dofs[0] = 0;
        dofs[1] = 3;
          break;
        }
      case 1:
        {
          dofs[0] = 1;
        dofs[1] = 4;
          break;
        }
      case 2:
        {
          dofs[0] = 2;
        dofs[1] = 5;
          break;
        }
      }
      
        break;
      }
    case 1:
      {
        if (i > 2)
      {
      throw std::runtime_error("i is larger than number of entities (2)");
      }
      
      switch (i)
      {
      case 0:
        {
          dofs[0] = 1;
        dofs[1] = 2;
        dofs[2] = 4;
        dofs[3] = 5;
          break;
        }
      case 1:
        {
          dofs[0] = 0;
        dofs[1] = 2;
        dofs[2] = 3;
        dofs[3] = 5;
          break;
        }
      case 2:
        {
          dofs[0] = 0;
        dofs[1] = 1;
        dofs[2] = 3;
        dofs[3] = 4;
          break;
        }
      }
      
        break;
      }
    case 2:
      {
        if (i > 0)
      {
      throw std::runtime_error("i is larger than number of entities (0)");
      }
      
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      dofs[3] = 3;
      dofs[4] = 4;
      dofs[5] = 5;
        break;
      }
    }
    
  }


  std::size_t num_sub_dofmaps() const final override
  {
    return 2;
  }

  ufc::dofmap * create_sub_dofmap(std::size_t i) const final override
  {
    switch (i)
    {
    case 0:
      {
        return new biharmonic_dofmap_0();
        break;
      }
    case 1:
      {
        return new biharmonic_dofmap_0();
        break;
      }
    }
    
    return 0;
  }

  ufc::dofmap * create() const final override
  {
    return new biharmonic_dofmap_1();
  }

};


class biharmonic_dofmap_2: public ufc::dofmap
{
public:

  biharmonic_dofmap_2() : ufc::dofmap()
  {
    // Do nothing
  }

  ~biharmonic_dofmap_2() override
  {
    // Do nothing
  }

  const char * signature() const final override
  {
    return "FFC dofmap for FiniteElement('Real', triangle, 0)";
  }

  bool needs_mesh_entities(std::size_t d) const final override
  {
    switch (d)
    {
    case 0:
      {
        return false;
        break;
      }
    case 1:
      {
        return false;
        break;
      }
    case 2:
      {
        return false;
        break;
      }
    }
    
    return false;
  }

  std::size_t topological_dimension() const final override
  {
    return 2;
  }

  std::size_t global_dimension(const std::vector<std::size_t>&
                               num_global_entities) const final override
  {
    return 1;
  }

  std::size_t num_element_dofs() const final override
  {
    return 1;
  }

  std::size_t num_facet_dofs() const final override
  {
    return 0;
  }

  std::size_t num_entity_dofs(std::size_t d) const final override
  {
    switch (d)
    {
    case 0:
      {
        return 0;
        break;
      }
    case 1:
      {
        return 0;
        break;
      }
    case 2:
      {
        return 1;
        break;
      }
    }
    
    return 0;
  }

  std::size_t num_entity_closure_dofs(std::size_t d) const final override
  {
    switch (d)
    {
    case 0:
      {
        return 0;
        break;
      }
    case 1:
      {
        return 0;
        break;
      }
    case 2:
      {
        return 1;
        break;
      }
    }
    
    return 0;
  }

  void tabulate_dofs(std::size_t * dofs,
                     const std::vector<std::size_t>& num_global_entities,
                     const std::vector<std::vector<std::size_t>>& entity_indices) const final override
  {
    dofs[0] = 0;
  }

  void tabulate_facet_dofs(std::size_t * dofs,
                           std::size_t facet) const final override
  {
    switch (facet)
    {
    case 0:
      {
        
        break;
      }
    case 1:
      {
        
        break;
      }
    case 2:
      {
        
        break;
      }
    }
    
  }

  void tabulate_entity_dofs(std::size_t * dofs,
                            std::size_t d, std::size_t i) const final override
  {
    if (d > 2)
    {
    throw std::runtime_error("d is larger than dimension (2)");
    }
    
    switch (d)
    {
    case 0:
      {
        
        break;
      }
    case 1:
      {
        
        break;
      }
    case 2:
      {
        if (i > 0)
      {
      throw std::runtime_error("i is larger than number of entities (0)");
      }
      
      dofs[0] = 0;
        break;
      }
    }
    
  }

  void tabulate_entity_closure_dofs(std::size_t * dofs,
                                    std::size_t d, std::size_t i) const final override
  {
    if (d > 2)
    {
    throw std::runtime_error("d is larger than dimension (2)");
    }
    
    switch (d)
    {
    case 0:
      {
        if (i > 2)
      {
      throw std::runtime_error("i is larger than number of entities (2)");
      }
      
      switch (i)
      {
      case 0:
        {
          
          break;
        }
      case 1:
        {
          
          break;
        }
      case 2:
        {
          
          break;
        }
      }
      
        break;
      }
    case 1:
      {
        if (i > 2)
      {
      throw std::runtime_error("i is larger than number of entities (2)");
      }
      
      switch (i)
      {
      case 0:
        {
          
          break;
        }
      case 1:
        {
          
          break;
        }
      case 2:
        {
          
          break;
        }
      }
      
        break;
      }
    case 2:
      {
        if (i > 0)
      {
      throw std::runtime_error("i is larger than number of entities (0)");
      }
      
      dofs[0] = 0;
        break;
      }
    }
    
  }


  std::size_t num_sub_dofmaps() const final override
  {
    return 0;
  }

  ufc::dofmap * create_sub_dofmap(std::size_t i) const final override
  {
    return 0;
  }

  ufc::dofmap * create() const final override
  {
    return new biharmonic_dofmap_2();
  }

};


class biharmonic_dofmap_3: public ufc::dofmap
{
public:

  biharmonic_dofmap_3() : ufc::dofmap()
  {
    // Do nothing
  }

  ~biharmonic_dofmap_3() override
  {
    // Do nothing
  }

  const char * signature() const final override
  {
    return "FFC dofmap for FiniteElement('Lagrange', triangle, 2)";
  }

  bool needs_mesh_entities(std::size_t d) const final override
  {
    switch (d)
    {
    case 0:
      {
        return true;
        break;
      }
    case 1:
      {
        return true;
        break;
      }
    case 2:
      {
        return false;
        break;
      }
    }
    
    return false;
  }

  std::size_t topological_dimension() const final override
  {
    return 2;
  }

  std::size_t global_dimension(const std::vector<std::size_t>&
                               num_global_entities) const final override
  {
    return num_global_entities[0] + num_global_entities[1];
  }

  std::size_t num_element_dofs() const final override
  {
    return 6;
  }

  std::size_t num_facet_dofs() const final override
  {
    return 3;
  }

  std::size_t num_entity_dofs(std::size_t d) const final override
  {
    switch (d)
    {
    case 0:
      {
        return 1;
        break;
      }
    case 1:
      {
        return 1;
        break;
      }
    case 2:
      {
        return 0;
        break;
      }
    }
    
    return 0;
  }

  std::size_t num_entity_closure_dofs(std::size_t d) const final override
  {
    switch (d)
    {
    case 0:
      {
        return 1;
        break;
      }
    case 1:
      {
        return 3;
        break;
      }
    case 2:
      {
        return 6;
        break;
      }
    }
    
    return 0;
  }

  void tabulate_dofs(std::size_t * dofs,
                     const std::vector<std::size_t>& num_global_entities,
                     const std::vector<std::vector<std::size_t>>& entity_indices) const final override
  {
    unsigned int offset = 0;
    dofs[0] = offset + entity_indices[0][0];
    dofs[1] = offset + entity_indices[0][1];
    dofs[2] = offset + entity_indices[0][2];
    offset += num_global_entities[0];
    dofs[3] = offset + entity_indices[1][0];
    dofs[4] = offset + entity_indices[1][1];
    dofs[5] = offset + entity_indices[1][2];
    offset += num_global_entities[1];
  }

  void tabulate_facet_dofs(std::size_t * dofs,
                           std::size_t facet) const final override
  {
    switch (facet)
    {
    case 0:
      {
        dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
        break;
      }
    case 1:
      {
        dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 4;
        break;
      }
    case 2:
      {
        dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 5;
        break;
      }
    }
    
  }

  void tabulate_entity_dofs(std::size_t * dofs,
                            std::size_t d, std::size_t i) const final override
  {
    if (d > 2)
    {
    throw std::runtime_error("d is larger than dimension (2)");
    }
    
    switch (d)
    {
    case 0:
      {
        if (i > 2)
      {
      throw std::runtime_error("i is larger than number of entities (2)");
      }
      
      switch (i)
      {
      case 0:
        {
          dofs[0] = 0;
          break;
        }
      case 1:
        {
          dofs[0] = 1;
          break;
        }
      case 2:
        {
          dofs[0] = 2;
          break;
        }
      }
      
        break;
      }
    case 1:
      {
        if (i > 2)
      {
      throw std::runtime_error("i is larger than number of entities (2)");
      }
      
      switch (i)
      {
      case 0:
        {
          dofs[0] = 3;
          break;
        }
      case 1:
        {
          dofs[0] = 4;
          break;
        }
      case 2:
        {
          dofs[0] = 5;
          break;
        }
      }
      
        break;
      }
    case 2:
      {
        
        break;
      }
    }
    
  }

  void tabulate_entity_closure_dofs(std::size_t * dofs,
                                    std::size_t d, std::size_t i) const final override
  {
    if (d > 2)
    {
    throw std::runtime_error("d is larger than dimension (2)");
    }
    
    switch (d)
    {
    case 0:
      {
        if (i > 2)
      {
      throw std::runtime_error("i is larger than number of entities (2)");
      }
      
      switch (i)
      {
      case 0:
        {
          dofs[0] = 0;
          break;
        }
      case 1:
        {
          dofs[0] = 1;
          break;
        }
      case 2:
        {
          dofs[0] = 2;
          break;
        }
      }
      
        break;
      }
    case 1:
      {
        if (i > 2)
      {
      throw std::runtime_error("i is larger than number of entities (2)");
      }
      
      switch (i)
      {
      case 0:
        {
          dofs[0] = 1;
        dofs[1] = 2;
        dofs[2] = 3;
          break;
        }
      case 1:
        {
          dofs[0] = 0;
        dofs[1] = 2;
        dofs[2] = 4;
          break;
        }
      case 2:
        {
          dofs[0] = 0;
        dofs[1] = 1;
        dofs[2] = 5;
          break;
        }
      }
      
        break;
      }
    case 2:
      {
        if (i > 0)
      {
      throw std::runtime_error("i is larger than number of entities (0)");
      }
      
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      dofs[3] = 3;
      dofs[4] = 4;
      dofs[5] = 5;
        break;
      }
    }
    
  }


  std::size_t num_sub_dofmaps() const final override
  {
    return 0;
  }

  ufc::dofmap * create_sub_dofmap(std::size_t i) const final override
  {
    return 0;
  }

  ufc::dofmap * create() const final override
  {
    return new biharmonic_dofmap_3();
  }

};


class biharmonic_cell_integral_0_otherwise: public ufc::cell_integral
{
public:

  biharmonic_cell_integral_0_otherwise() : ufc::cell_integral()
  {

  }

  ~biharmonic_cell_integral_0_otherwise() override
  {

  }

  const std::vector<bool> & enabled_coefficients() const final override
  {
static const std::vector<bool> enabled({false});
return enabled;
  }

  void tabulate_tensor(double * A,
                       const double * const * w,
                       const double * coordinate_dofs,
                       int cell_orientation) const final override
  {
    // This function was generated using 'uflacs' representation
    // with the following integrals metadata:
    // 
    //   num_cells:         None
    //   optimize:          True
    //   precision:         16
    //   quadrature_degree: 0
    //   quadrature_rule:   'default'
    //   representation:    'uflacs'
    // 
    // and the following integral 0 metadata:
    // 
    //   estimated_polynomial_degree: 0
    //   optimize:                    True
    //   precision:                   16
    //   quadrature_degree:           0
    //   quadrature_rule:             'default'
    //   representation:              'uflacs'
    
    // Precomputed values of basis functions and precomputations
    // FE* dimensions: [entities][points][dofs]
    // PI* dimensions: [entities][dofs][dofs] or [entities][dofs]
    // PM* dimensions: [entities][dofs][dofs]
    alignas(32) static const double FE3_C0_D01_Q1[1][1][2] = { { { -1.0, 1.0 } } };
    // Unstructured piecewise computations
    const double J_c0 = coordinate_dofs[0] * FE3_C0_D01_Q1[0][0][0] + coordinate_dofs[2] * FE3_C0_D01_Q1[0][0][1];
    const double J_c3 = coordinate_dofs[1] * FE3_C0_D01_Q1[0][0][0] + coordinate_dofs[5] * FE3_C0_D01_Q1[0][0][1];
    const double J_c1 = coordinate_dofs[0] * FE3_C0_D01_Q1[0][0][0] + coordinate_dofs[4] * FE3_C0_D01_Q1[0][0][1];
    const double J_c2 = coordinate_dofs[1] * FE3_C0_D01_Q1[0][0][0] + coordinate_dofs[3] * FE3_C0_D01_Q1[0][0][1];
    alignas(32) double sp[31];
    sp[0] = J_c0 * J_c3;
    sp[1] = J_c1 * J_c2;
    sp[2] = sp[0] + -1 * sp[1];
    sp[3] = J_c0 / sp[2];
    sp[4] = -1 * J_c1 / sp[2];
    sp[5] = sp[3] * sp[3];
    sp[6] = sp[3] * sp[4];
    sp[7] = sp[4] * sp[4];
    sp[8] = sp[6] + sp[6];
    sp[9] = J_c3 / sp[2];
    sp[10] = -1 * J_c2 / sp[2];
    sp[11] = sp[9] * sp[10];
    sp[12] = sp[9] * sp[9];
    sp[13] = sp[10] * sp[10];
    sp[14] = sp[11] + sp[11];
    sp[15] = sp[5] + sp[13];
    sp[16] = sp[8] + sp[14];
    sp[17] = sp[12] + sp[7];
    sp[18] = sp[15] * sp[15];
    sp[19] = sp[16] * sp[15];
    sp[20] = sp[17] * sp[15];
    sp[21] = sp[16] * sp[16];
    sp[22] = sp[16] * sp[17];
    sp[23] = sp[17] * sp[17];
    sp[24] = std::abs(sp[2]);
    sp[25] = sp[18] * sp[24];
    sp[26] = sp[19] * sp[24];
    sp[27] = sp[20] * sp[24];
    sp[28] = sp[21] * sp[24];
    sp[29] = sp[22] * sp[24];
    sp[30] = sp[23] * sp[24];
    A[0] = 8.000000000000007 * sp[30] + 8.000000000000007 * sp[27] + 8.000000000000009 * sp[29] + 8.000000000000007 * sp[27] + 8.000000000000007 * sp[25] + 8.000000000000009 * sp[26] + 8.000000000000009 * sp[29] + 8.000000000000009 * sp[26] + 8.000000000000011 * sp[28];
    A[1] = 8.000000000000005 * sp[30] + 8.000000000000005 * sp[27] + 8.000000000000007 * sp[29];
    A[2] = 8.000000000000005 * sp[27] + 8.000000000000005 * sp[25] + 8.000000000000007 * sp[26];
    A[3] = 7.999999999999992 * sp[29] + 7.999999999999992 * sp[26] + 7.999999999999994 * sp[28];
    A[4] = -16.00000000000001 * sp[27] + -7.999999999999992 * sp[29] + -16.00000000000001 * sp[25] + -7.999999999999992 * sp[26] + -16.00000000000001 * sp[26] + -7.999999999999994 * sp[28];
    A[5] = -16.00000000000001 * sp[30] + -8.000000000000025 * sp[29] + -16.00000000000001 * sp[27] + -8.000000000000025 * sp[26] + -16.00000000000001 * sp[29] + -8.000000000000027 * sp[28];
    A[6] = 8.000000000000005 * sp[30] + 8.000000000000005 * sp[27] + 8.000000000000007 * sp[29];
    A[7] = 8.000000000000004 * sp[30];
    A[8] = 8.000000000000004 * sp[27];
    A[9] = 7.99999999999999 * sp[29];
    A[10] = -16.0 * sp[27] + -7.99999999999999 * sp[29];
    A[11] = -16.0 * sp[30] + -8.000000000000023 * sp[29];
    A[12] = 8.000000000000005 * sp[27] + 8.000000000000005 * sp[25] + 8.000000000000007 * sp[26];
    A[13] = 8.000000000000004 * sp[27];
    A[14] = 8.000000000000004 * sp[25];
    A[15] = 7.99999999999999 * sp[26];
    A[16] = -16.0 * sp[25] + -7.99999999999999 * sp[26];
    A[17] = -16.0 * sp[27] + -8.000000000000023 * sp[26];
    A[18] = 7.999999999999992 * sp[29] + 7.999999999999992 * sp[26] + 7.999999999999994 * sp[28];
    A[19] = 7.99999999999999 * sp[29];
    A[20] = 7.99999999999999 * sp[26];
    A[21] = 7.999999999999977 * sp[28];
    A[22] = -15.99999999999998 * sp[26] + -7.999999999999977 * sp[28];
    A[23] = -15.99999999999998 * sp[29] + -8.000000000000009 * sp[28];
    A[24] = -16.00000000000001 * sp[27] + -16.00000000000001 * sp[25] + -16.00000000000001 * sp[26] + -7.999999999999992 * sp[29] + -7.999999999999992 * sp[26] + -7.999999999999994 * sp[28];
    A[25] = -16.0 * sp[27] + -7.99999999999999 * sp[29];
    A[26] = -16.0 * sp[25] + -7.99999999999999 * sp[26];
    A[27] = -15.99999999999998 * sp[26] + -7.999999999999977 * sp[28];
    A[28] = 32.0 * sp[25] + 15.99999999999998 * sp[26] + 15.99999999999998 * sp[26] + 7.999999999999977 * sp[28];
    A[29] = 32.0 * sp[27] + 16.00000000000004 * sp[26] + 15.99999999999998 * sp[29] + 8.000000000000009 * sp[28];
    A[30] = -16.00000000000001 * sp[30] + -16.00000000000001 * sp[27] + -16.00000000000001 * sp[29] + -8.000000000000025 * sp[29] + -8.000000000000025 * sp[26] + -8.000000000000027 * sp[28];
    A[31] = -16.0 * sp[30] + -8.000000000000023 * sp[29];
    A[32] = -16.0 * sp[27] + -8.000000000000023 * sp[26];
    A[33] = -15.99999999999998 * sp[29] + -8.000000000000009 * sp[28];
    A[34] = 32.0 * sp[27] + 15.99999999999998 * sp[29] + 16.00000000000004 * sp[26] + 8.000000000000009 * sp[28];
    A[35] = 32.0 * sp[30] + 16.00000000000004 * sp[29] + 16.00000000000004 * sp[29] + 8.000000000000043 * sp[28];
  }

};


class biharmonic_interior_facet_integral_0_otherwise: public ufc::interior_facet_integral
{
public:

  biharmonic_interior_facet_integral_0_otherwise() : ufc::interior_facet_integral()
  {

  }

  ~biharmonic_interior_facet_integral_0_otherwise() override
  {

  }

  const std::vector<bool> & enabled_coefficients() const final override
  {
static const std::vector<bool> enabled({true});
return enabled;
  }

  void tabulate_tensor(double * A,
                       const double * const * w,
                       const double * coordinate_dofs_0,
                       const double * coordinate_dofs_1,
                       std::size_t facet_0,
                       std::size_t facet_1,
                       int cell_orientation_0,
                       int cell_orientation_1) const final override
  {
    // This function was generated using 'uflacs' representation
    // with the following integrals metadata:
    // 
    //   num_cells:         None
    //   optimize:          True
    //   precision:         16
    //   quadrature_degree: 2
    //   quadrature_rule:   'default'
    //   representation:    'uflacs'
    // 
    // and the following integral 0 metadata:
    // 
    //   estimated_polynomial_degree: 2
    //   optimize:                    True
    //   precision:                   16
    //   quadrature_degree:           2
    //   quadrature_rule:             'default'
    //   representation:              'uflacs'
    
    // Precomputed values of basis functions and precomputations
    // FE* dimensions: [entities][points][dofs]
    // PI* dimensions: [entities][dofs][dofs] or [entities][dofs]
    // PM* dimensions: [entities][dofs][dofs]
    alignas(32) static const double FE7_C0_D01_F_Q2[1][1][2] = { { { -1.0, 1.0 } } };
    alignas(32) static const double PI0[3][3][5][5] =
        { { { { 1.0, 1.0, 1.999999999999997, -1.999999999999997, -2.000000000000002 },
              { 1.0, 2.333333333333333, 0.666666666666666, -0.666666666666666, -3.333333333333336 },
              { 1.999999999999997, 0.666666666666666, 5.333333333333316, -5.333333333333316, -2.666666666666666 },
              { -1.999999999999997, -0.666666666666666, -5.333333333333316, 5.333333333333316, 2.666666666666666 },
              { -2.000000000000002, -3.333333333333336, -2.666666666666666, 2.666666666666666, 5.333333333333343 } },
            { { -1.0, -1.0, 1.999999999999997, -1.999999999999997, 2.000000000000003 },
              { -2.333333333333334, -1.0, 0.666666666666666, -0.666666666666666, 3.333333333333338 },
              { -0.6666666666666669, -1.999999999999997, 5.333333333333316, -5.333333333333316, 2.666666666666667 },
              { 0.6666666666666669, 1.999999999999997, -5.333333333333316, 5.333333333333316, -2.666666666666667 },
              { 3.333333333333338, 2.000000000000002, -2.666666666666666, 2.666666666666666, -5.333333333333345 } },
            { { -1.0, 1.0, 0.0, 0.0, 0.0 },
              { -2.333333333333332, -0.3333333333333354, 0.0, 0.0, 2.666666666666668 },
              { -0.6666666666666617, 3.333333333333325, 0.0, 0.0, -2.666666666666663 },
              { 0.6666666666666617, -3.333333333333325, 0.0, 0.0, 2.666666666666663 },
              { 3.333333333333333, -0.6666666666666637, 0.0, 0.0, -2.66666666666667 } } },
          { { { -1.0, -2.333333333333334, -0.6666666666666669, 0.6666666666666669, 3.333333333333338 },
              { -1.0, -1.0, -1.999999999999997, 1.999999999999997, 2.000000000000002 },
              { 1.999999999999997, 0.666666666666666, 5.333333333333316, -5.333333333333316, -2.666666666666666 },
              { -1.999999999999997, -0.666666666666666, -5.333333333333316, 5.333333333333316, 2.666666666666666 },
              { 2.000000000000003, 3.333333333333338, 2.666666666666667, -2.666666666666667, -5.333333333333345 } },
            { { 2.333333333333336, 1.0, -0.6666666666666669, 0.6666666666666669, -3.33333333333334 },
              { 1.0, 1.0, -1.999999999999997, 1.999999999999997, -2.000000000000003 },
              { -0.6666666666666669, -1.999999999999997, 5.333333333333316, -5.333333333333316, 2.666666666666667 },
              { 0.6666666666666669, 1.999999999999997, -5.333333333333316, 5.333333333333316, -2.666666666666667 },
              { -3.33333333333334, -2.000000000000003, 2.666666666666667, -2.666666666666667, 5.333333333333348 } },
            { { 2.333333333333333, 0.3333333333333352, 0.0, 0.0, -2.666666666666669 },
              { 1.0, -1.0, 0.0, 0.0, 0.0 },
              { -0.6666666666666617, 3.333333333333325, 0.0, 0.0, -2.666666666666663 },
              { 0.6666666666666617, -3.333333333333325, 0.0, 0.0, 2.666666666666663 },
              { -3.333333333333336, 0.6666666666666636, 0.0, 0.0, 2.666666666666672 } } },
          { { { -1.0, -2.333333333333332, -0.6666666666666617, 0.6666666666666617, 3.333333333333333 },
              { 1.0, -0.3333333333333354, 3.333333333333325, -3.333333333333325, -0.6666666666666637 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 },
              { 0.0, 2.666666666666668, -2.666666666666663, 2.666666666666663, -2.66666666666667 } },
            { { 2.333333333333333, 1.0, -0.6666666666666617, 0.6666666666666617, -3.333333333333336 },
              { 0.3333333333333352, -1.0, 3.333333333333325, -3.333333333333325, 0.6666666666666636 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 },
              { -2.666666666666669, 0.0, -2.666666666666663, 2.666666666666663, 2.666666666666672 } },
            { { 2.333333333333332, 0.3333333333333379, 0.0, 0.0, -2.66666666666667 },
              { 0.3333333333333379, 2.333333333333331, 0.0, 0.0, -2.666666666666669 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 },
              { -2.66666666666667, -2.666666666666669, 0.0, 0.0, 5.333333333333339 } } } };
    alignas(32) static const double PI1[3][3][5][3] =
        { { { { 4.0, 3.999999999999998, -7.999999999999996 },
              { 4.000000000000001, 3.999999999999998, -7.999999999999998 },
              { 7.999999999999987, 7.999999999999982, -15.99999999999997 },
              { -7.999999999999987, -7.999999999999982, 15.99999999999997 },
              { -8.000000000000009, -8.000000000000004, 16.00000000000001 } },
            { { 4.0, 3.999999999999998, -7.999999999999996 },
              { 4.000000000000001, 3.999999999999998, -7.999999999999998 },
              { 7.999999999999987, 7.999999999999982, -15.99999999999997 },
              { -7.999999999999987, -7.999999999999982, 15.99999999999997 },
              { -8.000000000000009, -8.000000000000004, 16.00000000000001 } },
            { { 4.0, 3.999999999999998, -7.999999999999996 },
              { 4.000000000000001, 3.999999999999998, -7.999999999999998 },
              { 7.999999999999987, 7.999999999999982, -15.99999999999997 },
              { -7.999999999999987, -7.999999999999982, 15.99999999999997 },
              { -8.000000000000009, -8.000000000000004, 16.00000000000001 } } },
          { { { -4.000000000000004, -4.000000000000002, 8.000000000000005 },
              { -4.0, -3.999999999999998, 7.999999999999996 },
              { 7.999999999999987, 7.999999999999982, -15.99999999999997 },
              { -7.999999999999987, -7.999999999999982, 15.99999999999997 },
              { 8.000000000000012, 8.000000000000007, -16.00000000000002 } },
            { { -4.000000000000004, -4.000000000000002, 8.000000000000005 },
              { -4.0, -3.999999999999998, 7.999999999999996 },
              { 7.999999999999987, 7.999999999999982, -15.99999999999997 },
              { -7.999999999999987, -7.999999999999982, 15.99999999999997 },
              { 8.000000000000012, 8.000000000000007, -16.00000000000002 } },
            { { -4.000000000000004, -4.000000000000002, 8.000000000000005 },
              { -4.0, -3.999999999999998, 7.999999999999996 },
              { 7.999999999999987, 7.999999999999982, -15.99999999999997 },
              { -7.999999999999987, -7.999999999999982, 15.99999999999997 },
              { 8.000000000000012, 8.000000000000007, -16.00000000000002 } } },
          { { { -3.999999999999994, -3.999999999999992, 7.999999999999985 },
              { 3.999999999999992, 3.99999999999999, -7.999999999999981 },
              { 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0 } },
            { { -3.999999999999994, -3.999999999999992, 7.999999999999985 },
              { 3.999999999999992, 3.99999999999999, -7.999999999999981 },
              { 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0 } },
            { { -3.999999999999994, -3.999999999999992, 7.999999999999985 },
              { 3.999999999999992, 3.99999999999999, -7.999999999999981 },
              { 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0 } } } };
    alignas(32) static const double PI2[3][3][5][4] =
        { { { { 4.000000000000001, 3.999999999999993, -3.999999999999993, -4.000000000000011 },
              { 4.000000000000002, 3.999999999999994, -3.999999999999994, -4.000000000000012 },
              { 7.999999999999989, 7.999999999999973, -7.999999999999973, -8.000000000000007 },
              { -7.999999999999989, -7.999999999999973, 7.999999999999973, 8.000000000000007 },
              { -8.000000000000011, -7.999999999999995, 7.999999999999995, 8.00000000000003 } },
            { { 4.000000000000001, 3.999999999999993, -3.999999999999993, -4.000000000000011 },
              { 4.000000000000002, 3.999999999999994, -3.999999999999994, -4.000000000000012 },
              { 7.999999999999989, 7.999999999999973, -7.999999999999973, -8.000000000000007 },
              { -7.999999999999989, -7.999999999999973, 7.999999999999973, 8.000000000000007 },
              { -8.000000000000011, -7.999999999999995, 7.999999999999995, 8.00000000000003 } },
            { { 4.000000000000001, 3.999999999999993, -3.999999999999993, -4.000000000000011 },
              { 4.000000000000002, 3.999999999999994, -3.999999999999994, -4.000000000000012 },
              { 7.999999999999989, 7.999999999999973, -7.999999999999973, -8.000000000000007 },
              { -7.999999999999989, -7.999999999999973, 7.999999999999973, 8.000000000000007 },
              { -8.000000000000011, -7.999999999999995, 7.999999999999995, 8.00000000000003 } } },
          { { { -4.000000000000004, -3.999999999999997, 3.999999999999997, 4.000000000000015 },
              { -4.000000000000001, -3.999999999999993, 3.999999999999993, 4.000000000000011 },
              { 7.999999999999989, 7.999999999999973, -7.999999999999973, -8.000000000000007 },
              { -7.999999999999989, -7.999999999999973, 7.999999999999973, 8.000000000000007 },
              { 8.000000000000014, 7.999999999999998, -7.999999999999998, -8.000000000000034 } },
            { { -4.000000000000004, -3.999999999999997, 3.999999999999997, 4.000000000000015 },
              { -4.000000000000001, -3.999999999999993, 3.999999999999993, 4.000000000000011 },
              { 7.999999999999989, 7.999999999999973, -7.999999999999973, -8.000000000000007 },
              { -7.999999999999989, -7.999999999999973, 7.999999999999973, 8.000000000000007 },
              { 8.000000000000014, 7.999999999999998, -7.999999999999998, -8.000000000000034 } },
            { { -4.000000000000004, -3.999999999999997, 3.999999999999997, 4.000000000000015 },
              { -4.000000000000001, -3.999999999999993, 3.999999999999993, 4.000000000000011 },
              { 7.999999999999989, 7.999999999999973, -7.999999999999973, -8.000000000000007 },
              { -7.999999999999989, -7.999999999999973, 7.999999999999973, 8.000000000000007 },
              { 8.000000000000014, 7.999999999999998, -7.999999999999998, -8.000000000000034 } } },
          { { { -3.999999999999995, -3.999999999999988, 3.999999999999988, 4.000000000000005 },
              { 3.999999999999993, 3.999999999999986, -3.999999999999986, -4.000000000000003 },
              { 0.0, 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0, 0.0 } },
            { { -3.999999999999995, -3.999999999999988, 3.999999999999988, 4.000000000000005 },
              { 3.999999999999993, 3.999999999999986, -3.999999999999986, -4.000000000000003 },
              { 0.0, 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0, 0.0 } },
            { { -3.999999999999995, -3.999999999999988, 3.999999999999988, 4.000000000000005 },
              { 3.999999999999993, 3.999999999999986, -3.999999999999986, -4.000000000000003 },
              { 0.0, 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0, 0.0 } } } };
    alignas(32) static const double PI3[3][3][5][5] =
        { { { { 1.0, 1.0, 1.999999999999985, -1.99999999999999, -2.0 },
              { 1.0, -0.3333333333333318, 3.333333333333319, -0.6666666666666563, -3.333333333333335 },
              { 1.999999999999997, 3.333333333333333, 2.666666666666631, -5.333333333333307, -2.66666666666666 },
              { -1.999999999999997, -3.333333333333333, -2.666666666666631, 5.333333333333307, 2.66666666666666 },
              { -2.000000000000002, -0.6666666666666712, -5.333333333333308, 2.66666666666665, 5.33333333333334 } },
            { { -1.0, 1.0, 0.0, 0.0, 0.0 },
              { -2.333333333333337, -0.3333333333333318, 0.0, 2.666666666666673, 0.0 },
              { -0.6666666666666754, 3.333333333333333, 0.0, -2.666666666666647, 0.0 },
              { 0.6666666666666754, -3.333333333333333, 0.0, 2.666666666666647, 0.0 },
              { 3.333333333333345, -0.6666666666666712, 0.0, -2.666666666666682, 0.0 } },
            { { -1.0, -1.0, 1.999999999999988, 2.000000000000009, -2.0 },
              { -2.333333333333337, -1.0, 0.6666666666666581, 3.33333333333334, -0.6666666666666672 },
              { -0.6666666666666741, -1.999999999999997, 5.333333333333299, 2.666666666666682, -5.333333333333326 },
              { 0.6666666666666741, 1.999999999999997, -5.333333333333299, -2.666666666666682, 5.333333333333326 },
              { 3.333333333333343, 2.000000000000002, -2.666666666666649, -5.333333333333354, 2.666666666666671 } } },
          { { { -1.0, 0.3333333333333317, -3.333333333333321, 0.6666666666666572, 3.333333333333337 },
              { -1.0, -1.0, -1.999999999999985, 1.99999999999999, 2.0 },
              { 1.999999999999997, 3.333333333333333, 2.666666666666631, -5.333333333333307, -2.66666666666666 },
              { -1.999999999999997, -3.333333333333333, -2.666666666666631, 5.333333333333307, 2.66666666666666 },
              { 2.000000000000003, 0.666666666666671, 5.333333333333311, -2.666666666666651, -5.333333333333344 } },
            { { 2.333333333333339, 0.3333333333333317, 0.0, -2.666666666666674, 0.0 },
              { 1.0, -1.0, 0.0, 0.0, 0.0 },
              { -0.6666666666666754, 3.333333333333333, 0.0, -2.666666666666647, 0.0 },
              { 0.6666666666666754, -3.333333333333333, 0.0, 2.666666666666647, 0.0 },
              { -3.333333333333347, 0.666666666666671, 0.0, 2.666666666666684, 0.0 } },
            { { 2.333333333333338, 1.0, -0.666666666666659, -3.333333333333342, 0.6666666666666681 },
              { 1.0, 1.0, -1.999999999999988, -2.000000000000009, 2.0 },
              { -0.6666666666666741, -1.999999999999997, 5.333333333333299, 2.666666666666682, -5.333333333333326 },
              { 0.6666666666666741, 1.999999999999997, -5.333333333333299, -2.666666666666682, 5.333333333333326 },
              { -3.333333333333345, -2.000000000000003, 2.66666666666665, 5.333333333333357, -2.666666666666671 } } },
          { { { -1.0, 0.3333333333333344, -3.333333333333316, 0.6666666666666521, 3.333333333333333 },
              { 1.0, 2.333333333333334, 0.6666666666666461, -3.333333333333322, -0.6666666666666605 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 },
              { 0.0, -2.666666666666669, 2.666666666666671, 2.666666666666671, -2.666666666666672 } },
            { { 2.333333333333337, 0.3333333333333344, 0.0, -2.666666666666675, 0.0 },
              { 0.3333333333333305, 2.333333333333334, 0.0, -2.666666666666659, 0.0 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 },
              { -2.666666666666667, -2.666666666666669, 0.0, 5.333333333333335, 0.0 } },
            { { 2.333333333333336, 1.0, -0.6666666666666538, -3.333333333333337, 0.666666666666663 },
              { 0.3333333333333312, -1.0, 3.333333333333316, 0.6666666666666734, -3.333333333333331 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 },
              { -2.666666666666667, 0.0, -2.666666666666663, 2.666666666666664, 2.666666666666668 } } } };
    alignas(32) static const double PI4[3][3][3][5] =
        { { { { 4.0, 4.000000000000007, 7.999999999999939, -7.999999999999962, -8.0 },
              { 3.999999999999998, 4.000000000000004, 7.999999999999934, -7.999999999999957, -7.999999999999996 },
              { -7.999999999999996, -8.000000000000011, -15.99999999999987, 15.99999999999992, 15.99999999999999 } },
            { { -4.00000000000002, 4.000000000000007, 0.0, 0.0, 0.0 },
              { -4.000000000000017, 4.000000000000004, 0.0, 0.0, 0.0 },
              { 8.000000000000036, -8.000000000000011, 0.0, 0.0, 0.0 } },
            { { -4.000000000000016, -4.0, 7.999999999999954, 8.000000000000036, -8.000000000000002 },
              { -4.000000000000013, -3.999999999999998, 7.999999999999949, 8.000000000000032, -7.999999999999996 },
              { 8.000000000000028, 7.999999999999996, -15.9999999999999, -16.00000000000006, 16.0 } } },
          { { { 4.0, 4.000000000000007, 7.999999999999939, -7.999999999999962, -8.0 },
              { 3.999999999999998, 4.000000000000004, 7.999999999999934, -7.999999999999957, -7.999999999999996 },
              { -7.999999999999996, -8.000000000000011, -15.99999999999987, 15.99999999999992, 15.99999999999999 } },
            { { -4.00000000000002, 4.000000000000007, 0.0, 0.0, 0.0 },
              { -4.000000000000017, 4.000000000000004, 0.0, 0.0, 0.0 },
              { 8.000000000000036, -8.000000000000011, 0.0, 0.0, 0.0 } },
            { { -4.000000000000016, -4.0, 7.999999999999954, 8.000000000000036, -8.000000000000002 },
              { -4.000000000000013, -3.999999999999998, 7.999999999999949, 8.000000000000032, -7.999999999999996 },
              { 8.000000000000028, 7.999999999999996, -15.9999999999999, -16.00000000000006, 16.0 } } },
          { { { 4.0, 4.000000000000007, 7.999999999999939, -7.999999999999962, -8.0 },
              { 3.999999999999998, 4.000000000000004, 7.999999999999934, -7.999999999999957, -7.999999999999996 },
              { -7.999999999999996, -8.000000000000011, -15.99999999999987, 15.99999999999992, 15.99999999999999 } },
            { { -4.00000000000002, 4.000000000000007, 0.0, 0.0, 0.0 },
              { -4.000000000000017, 4.000000000000004, 0.0, 0.0, 0.0 },
              { 8.000000000000036, -8.000000000000011, 0.0, 0.0, 0.0 } },
            { { -4.000000000000016, -4.0, 7.999999999999954, 8.000000000000036, -8.000000000000002 },
              { -4.000000000000013, -3.999999999999998, 7.999999999999949, 8.000000000000032, -7.999999999999996 },
              { 8.000000000000028, 7.999999999999996, -15.9999999999999, -16.00000000000006, 16.0 } } } };
    alignas(32) static const double PI5[3][3][4][5] =
        { { { { 4.000000000000001, 4.000000000000008, 7.999999999999941, -7.999999999999964, -8.000000000000004 },
              { 3.999999999999993, 4.000000000000001, 7.999999999999925, -7.999999999999948, -7.999999999999988 },
              { -3.999999999999993, -4.000000000000001, -7.999999999999925, 7.999999999999948, 7.999999999999988 },
              { -4.000000000000011, -4.000000000000018, -7.99999999999996, 7.999999999999983, 8.000000000000021 } },
            { { -4.00000000000002, 4.000000000000008, 0.0, 0.0, 0.0 },
              { -4.000000000000012, 4.000000000000001, 0.0, 0.0, 0.0 },
              { 4.000000000000012, -4.000000000000001, 0.0, 0.0, 0.0 },
              { 4.00000000000003, -4.000000000000018, 0.0, 0.0, 0.0 } },
            { { -4.000000000000017, -4.000000000000001, 7.999999999999956, 8.000000000000039, -8.000000000000004 },
              { -4.00000000000001, -3.999999999999993, 7.99999999999994, 8.000000000000023, -7.999999999999988 },
              { 4.00000000000001, 3.999999999999993, -7.99999999999994, -8.000000000000023, 7.999999999999988 },
              { 4.000000000000027, 4.000000000000011, -7.999999999999975, -8.000000000000057, 8.000000000000023 } } },
          { { { 4.000000000000001, 4.000000000000008, 7.999999999999941, -7.999999999999964, -8.000000000000004 },
              { 3.999999999999993, 4.000000000000001, 7.999999999999925, -7.999999999999948, -7.999999999999988 },
              { -3.999999999999993, -4.000000000000001, -7.999999999999925, 7.999999999999948, 7.999999999999988 },
              { -4.000000000000011, -4.000000000000018, -7.99999999999996, 7.999999999999983, 8.000000000000021 } },
            { { -4.00000000000002, 4.000000000000008, 0.0, 0.0, 0.0 },
              { -4.000000000000012, 4.000000000000001, 0.0, 0.0, 0.0 },
              { 4.000000000000012, -4.000000000000001, 0.0, 0.0, 0.0 },
              { 4.00000000000003, -4.000000000000018, 0.0, 0.0, 0.0 } },
            { { -4.000000000000017, -4.000000000000001, 7.999999999999956, 8.000000000000039, -8.000000000000004 },
              { -4.00000000000001, -3.999999999999993, 7.99999999999994, 8.000000000000023, -7.999999999999988 },
              { 4.00000000000001, 3.999999999999993, -7.99999999999994, -8.000000000000023, 7.999999999999988 },
              { 4.000000000000027, 4.000000000000011, -7.999999999999975, -8.000000000000057, 8.000000000000023 } } },
          { { { 4.000000000000001, 4.000000000000008, 7.999999999999941, -7.999999999999964, -8.000000000000004 },
              { 3.999999999999993, 4.000000000000001, 7.999999999999925, -7.999999999999948, -7.999999999999988 },
              { -3.999999999999993, -4.000000000000001, -7.999999999999925, 7.999999999999948, 7.999999999999988 },
              { -4.000000000000011, -4.000000000000018, -7.99999999999996, 7.999999999999983, 8.000000000000021 } },
            { { -4.00000000000002, 4.000000000000008, 0.0, 0.0, 0.0 },
              { -4.000000000000012, 4.000000000000001, 0.0, 0.0, 0.0 },
              { 4.000000000000012, -4.000000000000001, 0.0, 0.0, 0.0 },
              { 4.00000000000003, -4.000000000000018, 0.0, 0.0, 0.0 } },
            { { -4.000000000000017, -4.000000000000001, 7.999999999999956, 8.000000000000039, -8.000000000000004 },
              { -4.00000000000001, -3.999999999999993, 7.99999999999994, 8.000000000000023, -7.999999999999988 },
              { 4.00000000000001, 3.999999999999993, -7.99999999999994, -8.000000000000023, 7.999999999999988 },
              { 4.000000000000027, 4.000000000000011, -7.999999999999975, -8.000000000000057, 8.000000000000023 } } } };
    alignas(32) static const double PI6[3][3][5][5] =
        { { { { 1.0, 1.0, 1.999999999999985, -1.99999999999999, -2.0 },
              { 1.0, 2.333333333333339, 0.6666666666666534, -3.33333333333333, -0.6666666666666679 },
              { 1.999999999999985, 0.6666666666666534, 5.333333333333274, -2.666666666666615, -5.333333333333306 },
              { -1.99999999999999, -3.33333333333333, -2.666666666666615, 5.333333333333298, 2.666666666666645 },
              { -2.0, -0.6666666666666679, -5.333333333333306, 2.666666666666645, 5.333333333333337 } },
            { { -1.0, 1.0, 0.0, 0.0, 0.0 },
              { 0.333333333333327, 2.333333333333339, 0.0, -2.66666666666666, 0.0 },
              { -3.333333333333328, 0.6666666666666534, 0.0, 2.666666666666682, 0.0 },
              { 0.6666666666666657, -3.33333333333333, 0.0, 2.666666666666654, 0.0 },
              { 3.333333333333344, -0.6666666666666679, 0.0, -2.666666666666685, 0.0 } },
            { { -1.0, -1.0, 1.999999999999988, 2.000000000000009, -2.0 },
              { 0.3333333333333277, -1.0, 3.333333333333324, 0.6666666666666807, -3.333333333333339 },
              { -3.333333333333326, -1.999999999999985, 2.666666666666615, 5.333333333333319, -2.666666666666635 },
              { 0.6666666666666643, 1.99999999999999, -5.333333333333291, -2.666666666666666, 5.333333333333317 },
              { 3.333333333333343, 2.0, -2.666666666666644, -5.333333333333351, 2.666666666666665 } } },
          { { { -1.0, 0.333333333333327, -3.333333333333328, 0.6666666666666657, 3.333333333333344 },
              { 1.0, 2.333333333333339, 0.6666666666666534, -3.33333333333333, -0.6666666666666679 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 },
              { 0.0, -2.66666666666666, 2.666666666666682, 2.666666666666654, -2.666666666666685 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 } },
            { { 2.333333333333342, 0.333333333333327, 0.0, -2.666666666666672, 0.0 },
              { 0.333333333333327, 2.333333333333339, 0.0, -2.66666666666666, 0.0 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 },
              { -2.666666666666672, -2.66666666666666, 0.0, 5.33333333333333, 0.0 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 } },
            { { 2.333333333333341, 1.0, -0.6666666666666674, -3.333333333333349, 0.6666666666666766 },
              { 0.3333333333333277, -1.0, 3.333333333333324, 0.6666666666666807, -3.333333333333339 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 },
              { -2.666666666666672, 0.0, -2.666666666666647, 2.666666666666677, 2.666666666666652 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 } } },
          { { { -1.0, 0.3333333333333277, -3.333333333333326, 0.6666666666666643, 3.333333333333343 },
              { -1.0, -1.0, -1.999999999999985, 1.99999999999999, 2.0 },
              { 1.999999999999988, 3.333333333333324, 2.666666666666615, -5.333333333333291, -2.666666666666644 },
              { 2.000000000000009, 0.6666666666666807, 5.333333333333319, -2.666666666666666, -5.333333333333351 },
              { -2.0, -3.333333333333339, -2.666666666666635, 5.333333333333317, 2.666666666666665 } },
            { { 2.333333333333341, 0.3333333333333277, 0.0, -2.666666666666672, 0.0 },
              { 1.0, -1.0, 0.0, 0.0, 0.0 },
              { -0.6666666666666674, 3.333333333333324, 0.0, -2.666666666666647, 0.0 },
              { -3.333333333333349, 0.6666666666666807, 0.0, 2.666666666666677, 0.0 },
              { 0.6666666666666766, -3.333333333333339, 0.0, 2.666666666666652, 0.0 } },
            { { 2.33333333333334, 1.0, -0.666666666666666, -3.333333333333348, 0.6666666666666752 },
              { 1.0, 1.0, -1.999999999999988, -2.000000000000009, 2.0 },
              { -0.666666666666666, -1.999999999999988, 5.333333333333282, 2.666666666666666, -5.333333333333309 },
              { -3.333333333333348, -2.000000000000009, 2.666666666666666, 5.333333333333365, -2.666666666666687 },
              { 0.6666666666666752, 2.0, -5.333333333333309, -2.666666666666687, 5.333333333333336 } } } };
    // Unstructured piecewise computations
    const double J_r0_c0 = coordinate_dofs_0[0] * FE7_C0_D01_F_Q2[0][0][0] + coordinate_dofs_0[2] * FE7_C0_D01_F_Q2[0][0][1];
    const double J_r0_c3 = coordinate_dofs_0[1] * FE7_C0_D01_F_Q2[0][0][0] + coordinate_dofs_0[5] * FE7_C0_D01_F_Q2[0][0][1];
    const double J_r0_c1 = coordinate_dofs_0[0] * FE7_C0_D01_F_Q2[0][0][0] + coordinate_dofs_0[4] * FE7_C0_D01_F_Q2[0][0][1];
    const double J_r0_c2 = coordinate_dofs_0[1] * FE7_C0_D01_F_Q2[0][0][0] + coordinate_dofs_0[3] * FE7_C0_D01_F_Q2[0][0][1];
    const double J_r1_c0 = coordinate_dofs_1[0] * FE7_C0_D01_F_Q2[0][0][0] + coordinate_dofs_1[2] * FE7_C0_D01_F_Q2[0][0][1];
    const double J_r1_c3 = coordinate_dofs_1[1] * FE7_C0_D01_F_Q2[0][0][0] + coordinate_dofs_1[5] * FE7_C0_D01_F_Q2[0][0][1];
    const double J_r1_c1 = coordinate_dofs_1[0] * FE7_C0_D01_F_Q2[0][0][0] + coordinate_dofs_1[4] * FE7_C0_D01_F_Q2[0][0][1];
    const double J_r1_c2 = coordinate_dofs_1[1] * FE7_C0_D01_F_Q2[0][0][0] + coordinate_dofs_1[3] * FE7_C0_D01_F_Q2[0][0][1];
    alignas(32) double sp[232];
    sp[0] = J_r0_c0 * J_r0_c3;
    sp[1] = J_r0_c1 * J_r0_c2;
    sp[2] = sp[0] + -1 * sp[1];
    sp[3] = J_r0_c0 / sp[2];
    sp[4] = -1 * J_r0_c1 / sp[2];
    sp[5] = triangle_reference_facet_normals[facet_0][1] * sp[3];
    sp[6] = triangle_reference_facet_normals[facet_0][0] * sp[4];
    sp[7] = sp[5] + sp[6];
    sp[8] = sp[7] * sp[7];
    sp[9] = J_r0_c3 / sp[2];
    sp[10] = triangle_reference_facet_normals[facet_0][0] * sp[9];
    sp[11] = -1 * J_r0_c2 / sp[2];
    sp[12] = triangle_reference_facet_normals[facet_0][1] * sp[11];
    sp[13] = sp[10] + sp[12];
    sp[14] = sp[13] * sp[13];
    sp[15] = sp[8] + sp[14];
    sp[16] = std::sqrt(sp[15]);
    sp[17] = sp[7] / sp[16];
    sp[18] = sp[3] * sp[17];
    sp[19] = sp[4] * sp[17];
    sp[20] = sp[13] / sp[16];
    sp[21] = sp[11] * sp[20];
    sp[22] = sp[9] * sp[20];
    sp[23] = sp[18] + sp[21];
    sp[24] = sp[19] + sp[22];
    sp[25] = J_r1_c0 * J_r1_c3;
    sp[26] = J_r1_c1 * J_r1_c2;
    sp[27] = sp[25] + -1 * sp[26];
    sp[28] = J_r1_c0 / sp[27];
    sp[29] = -1 * J_r1_c1 / sp[27];
    sp[30] = triangle_reference_facet_normals[facet_1][1] * sp[28];
    sp[31] = triangle_reference_facet_normals[facet_1][0] * sp[29];
    sp[32] = sp[30] + sp[31];
    sp[33] = sp[32] * sp[32];
    sp[34] = J_r1_c3 / sp[27];
    sp[35] = triangle_reference_facet_normals[facet_1][0] * sp[34];
    sp[36] = -1 * J_r1_c2 / sp[27];
    sp[37] = triangle_reference_facet_normals[facet_1][1] * sp[36];
    sp[38] = sp[35] + sp[37];
    sp[39] = sp[38] * sp[38];
    sp[40] = sp[33] + sp[39];
    sp[41] = std::sqrt(sp[40]);
    sp[42] = sp[32] / sp[41];
    sp[43] = sp[28] * sp[42];
    sp[44] = sp[29] * sp[42];
    sp[45] = sp[38] / sp[41];
    sp[46] = sp[36] * sp[45];
    sp[47] = sp[34] * sp[45];
    sp[48] = sp[43] + sp[46];
    sp[49] = sp[44] + sp[47];
    sp[50] = sp[3] * sp[3];
    sp[51] = sp[3] * sp[4];
    sp[52] = sp[4] * sp[4];
    sp[53] = sp[51] + sp[51];
    sp[54] = sp[9] * sp[11];
    sp[55] = sp[9] * sp[9];
    sp[56] = sp[11] * sp[11];
    sp[57] = sp[54] + sp[54];
    sp[58] = sp[50] + sp[56];
    sp[59] = sp[53] + sp[57];
    sp[60] = sp[55] + sp[52];
    sp[61] = sp[28] * sp[28];
    sp[62] = sp[28] * sp[29];
    sp[63] = sp[29] * sp[29];
    sp[64] = sp[62] + sp[62];
    sp[65] = sp[34] * sp[36];
    sp[66] = sp[34] * sp[34];
    sp[67] = sp[36] * sp[36];
    sp[68] = sp[65] + sp[65];
    sp[69] = sp[61] + sp[67];
    sp[70] = sp[64] + sp[68];
    sp[71] = sp[66] + sp[63];
    sp[72] = sp[23] * (0.5 * sp[58]);
    sp[73] = sp[23] * (0.5 * sp[59]);
    sp[74] = sp[23] * (0.5 * sp[60]);
    sp[75] = sp[23] * (0.5 * sp[69]);
    sp[76] = sp[23] * (0.5 * sp[70]);
    sp[77] = sp[23] * (0.5 * sp[71]);
    sp[78] = sp[24] * (0.5 * sp[58]);
    sp[79] = sp[24] * (0.5 * sp[59]);
    sp[80] = sp[24] * (0.5 * sp[60]);
    sp[81] = sp[24] * (0.5 * sp[69]);
    sp[82] = sp[24] * (0.5 * sp[70]);
    sp[83] = sp[24] * (0.5 * sp[71]);
    sp[84] = sp[48] * (0.5 * sp[58]);
    sp[85] = sp[48] * (0.5 * sp[59]);
    sp[86] = sp[48] * (0.5 * sp[60]);
    sp[87] = sp[48] * (0.5 * sp[69]);
    sp[88] = sp[48] * (0.5 * sp[70]);
    sp[89] = sp[48] * (0.5 * sp[71]);
    sp[90] = sp[49] * (0.5 * sp[58]);
    sp[91] = sp[49] * (0.5 * sp[59]);
    sp[92] = sp[49] * (0.5 * sp[60]);
    sp[93] = sp[49] * (0.5 * sp[69]);
    sp[94] = sp[49] * (0.5 * sp[70]);
    sp[95] = sp[49] * (0.5 * sp[71]);
    sp[96] = sp[23] * sp[23];
    sp[97] = sp[24] * sp[23];
    sp[98] = sp[23] * sp[48];
    sp[99] = sp[23] * sp[49];
    sp[100] = sp[24] * sp[24];
    sp[101] = sp[24] * sp[48];
    sp[102] = sp[24] * sp[49];
    sp[103] = sp[48] * sp[48];
    sp[104] = sp[49] * sp[48];
    sp[105] = sp[49] * sp[49];
    sp[106] = J_r0_c0 * triangle_reference_edge_vectors[0][0];
    sp[107] = J_r0_c1 * triangle_reference_edge_vectors[0][1];
    sp[108] = sp[106] + sp[107];
    sp[109] = sp[108] * sp[108];
    sp[110] = triangle_reference_edge_vectors[0][0] * J_r0_c2;
    sp[111] = triangle_reference_edge_vectors[0][1] * J_r0_c3;
    sp[112] = sp[110] + sp[111];
    sp[113] = sp[112] * sp[112];
    sp[114] = sp[109] + sp[113];
    sp[115] = std::sqrt(sp[114]);
    sp[116] = J_r0_c0 * triangle_reference_edge_vectors[1][0];
    sp[117] = J_r0_c1 * triangle_reference_edge_vectors[1][1];
    sp[118] = sp[116] + sp[117];
    sp[119] = sp[118] * sp[118];
    sp[120] = J_r0_c2 * triangle_reference_edge_vectors[1][0];
    sp[121] = J_r0_c3 * triangle_reference_edge_vectors[1][1];
    sp[122] = sp[120] + sp[121];
    sp[123] = sp[122] * sp[122];
    sp[124] = sp[119] + sp[123];
    sp[125] = std::sqrt(sp[124]);
    sp[126] = sp[115] * sp[125];
    sp[127] = J_r0_c0 * triangle_reference_edge_vectors[2][0];
    sp[128] = J_r0_c1 * triangle_reference_edge_vectors[2][1];
    sp[129] = sp[127] + sp[128];
    sp[130] = sp[129] * sp[129];
    sp[131] = J_r0_c2 * triangle_reference_edge_vectors[2][0];
    sp[132] = J_r0_c3 * triangle_reference_edge_vectors[2][1];
    sp[133] = sp[131] + sp[132];
    sp[134] = sp[133] * sp[133];
    sp[135] = sp[130] + sp[134];
    sp[136] = std::sqrt(sp[135]);
    sp[137] = sp[126] * sp[136];
    sp[138] = triangle_reference_cell_volume * sp[2];
    sp[139] = std::abs(sp[138]);
    sp[140] = sp[137] / (4.0 * sp[139]);
    sp[141] = J_r1_c0 * triangle_reference_edge_vectors[0][0];
    sp[142] = J_r1_c1 * triangle_reference_edge_vectors[0][1];
    sp[143] = sp[141] + sp[142];
    sp[144] = sp[143] * sp[143];
    sp[145] = triangle_reference_edge_vectors[0][0] * J_r1_c2;
    sp[146] = triangle_reference_edge_vectors[0][1] * J_r1_c3;
    sp[147] = sp[145] + sp[146];
    sp[148] = sp[147] * sp[147];
    sp[149] = sp[144] + sp[148];
    sp[150] = std::sqrt(sp[149]);
    sp[151] = J_r1_c0 * triangle_reference_edge_vectors[1][0];
    sp[152] = J_r1_c1 * triangle_reference_edge_vectors[1][1];
    sp[153] = sp[151] + sp[152];
    sp[154] = sp[153] * sp[153];
    sp[155] = J_r1_c2 * triangle_reference_edge_vectors[1][0];
    sp[156] = J_r1_c3 * triangle_reference_edge_vectors[1][1];
    sp[157] = sp[155] + sp[156];
    sp[158] = sp[157] * sp[157];
    sp[159] = sp[154] + sp[158];
    sp[160] = std::sqrt(sp[159]);
    sp[161] = sp[150] * sp[160];
    sp[162] = J_r1_c0 * triangle_reference_edge_vectors[2][0];
    sp[163] = J_r1_c1 * triangle_reference_edge_vectors[2][1];
    sp[164] = sp[162] + sp[163];
    sp[165] = sp[164] * sp[164];
    sp[166] = J_r1_c2 * triangle_reference_edge_vectors[2][0];
    sp[167] = J_r1_c3 * triangle_reference_edge_vectors[2][1];
    sp[168] = sp[166] + sp[167];
    sp[169] = sp[168] * sp[168];
    sp[170] = sp[165] + sp[169];
    sp[171] = std::sqrt(sp[170]);
    sp[172] = sp[161] * sp[171];
    sp[173] = triangle_reference_cell_volume * sp[27];
    sp[174] = std::abs(sp[173]);
    sp[175] = sp[172] / (4.0 * sp[174]);
    sp[176] = 2.0 * sp[140] + 2.0 * sp[175];
    sp[177] = w[0][0] / (sp[176] / 2);
    sp[178] = sp[96] * sp[177];
    sp[179] = sp[97] * sp[177];
    sp[180] = sp[98] * sp[177];
    sp[181] = sp[99] * sp[177];
    sp[182] = sp[100] * sp[177];
    sp[183] = sp[101] * sp[177];
    sp[184] = sp[102] * sp[177];
    sp[185] = sp[103] * sp[177];
    sp[186] = sp[104] * sp[177];
    sp[187] = sp[105] * sp[177];
    sp[188] = J_r0_c0 * triangle_reference_facet_jacobian[facet_0][0][0];
    sp[189] = J_r0_c1 * triangle_reference_facet_jacobian[facet_0][1][0];
    sp[190] = sp[188] + sp[189];
    sp[191] = sp[190] * sp[190];
    sp[192] = triangle_reference_facet_jacobian[facet_0][0][0] * J_r0_c2;
    sp[193] = triangle_reference_facet_jacobian[facet_0][1][0] * J_r0_c3;
    sp[194] = sp[192] + sp[193];
    sp[195] = sp[194] * sp[194];
    sp[196] = sp[191] + sp[195];
    sp[197] = std::sqrt(sp[196]);
    sp[198] = -1 * sp[72] * sp[197];
    sp[199] = -1 * sp[73] * sp[197];
    sp[200] = -1 * sp[74] * sp[197];
    sp[201] = -1 * sp[75] * sp[197];
    sp[202] = -1 * sp[76] * sp[197];
    sp[203] = -1 * sp[77] * sp[197];
    sp[204] = sp[178] * sp[197];
    sp[205] = sp[179] * sp[197];
    sp[206] = sp[180] * sp[197];
    sp[207] = sp[181] * sp[197];
    sp[208] = -1 * sp[78] * sp[197];
    sp[209] = -1 * sp[79] * sp[197];
    sp[210] = -1 * sp[80] * sp[197];
    sp[211] = -1 * sp[81] * sp[197];
    sp[212] = -1 * sp[82] * sp[197];
    sp[213] = -1 * sp[83] * sp[197];
    sp[214] = sp[182] * sp[197];
    sp[215] = sp[183] * sp[197];
    sp[216] = sp[184] * sp[197];
    sp[217] = -1 * sp[84] * sp[197];
    sp[218] = -1 * sp[85] * sp[197];
    sp[219] = -1 * sp[86] * sp[197];
    sp[220] = -1 * sp[87] * sp[197];
    sp[221] = -1 * sp[88] * sp[197];
    sp[222] = -1 * sp[89] * sp[197];
    sp[223] = sp[185] * sp[197];
    sp[224] = sp[186] * sp[197];
    sp[225] = -1 * sp[90] * sp[197];
    sp[226] = -1 * sp[91] * sp[197];
    sp[227] = -1 * sp[92] * sp[197];
    sp[228] = -1 * sp[93] * sp[197];
    sp[229] = -1 * sp[94] * sp[197];
    sp[230] = -1 * sp[95] * sp[197];
    sp[231] = sp[187] * sp[197];
    A[0] = sp[214] * PI0[facet_0][facet_0][0][0] + sp[210] * PI1[facet_0][0][0][0] + sp[205] * PI3[facet_0][facet_0][0][0] + sp[208] * PI1[facet_0][0][0][0] + sp[209] * PI2[facet_0][0][0][0] + sp[210] * PI1[facet_0][0][0][0] + sp[200] * PI4[0][facet_0][0][0] + sp[205] * PI3[facet_0][facet_0][0][0] + sp[200] * PI4[0][facet_0][0][0] + sp[204] * PI6[facet_0][facet_0][0][0] + sp[198] * PI4[0][facet_0][0][0] + sp[199] * PI5[0][facet_0][0][0] + sp[208] * PI1[facet_0][0][0][0] + sp[198] * PI4[0][facet_0][0][0] + sp[209] * PI2[facet_0][0][0][0] + sp[199] * PI5[0][facet_0][0][0];
    A[1] = sp[214] * PI0[facet_0][facet_0][0][1] + sp[210] * PI1[facet_0][0][0][1] + sp[210] * PI1[facet_0][0][1][0] + sp[205] * PI3[facet_0][facet_0][1][0] + sp[200] * PI4[0][facet_0][1][0] + sp[208] * PI1[facet_0][0][1][0] + sp[209] * PI2[facet_0][0][1][0];
    A[2] = sp[205] * PI3[facet_0][facet_0][0][1] + sp[208] * PI1[facet_0][0][0][1] + sp[200] * PI4[0][facet_0][0][1] + sp[204] * PI6[facet_0][facet_0][0][1] + sp[198] * PI4[0][facet_0][1][0] + sp[198] * PI4[0][facet_0][0][1] + sp[199] * PI5[0][facet_0][0][1];
    A[3] = sp[214] * PI0[facet_0][facet_0][0][2] + sp[205] * PI3[facet_0][facet_0][0][2] + sp[209] * PI2[facet_0][0][0][1] + sp[210] * PI1[facet_0][0][2][0] + sp[200] * PI4[0][facet_0][0][2] + sp[205] * PI3[facet_0][facet_0][2][0] + sp[204] * PI6[facet_0][facet_0][0][2] + sp[199] * PI5[0][facet_0][1][0] + sp[208] * PI1[facet_0][0][2][0] + sp[198] * PI4[0][facet_0][0][2] + sp[209] * PI2[facet_0][0][2][0] + sp[199] * PI5[0][facet_0][0][2];
    A[4] = sp[214] * PI0[facet_0][facet_0][0][3] + sp[205] * PI3[facet_0][facet_0][0][3] + sp[208] * PI1[facet_0][0][0][2] + sp[209] * PI2[facet_0][0][0][2] + sp[210] * PI1[facet_0][0][3][0] + sp[200] * PI4[0][facet_0][0][3] + sp[205] * PI3[facet_0][facet_0][3][0] + sp[204] * PI6[facet_0][facet_0][0][3] + sp[198] * PI4[0][facet_0][2][0] + sp[199] * PI5[0][facet_0][2][0] + sp[208] * PI1[facet_0][0][3][0] + sp[198] * PI4[0][facet_0][0][3] + sp[209] * PI2[facet_0][0][3][0] + sp[199] * PI5[0][facet_0][0][3];
    A[5] = sp[214] * PI0[facet_0][facet_0][0][4] + sp[210] * PI1[facet_0][0][0][2] + sp[205] * PI3[facet_0][facet_0][0][4] + sp[209] * PI2[facet_0][0][0][3] + sp[210] * PI1[facet_0][0][4][0] + sp[200] * PI4[0][facet_0][0][4] + sp[205] * PI3[facet_0][facet_0][4][0] + sp[200] * PI4[0][facet_0][2][0] + sp[204] * PI6[facet_0][facet_0][0][4] + sp[199] * PI5[0][facet_0][3][0] + sp[208] * PI1[facet_0][0][4][0] + sp[198] * PI4[0][facet_0][0][4] + sp[209] * PI2[facet_0][0][4][0] + sp[199] * PI5[0][facet_0][0][4];
    A[6] = sp[216] * PI0[facet_0][facet_1][0][0] + sp[213] * PI1[facet_0][0][0][0] + sp[215] * PI3[facet_0][facet_1][0][0] + sp[211] * PI1[facet_0][0][0][0] + sp[212] * PI2[facet_0][0][0][0] + sp[227] * PI1[facet_1][0][0][0] + sp[219] * PI4[0][facet_1][0][0] + sp[207] * PI3[facet_1][facet_0][0][0] + sp[203] * PI4[0][facet_0][0][0] + sp[206] * PI6[facet_0][facet_1][0][0] + sp[201] * PI4[0][facet_0][0][0] + sp[202] * PI5[0][facet_0][0][0] + sp[225] * PI1[facet_1][0][0][0] + sp[217] * PI4[0][facet_1][0][0] + sp[226] * PI2[facet_1][0][0][0] + sp[218] * PI5[0][facet_1][0][0];
    A[7] = sp[216] * PI0[facet_0][facet_1][0][1] + sp[213] * PI1[facet_0][0][0][1] + sp[227] * PI1[facet_1][0][1][0] + sp[207] * PI3[facet_1][facet_0][1][0] + sp[203] * PI4[0][facet_0][1][0] + sp[225] * PI1[facet_1][0][1][0] + sp[226] * PI2[facet_1][0][1][0];
    A[8] = sp[215] * PI3[facet_0][facet_1][0][1] + sp[211] * PI1[facet_0][0][0][1] + sp[219] * PI4[0][facet_1][0][1] + sp[206] * PI6[facet_0][facet_1][0][1] + sp[201] * PI4[0][facet_0][1][0] + sp[217] * PI4[0][facet_1][0][1] + sp[218] * PI5[0][facet_1][0][1];
    A[9] = sp[216] * PI0[facet_0][facet_1][0][2] + sp[215] * PI3[facet_0][facet_1][0][2] + sp[212] * PI2[facet_0][0][0][1] + sp[227] * PI1[facet_1][0][2][0] + sp[219] * PI4[0][facet_1][0][2] + sp[207] * PI3[facet_1][facet_0][2][0] + sp[206] * PI6[facet_0][facet_1][0][2] + sp[202] * PI5[0][facet_0][1][0] + sp[225] * PI1[facet_1][0][2][0] + sp[217] * PI4[0][facet_1][0][2] + sp[226] * PI2[facet_1][0][2][0] + sp[218] * PI5[0][facet_1][0][2];
    A[10] = sp[216] * PI0[facet_0][facet_1][0][3] + sp[215] * PI3[facet_0][facet_1][0][3] + sp[211] * PI1[facet_0][0][0][2] + sp[212] * PI2[facet_0][0][0][2] + sp[227] * PI1[facet_1][0][3][0] + sp[219] * PI4[0][facet_1][0][3] + sp[207] * PI3[facet_1][facet_0][3][0] + sp[206] * PI6[facet_0][facet_1][0][3] + sp[201] * PI4[0][facet_0][2][0] + sp[202] * PI5[0][facet_0][2][0] + sp[225] * PI1[facet_1][0][3][0] + sp[217] * PI4[0][facet_1][0][3] + sp[226] * PI2[facet_1][0][3][0] + sp[218] * PI5[0][facet_1][0][3];
    A[11] = sp[216] * PI0[facet_0][facet_1][0][4] + sp[213] * PI1[facet_0][0][0][2] + sp[215] * PI3[facet_0][facet_1][0][4] + sp[212] * PI2[facet_0][0][0][3] + sp[227] * PI1[facet_1][0][4][0] + sp[219] * PI4[0][facet_1][0][4] + sp[207] * PI3[facet_1][facet_0][4][0] + sp[203] * PI4[0][facet_0][2][0] + sp[206] * PI6[facet_0][facet_1][0][4] + sp[202] * PI5[0][facet_0][3][0] + sp[225] * PI1[facet_1][0][4][0] + sp[217] * PI4[0][facet_1][0][4] + sp[226] * PI2[facet_1][0][4][0] + sp[218] * PI5[0][facet_1][0][4];
    A[12] = sp[214] * PI0[facet_0][facet_0][1][0] + sp[210] * PI1[facet_0][0][1][0] + sp[205] * PI3[facet_0][facet_0][1][0] + sp[208] * PI1[facet_0][0][1][0] + sp[209] * PI2[facet_0][0][1][0] + sp[210] * PI1[facet_0][0][0][1] + sp[200] * PI4[0][facet_0][1][0];
    A[13] = sp[214] * PI0[facet_0][facet_0][1][1] + sp[210] * PI1[facet_0][0][1][1] + sp[210] * PI1[facet_0][0][1][1];
    A[14] = sp[205] * PI3[facet_0][facet_0][1][1] + sp[208] * PI1[facet_0][0][1][1] + sp[200] * PI4[0][facet_0][1][1];
    A[15] = sp[214] * PI0[facet_0][facet_0][1][2] + sp[205] * PI3[facet_0][facet_0][1][2] + sp[209] * PI2[facet_0][0][1][1] + sp[210] * PI1[facet_0][0][2][1] + sp[200] * PI4[0][facet_0][1][2];
    A[16] = sp[214] * PI0[facet_0][facet_0][1][3] + sp[205] * PI3[facet_0][facet_0][1][3] + sp[208] * PI1[facet_0][0][1][2] + sp[209] * PI2[facet_0][0][1][2] + sp[210] * PI1[facet_0][0][3][1] + sp[200] * PI4[0][facet_0][1][3];
    A[17] = sp[214] * PI0[facet_0][facet_0][1][4] + sp[210] * PI1[facet_0][0][1][2] + sp[205] * PI3[facet_0][facet_0][1][4] + sp[209] * PI2[facet_0][0][1][3] + sp[210] * PI1[facet_0][0][4][1] + sp[200] * PI4[0][facet_0][1][4];
    A[18] = sp[216] * PI0[facet_0][facet_1][1][0] + sp[213] * PI1[facet_0][0][1][0] + sp[215] * PI3[facet_0][facet_1][1][0] + sp[211] * PI1[facet_0][0][1][0] + sp[212] * PI2[facet_0][0][1][0] + sp[227] * PI1[facet_1][0][0][1] + sp[219] * PI4[0][facet_1][1][0];
    A[19] = sp[216] * PI0[facet_0][facet_1][1][1] + sp[213] * PI1[facet_0][0][1][1] + sp[227] * PI1[facet_1][0][1][1];
    A[20] = sp[215] * PI3[facet_0][facet_1][1][1] + sp[211] * PI1[facet_0][0][1][1] + sp[219] * PI4[0][facet_1][1][1];
    A[21] = sp[216] * PI0[facet_0][facet_1][1][2] + sp[215] * PI3[facet_0][facet_1][1][2] + sp[212] * PI2[facet_0][0][1][1] + sp[227] * PI1[facet_1][0][2][1] + sp[219] * PI4[0][facet_1][1][2];
    A[22] = sp[216] * PI0[facet_0][facet_1][1][3] + sp[215] * PI3[facet_0][facet_1][1][3] + sp[211] * PI1[facet_0][0][1][2] + sp[212] * PI2[facet_0][0][1][2] + sp[227] * PI1[facet_1][0][3][1] + sp[219] * PI4[0][facet_1][1][3];
    A[23] = sp[216] * PI0[facet_0][facet_1][1][4] + sp[213] * PI1[facet_0][0][1][2] + sp[215] * PI3[facet_0][facet_1][1][4] + sp[212] * PI2[facet_0][0][1][3] + sp[227] * PI1[facet_1][0][4][1] + sp[219] * PI4[0][facet_1][1][4];
    A[24] = sp[205] * PI3[facet_0][facet_0][0][1] + sp[200] * PI4[0][facet_0][0][1] + sp[204] * PI6[facet_0][facet_0][1][0] + sp[198] * PI4[0][facet_0][0][1] + sp[199] * PI5[0][facet_0][0][1] + sp[208] * PI1[facet_0][0][0][1] + sp[198] * PI4[0][facet_0][1][0];
    A[25] = sp[205] * PI3[facet_0][facet_0][1][1] + sp[200] * PI4[0][facet_0][1][1] + sp[208] * PI1[facet_0][0][1][1];
    A[26] = sp[204] * PI6[facet_0][facet_0][1][1] + sp[198] * PI4[0][facet_0][1][1] + sp[198] * PI4[0][facet_0][1][1];
    A[27] = sp[205] * PI3[facet_0][facet_0][2][1] + sp[204] * PI6[facet_0][facet_0][1][2] + sp[199] * PI5[0][facet_0][1][1] + sp[208] * PI1[facet_0][0][2][1] + sp[198] * PI4[0][facet_0][1][2];
    A[28] = sp[205] * PI3[facet_0][facet_0][3][1] + sp[204] * PI6[facet_0][facet_0][1][3] + sp[198] * PI4[0][facet_0][2][1] + sp[199] * PI5[0][facet_0][2][1] + sp[208] * PI1[facet_0][0][3][1] + sp[198] * PI4[0][facet_0][1][3];
    A[29] = sp[205] * PI3[facet_0][facet_0][4][1] + sp[200] * PI4[0][facet_0][2][1] + sp[204] * PI6[facet_0][facet_0][1][4] + sp[199] * PI5[0][facet_0][3][1] + sp[208] * PI1[facet_0][0][4][1] + sp[198] * PI4[0][facet_0][1][4];
    A[30] = sp[207] * PI3[facet_1][facet_0][0][1] + sp[203] * PI4[0][facet_0][0][1] + sp[206] * PI6[facet_0][facet_1][1][0] + sp[201] * PI4[0][facet_0][0][1] + sp[202] * PI5[0][facet_0][0][1] + sp[225] * PI1[facet_1][0][0][1] + sp[217] * PI4[0][facet_1][1][0];
    A[31] = sp[207] * PI3[facet_1][facet_0][1][1] + sp[203] * PI4[0][facet_0][1][1] + sp[225] * PI1[facet_1][0][1][1];
    A[32] = sp[206] * PI6[facet_0][facet_1][1][1] + sp[201] * PI4[0][facet_0][1][1] + sp[217] * PI4[0][facet_1][1][1];
    A[33] = sp[207] * PI3[facet_1][facet_0][2][1] + sp[206] * PI6[facet_0][facet_1][1][2] + sp[202] * PI5[0][facet_0][1][1] + sp[225] * PI1[facet_1][0][2][1] + sp[217] * PI4[0][facet_1][1][2];
    A[34] = sp[207] * PI3[facet_1][facet_0][3][1] + sp[206] * PI6[facet_0][facet_1][1][3] + sp[201] * PI4[0][facet_0][2][1] + sp[202] * PI5[0][facet_0][2][1] + sp[225] * PI1[facet_1][0][3][1] + sp[217] * PI4[0][facet_1][1][3];
    A[35] = sp[207] * PI3[facet_1][facet_0][4][1] + sp[203] * PI4[0][facet_0][2][1] + sp[206] * PI6[facet_0][facet_1][1][4] + sp[202] * PI5[0][facet_0][3][1] + sp[225] * PI1[facet_1][0][4][1] + sp[217] * PI4[0][facet_1][1][4];
    A[36] = sp[214] * PI0[facet_0][facet_0][2][0] + sp[210] * PI1[facet_0][0][2][0] + sp[205] * PI3[facet_0][facet_0][2][0] + sp[208] * PI1[facet_0][0][2][0] + sp[209] * PI2[facet_0][0][2][0] + sp[205] * PI3[facet_0][facet_0][0][2] + sp[200] * PI4[0][facet_0][0][2] + sp[204] * PI6[facet_0][facet_0][2][0] + sp[198] * PI4[0][facet_0][0][2] + sp[199] * PI5[0][facet_0][0][2] + sp[209] * PI2[facet_0][0][0][1] + sp[199] * PI5[0][facet_0][1][0];
    A[37] = sp[214] * PI0[facet_0][facet_0][2][1] + sp[210] * PI1[facet_0][0][2][1] + sp[205] * PI3[facet_0][facet_0][1][2] + sp[200] * PI4[0][facet_0][1][2] + sp[209] * PI2[facet_0][0][1][1];
    A[38] = sp[205] * PI3[facet_0][facet_0][2][1] + sp[208] * PI1[facet_0][0][2][1] + sp[204] * PI6[facet_0][facet_0][2][1] + sp[198] * PI4[0][facet_0][1][2] + sp[199] * PI5[0][facet_0][1][1];
    A[39] = sp[214] * PI0[facet_0][facet_0][2][2] + sp[205] * PI3[facet_0][facet_0][2][2] + sp[209] * PI2[facet_0][0][2][1] + sp[205] * PI3[facet_0][facet_0][2][2] + sp[204] * PI6[facet_0][facet_0][2][2] + sp[199] * PI5[0][facet_0][1][2] + sp[209] * PI2[facet_0][0][2][1] + sp[199] * PI5[0][facet_0][1][2];
    A[40] = sp[214] * PI0[facet_0][facet_0][2][3] + sp[205] * PI3[facet_0][facet_0][2][3] + sp[208] * PI1[facet_0][0][2][2] + sp[209] * PI2[facet_0][0][2][2] + sp[205] * PI3[facet_0][facet_0][3][2] + sp[204] * PI6[facet_0][facet_0][2][3] + sp[198] * PI4[0][facet_0][2][2] + sp[199] * PI5[0][facet_0][2][2] + sp[209] * PI2[facet_0][0][3][1] + sp[199] * PI5[0][facet_0][1][3];
    A[41] = sp[214] * PI0[facet_0][facet_0][2][4] + sp[210] * PI1[facet_0][0][2][2] + sp[205] * PI3[facet_0][facet_0][2][4] + sp[209] * PI2[facet_0][0][2][3] + sp[205] * PI3[facet_0][facet_0][4][2] + sp[200] * PI4[0][facet_0][2][2] + sp[204] * PI6[facet_0][facet_0][2][4] + sp[199] * PI5[0][facet_0][3][2] + sp[209] * PI2[facet_0][0][4][1] + sp[199] * PI5[0][facet_0][1][4];
    A[42] = sp[216] * PI0[facet_0][facet_1][2][0] + sp[213] * PI1[facet_0][0][2][0] + sp[215] * PI3[facet_0][facet_1][2][0] + sp[211] * PI1[facet_0][0][2][0] + sp[212] * PI2[facet_0][0][2][0] + sp[207] * PI3[facet_1][facet_0][0][2] + sp[203] * PI4[0][facet_0][0][2] + sp[206] * PI6[facet_0][facet_1][2][0] + sp[201] * PI4[0][facet_0][0][2] + sp[202] * PI5[0][facet_0][0][2] + sp[226] * PI2[facet_1][0][0][1] + sp[218] * PI5[0][facet_1][1][0];
    A[43] = sp[216] * PI0[facet_0][facet_1][2][1] + sp[213] * PI1[facet_0][0][2][1] + sp[207] * PI3[facet_1][facet_0][1][2] + sp[203] * PI4[0][facet_0][1][2] + sp[226] * PI2[facet_1][0][1][1];
    A[44] = sp[215] * PI3[facet_0][facet_1][2][1] + sp[211] * PI1[facet_0][0][2][1] + sp[206] * PI6[facet_0][facet_1][2][1] + sp[201] * PI4[0][facet_0][1][2] + sp[218] * PI5[0][facet_1][1][1];
    A[45] = sp[216] * PI0[facet_0][facet_1][2][2] + sp[215] * PI3[facet_0][facet_1][2][2] + sp[212] * PI2[facet_0][0][2][1] + sp[207] * PI3[facet_1][facet_0][2][2] + sp[206] * PI6[facet_0][facet_1][2][2] + sp[202] * PI5[0][facet_0][1][2] + sp[226] * PI2[facet_1][0][2][1] + sp[218] * PI5[0][facet_1][1][2];
    A[46] = sp[216] * PI0[facet_0][facet_1][2][3] + sp[215] * PI3[facet_0][facet_1][2][3] + sp[211] * PI1[facet_0][0][2][2] + sp[212] * PI2[facet_0][0][2][2] + sp[207] * PI3[facet_1][facet_0][3][2] + sp[206] * PI6[facet_0][facet_1][2][3] + sp[201] * PI4[0][facet_0][2][2] + sp[202] * PI5[0][facet_0][2][2] + sp[226] * PI2[facet_1][0][3][1] + sp[218] * PI5[0][facet_1][1][3];
    A[47] = sp[216] * PI0[facet_0][facet_1][2][4] + sp[213] * PI1[facet_0][0][2][2] + sp[215] * PI3[facet_0][facet_1][2][4] + sp[212] * PI2[facet_0][0][2][3] + sp[207] * PI3[facet_1][facet_0][4][2] + sp[203] * PI4[0][facet_0][2][2] + sp[206] * PI6[facet_0][facet_1][2][4] + sp[202] * PI5[0][facet_0][3][2] + sp[226] * PI2[facet_1][0][4][1] + sp[218] * PI5[0][facet_1][1][4];
    A[48] = sp[214] * PI0[facet_0][facet_0][3][0] + sp[210] * PI1[facet_0][0][3][0] + sp[205] * PI3[facet_0][facet_0][3][0] + sp[208] * PI1[facet_0][0][3][0] + sp[209] * PI2[facet_0][0][3][0] + sp[205] * PI3[facet_0][facet_0][0][3] + sp[200] * PI4[0][facet_0][0][3] + sp[204] * PI6[facet_0][facet_0][3][0] + sp[198] * PI4[0][facet_0][0][3] + sp[199] * PI5[0][facet_0][0][3] + sp[208] * PI1[facet_0][0][0][2] + sp[198] * PI4[0][facet_0][2][0] + sp[209] * PI2[facet_0][0][0][2] + sp[199] * PI5[0][facet_0][2][0];
    A[49] = sp[214] * PI0[facet_0][facet_0][3][1] + sp[210] * PI1[facet_0][0][3][1] + sp[205] * PI3[facet_0][facet_0][1][3] + sp[200] * PI4[0][facet_0][1][3] + sp[208] * PI1[facet_0][0][1][2] + sp[209] * PI2[facet_0][0][1][2];
    A[50] = sp[205] * PI3[facet_0][facet_0][3][1] + sp[208] * PI1[facet_0][0][3][1] + sp[204] * PI6[facet_0][facet_0][3][1] + sp[198] * PI4[0][facet_0][1][3] + sp[198] * PI4[0][facet_0][2][1] + sp[199] * PI5[0][facet_0][2][1];
    A[51] = sp[214] * PI0[facet_0][facet_0][3][2] + sp[205] * PI3[facet_0][facet_0][3][2] + sp[209] * PI2[facet_0][0][3][1] + sp[205] * PI3[facet_0][facet_0][2][3] + sp[204] * PI6[facet_0][facet_0][3][2] + sp[199] * PI5[0][facet_0][1][3] + sp[208] * PI1[facet_0][0][2][2] + sp[198] * PI4[0][facet_0][2][2] + sp[209] * PI2[facet_0][0][2][2] + sp[199] * PI5[0][facet_0][2][2];
    A[52] = sp[214] * PI0[facet_0][facet_0][3][3] + sp[205] * PI3[facet_0][facet_0][3][3] + sp[208] * PI1[facet_0][0][3][2] + sp[209] * PI2[facet_0][0][3][2] + sp[205] * PI3[facet_0][facet_0][3][3] + sp[204] * PI6[facet_0][facet_0][3][3] + sp[198] * PI4[0][facet_0][2][3] + sp[199] * PI5[0][facet_0][2][3] + sp[208] * PI1[facet_0][0][3][2] + sp[198] * PI4[0][facet_0][2][3] + sp[209] * PI2[facet_0][0][3][2] + sp[199] * PI5[0][facet_0][2][3];
    A[53] = sp[214] * PI0[facet_0][facet_0][3][4] + sp[210] * PI1[facet_0][0][3][2] + sp[205] * PI3[facet_0][facet_0][3][4] + sp[209] * PI2[facet_0][0][3][3] + sp[205] * PI3[facet_0][facet_0][4][3] + sp[200] * PI4[0][facet_0][2][3] + sp[204] * PI6[facet_0][facet_0][3][4] + sp[199] * PI5[0][facet_0][3][3] + sp[208] * PI1[facet_0][0][4][2] + sp[198] * PI4[0][facet_0][2][4] + sp[209] * PI2[facet_0][0][4][2] + sp[199] * PI5[0][facet_0][2][4];
    A[54] = sp[216] * PI0[facet_0][facet_1][3][0] + sp[213] * PI1[facet_0][0][3][0] + sp[215] * PI3[facet_0][facet_1][3][0] + sp[211] * PI1[facet_0][0][3][0] + sp[212] * PI2[facet_0][0][3][0] + sp[207] * PI3[facet_1][facet_0][0][3] + sp[203] * PI4[0][facet_0][0][3] + sp[206] * PI6[facet_0][facet_1][3][0] + sp[201] * PI4[0][facet_0][0][3] + sp[202] * PI5[0][facet_0][0][3] + sp[225] * PI1[facet_1][0][0][2] + sp[217] * PI4[0][facet_1][2][0] + sp[226] * PI2[facet_1][0][0][2] + sp[218] * PI5[0][facet_1][2][0];
    A[55] = sp[216] * PI0[facet_0][facet_1][3][1] + sp[213] * PI1[facet_0][0][3][1] + sp[207] * PI3[facet_1][facet_0][1][3] + sp[203] * PI4[0][facet_0][1][3] + sp[225] * PI1[facet_1][0][1][2] + sp[226] * PI2[facet_1][0][1][2];
    A[56] = sp[215] * PI3[facet_0][facet_1][3][1] + sp[211] * PI1[facet_0][0][3][1] + sp[206] * PI6[facet_0][facet_1][3][1] + sp[201] * PI4[0][facet_0][1][3] + sp[217] * PI4[0][facet_1][2][1] + sp[218] * PI5[0][facet_1][2][1];
    A[57] = sp[216] * PI0[facet_0][facet_1][3][2] + sp[215] * PI3[facet_0][facet_1][3][2] + sp[212] * PI2[facet_0][0][3][1] + sp[207] * PI3[facet_1][facet_0][2][3] + sp[206] * PI6[facet_0][facet_1][3][2] + sp[202] * PI5[0][facet_0][1][3] + sp[225] * PI1[facet_1][0][2][2] + sp[217] * PI4[0][facet_1][2][2] + sp[226] * PI2[facet_1][0][2][2] + sp[218] * PI5[0][facet_1][2][2];
    A[58] = sp[216] * PI0[facet_0][facet_1][3][3] + sp[215] * PI3[facet_0][facet_1][3][3] + sp[211] * PI1[facet_0][0][3][2] + sp[212] * PI2[facet_0][0][3][2] + sp[207] * PI3[facet_1][facet_0][3][3] + sp[206] * PI6[facet_0][facet_1][3][3] + sp[201] * PI4[0][facet_0][2][3] + sp[202] * PI5[0][facet_0][2][3] + sp[225] * PI1[facet_1][0][3][2] + sp[217] * PI4[0][facet_1][2][3] + sp[226] * PI2[facet_1][0][3][2] + sp[218] * PI5[0][facet_1][2][3];
    A[59] = sp[216] * PI0[facet_0][facet_1][3][4] + sp[213] * PI1[facet_0][0][3][2] + sp[215] * PI3[facet_0][facet_1][3][4] + sp[212] * PI2[facet_0][0][3][3] + sp[207] * PI3[facet_1][facet_0][4][3] + sp[203] * PI4[0][facet_0][2][3] + sp[206] * PI6[facet_0][facet_1][3][4] + sp[202] * PI5[0][facet_0][3][3] + sp[225] * PI1[facet_1][0][4][2] + sp[217] * PI4[0][facet_1][2][4] + sp[226] * PI2[facet_1][0][4][2] + sp[218] * PI5[0][facet_1][2][4];
    A[60] = sp[214] * PI0[facet_0][facet_0][4][0] + sp[210] * PI1[facet_0][0][4][0] + sp[205] * PI3[facet_0][facet_0][4][0] + sp[208] * PI1[facet_0][0][4][0] + sp[209] * PI2[facet_0][0][4][0] + sp[210] * PI1[facet_0][0][0][2] + sp[200] * PI4[0][facet_0][2][0] + sp[205] * PI3[facet_0][facet_0][0][4] + sp[200] * PI4[0][facet_0][0][4] + sp[204] * PI6[facet_0][facet_0][4][0] + sp[198] * PI4[0][facet_0][0][4] + sp[199] * PI5[0][facet_0][0][4] + sp[209] * PI2[facet_0][0][0][3] + sp[199] * PI5[0][facet_0][3][0];
    A[61] = sp[214] * PI0[facet_0][facet_0][4][1] + sp[210] * PI1[facet_0][0][4][1] + sp[210] * PI1[facet_0][0][1][2] + sp[205] * PI3[facet_0][facet_0][1][4] + sp[200] * PI4[0][facet_0][1][4] + sp[209] * PI2[facet_0][0][1][3];
    A[62] = sp[205] * PI3[facet_0][facet_0][4][1] + sp[208] * PI1[facet_0][0][4][1] + sp[200] * PI4[0][facet_0][2][1] + sp[204] * PI6[facet_0][facet_0][4][1] + sp[198] * PI4[0][facet_0][1][4] + sp[199] * PI5[0][facet_0][3][1];
    A[63] = sp[214] * PI0[facet_0][facet_0][4][2] + sp[205] * PI3[facet_0][facet_0][4][2] + sp[209] * PI2[facet_0][0][4][1] + sp[210] * PI1[facet_0][0][2][2] + sp[200] * PI4[0][facet_0][2][2] + sp[205] * PI3[facet_0][facet_0][2][4] + sp[204] * PI6[facet_0][facet_0][4][2] + sp[199] * PI5[0][facet_0][1][4] + sp[209] * PI2[facet_0][0][2][3] + sp[199] * PI5[0][facet_0][3][2];
    A[64] = sp[214] * PI0[facet_0][facet_0][4][3] + sp[205] * PI3[facet_0][facet_0][4][3] + sp[208] * PI1[facet_0][0][4][2] + sp[209] * PI2[facet_0][0][4][2] + sp[210] * PI1[facet_0][0][3][2] + sp[200] * PI4[0][facet_0][2][3] + sp[205] * PI3[facet_0][facet_0][3][4] + sp[204] * PI6[facet_0][facet_0][4][3] + sp[198] * PI4[0][facet_0][2][4] + sp[199] * PI5[0][facet_0][2][4] + sp[209] * PI2[facet_0][0][3][3] + sp[199] * PI5[0][facet_0][3][3];
    A[65] = sp[214] * PI0[facet_0][facet_0][4][4] + sp[210] * PI1[facet_0][0][4][2] + sp[205] * PI3[facet_0][facet_0][4][4] + sp[209] * PI2[facet_0][0][4][3] + sp[210] * PI1[facet_0][0][4][2] + sp[200] * PI4[0][facet_0][2][4] + sp[205] * PI3[facet_0][facet_0][4][4] + sp[200] * PI4[0][facet_0][2][4] + sp[204] * PI6[facet_0][facet_0][4][4] + sp[199] * PI5[0][facet_0][3][4] + sp[209] * PI2[facet_0][0][4][3] + sp[199] * PI5[0][facet_0][3][4];
    A[66] = sp[216] * PI0[facet_0][facet_1][4][0] + sp[213] * PI1[facet_0][0][4][0] + sp[215] * PI3[facet_0][facet_1][4][0] + sp[211] * PI1[facet_0][0][4][0] + sp[212] * PI2[facet_0][0][4][0] + sp[227] * PI1[facet_1][0][0][2] + sp[219] * PI4[0][facet_1][2][0] + sp[207] * PI3[facet_1][facet_0][0][4] + sp[203] * PI4[0][facet_0][0][4] + sp[206] * PI6[facet_0][facet_1][4][0] + sp[201] * PI4[0][facet_0][0][4] + sp[202] * PI5[0][facet_0][0][4] + sp[226] * PI2[facet_1][0][0][3] + sp[218] * PI5[0][facet_1][3][0];
    A[67] = sp[216] * PI0[facet_0][facet_1][4][1] + sp[213] * PI1[facet_0][0][4][1] + sp[227] * PI1[facet_1][0][1][2] + sp[207] * PI3[facet_1][facet_0][1][4] + sp[203] * PI4[0][facet_0][1][4] + sp[226] * PI2[facet_1][0][1][3];
    A[68] = sp[215] * PI3[facet_0][facet_1][4][1] + sp[211] * PI1[facet_0][0][4][1] + sp[219] * PI4[0][facet_1][2][1] + sp[206] * PI6[facet_0][facet_1][4][1] + sp[201] * PI4[0][facet_0][1][4] + sp[218] * PI5[0][facet_1][3][1];
    A[69] = sp[216] * PI0[facet_0][facet_1][4][2] + sp[215] * PI3[facet_0][facet_1][4][2] + sp[212] * PI2[facet_0][0][4][1] + sp[227] * PI1[facet_1][0][2][2] + sp[219] * PI4[0][facet_1][2][2] + sp[207] * PI3[facet_1][facet_0][2][4] + sp[206] * PI6[facet_0][facet_1][4][2] + sp[202] * PI5[0][facet_0][1][4] + sp[226] * PI2[facet_1][0][2][3] + sp[218] * PI5[0][facet_1][3][2];
    A[70] = sp[216] * PI0[facet_0][facet_1][4][3] + sp[215] * PI3[facet_0][facet_1][4][3] + sp[211] * PI1[facet_0][0][4][2] + sp[212] * PI2[facet_0][0][4][2] + sp[227] * PI1[facet_1][0][3][2] + sp[219] * PI4[0][facet_1][2][3] + sp[207] * PI3[facet_1][facet_0][3][4] + sp[206] * PI6[facet_0][facet_1][4][3] + sp[201] * PI4[0][facet_0][2][4] + sp[202] * PI5[0][facet_0][2][4] + sp[226] * PI2[facet_1][0][3][3] + sp[218] * PI5[0][facet_1][3][3];
    A[71] = sp[216] * PI0[facet_0][facet_1][4][4] + sp[213] * PI1[facet_0][0][4][2] + sp[215] * PI3[facet_0][facet_1][4][4] + sp[212] * PI2[facet_0][0][4][3] + sp[227] * PI1[facet_1][0][4][2] + sp[219] * PI4[0][facet_1][2][4] + sp[207] * PI3[facet_1][facet_0][4][4] + sp[203] * PI4[0][facet_0][2][4] + sp[206] * PI6[facet_0][facet_1][4][4] + sp[202] * PI5[0][facet_0][3][4] + sp[226] * PI2[facet_1][0][4][3] + sp[218] * PI5[0][facet_1][3][4];
    A[72] = sp[216] * PI0[facet_1][facet_0][0][0] + sp[227] * PI1[facet_1][0][0][0] + sp[207] * PI3[facet_1][facet_0][0][0] + sp[225] * PI1[facet_1][0][0][0] + sp[226] * PI2[facet_1][0][0][0] + sp[213] * PI1[facet_0][0][0][0] + sp[203] * PI4[0][facet_0][0][0] + sp[215] * PI3[facet_0][facet_1][0][0] + sp[219] * PI4[0][facet_1][0][0] + sp[206] * PI6[facet_1][facet_0][0][0] + sp[217] * PI4[0][facet_1][0][0] + sp[218] * PI5[0][facet_1][0][0] + sp[211] * PI1[facet_0][0][0][0] + sp[201] * PI4[0][facet_0][0][0] + sp[212] * PI2[facet_0][0][0][0] + sp[202] * PI5[0][facet_0][0][0];
    A[73] = sp[216] * PI0[facet_1][facet_0][0][1] + sp[227] * PI1[facet_1][0][0][1] + sp[213] * PI1[facet_0][0][1][0] + sp[215] * PI3[facet_0][facet_1][1][0] + sp[219] * PI4[0][facet_1][1][0] + sp[211] * PI1[facet_0][0][1][0] + sp[212] * PI2[facet_0][0][1][0];
    A[74] = sp[207] * PI3[facet_1][facet_0][0][1] + sp[225] * PI1[facet_1][0][0][1] + sp[203] * PI4[0][facet_0][0][1] + sp[206] * PI6[facet_1][facet_0][0][1] + sp[217] * PI4[0][facet_1][1][0] + sp[201] * PI4[0][facet_0][0][1] + sp[202] * PI5[0][facet_0][0][1];
    A[75] = sp[216] * PI0[facet_1][facet_0][0][2] + sp[207] * PI3[facet_1][facet_0][0][2] + sp[226] * PI2[facet_1][0][0][1] + sp[213] * PI1[facet_0][0][2][0] + sp[203] * PI4[0][facet_0][0][2] + sp[215] * PI3[facet_0][facet_1][2][0] + sp[206] * PI6[facet_1][facet_0][0][2] + sp[218] * PI5[0][facet_1][1][0] + sp[211] * PI1[facet_0][0][2][0] + sp[201] * PI4[0][facet_0][0][2] + sp[212] * PI2[facet_0][0][2][0] + sp[202] * PI5[0][facet_0][0][2];
    A[76] = sp[216] * PI0[facet_1][facet_0][0][3] + sp[207] * PI3[facet_1][facet_0][0][3] + sp[225] * PI1[facet_1][0][0][2] + sp[226] * PI2[facet_1][0][0][2] + sp[213] * PI1[facet_0][0][3][0] + sp[203] * PI4[0][facet_0][0][3] + sp[215] * PI3[facet_0][facet_1][3][0] + sp[206] * PI6[facet_1][facet_0][0][3] + sp[217] * PI4[0][facet_1][2][0] + sp[218] * PI5[0][facet_1][2][0] + sp[211] * PI1[facet_0][0][3][0] + sp[201] * PI4[0][facet_0][0][3] + sp[212] * PI2[facet_0][0][3][0] + sp[202] * PI5[0][facet_0][0][3];
    A[77] = sp[216] * PI0[facet_1][facet_0][0][4] + sp[227] * PI1[facet_1][0][0][2] + sp[207] * PI3[facet_1][facet_0][0][4] + sp[226] * PI2[facet_1][0][0][3] + sp[213] * PI1[facet_0][0][4][0] + sp[203] * PI4[0][facet_0][0][4] + sp[215] * PI3[facet_0][facet_1][4][0] + sp[219] * PI4[0][facet_1][2][0] + sp[206] * PI6[facet_1][facet_0][0][4] + sp[218] * PI5[0][facet_1][3][0] + sp[211] * PI1[facet_0][0][4][0] + sp[201] * PI4[0][facet_0][0][4] + sp[212] * PI2[facet_0][0][4][0] + sp[202] * PI5[0][facet_0][0][4];
    A[78] = sp[231] * PI0[facet_1][facet_1][0][0] + sp[230] * PI1[facet_1][0][0][0] + sp[224] * PI3[facet_1][facet_1][0][0] + sp[228] * PI1[facet_1][0][0][0] + sp[229] * PI2[facet_1][0][0][0] + sp[230] * PI1[facet_1][0][0][0] + sp[222] * PI4[0][facet_1][0][0] + sp[224] * PI3[facet_1][facet_1][0][0] + sp[222] * PI4[0][facet_1][0][0] + sp[223] * PI6[facet_1][facet_1][0][0] + sp[220] * PI4[0][facet_1][0][0] + sp[221] * PI5[0][facet_1][0][0] + sp[228] * PI1[facet_1][0][0][0] + sp[220] * PI4[0][facet_1][0][0] + sp[229] * PI2[facet_1][0][0][0] + sp[221] * PI5[0][facet_1][0][0];
    A[79] = sp[231] * PI0[facet_1][facet_1][0][1] + sp[230] * PI1[facet_1][0][0][1] + sp[230] * PI1[facet_1][0][1][0] + sp[224] * PI3[facet_1][facet_1][1][0] + sp[222] * PI4[0][facet_1][1][0] + sp[228] * PI1[facet_1][0][1][0] + sp[229] * PI2[facet_1][0][1][0];
    A[80] = sp[224] * PI3[facet_1][facet_1][0][1] + sp[228] * PI1[facet_1][0][0][1] + sp[222] * PI4[0][facet_1][0][1] + sp[223] * PI6[facet_1][facet_1][0][1] + sp[220] * PI4[0][facet_1][1][0] + sp[220] * PI4[0][facet_1][0][1] + sp[221] * PI5[0][facet_1][0][1];
    A[81] = sp[231] * PI0[facet_1][facet_1][0][2] + sp[224] * PI3[facet_1][facet_1][0][2] + sp[229] * PI2[facet_1][0][0][1] + sp[230] * PI1[facet_1][0][2][0] + sp[222] * PI4[0][facet_1][0][2] + sp[224] * PI3[facet_1][facet_1][2][0] + sp[223] * PI6[facet_1][facet_1][0][2] + sp[221] * PI5[0][facet_1][1][0] + sp[228] * PI1[facet_1][0][2][0] + sp[220] * PI4[0][facet_1][0][2] + sp[229] * PI2[facet_1][0][2][0] + sp[221] * PI5[0][facet_1][0][2];
    A[82] = sp[231] * PI0[facet_1][facet_1][0][3] + sp[224] * PI3[facet_1][facet_1][0][3] + sp[228] * PI1[facet_1][0][0][2] + sp[229] * PI2[facet_1][0][0][2] + sp[230] * PI1[facet_1][0][3][0] + sp[222] * PI4[0][facet_1][0][3] + sp[224] * PI3[facet_1][facet_1][3][0] + sp[223] * PI6[facet_1][facet_1][0][3] + sp[220] * PI4[0][facet_1][2][0] + sp[221] * PI5[0][facet_1][2][0] + sp[228] * PI1[facet_1][0][3][0] + sp[220] * PI4[0][facet_1][0][3] + sp[229] * PI2[facet_1][0][3][0] + sp[221] * PI5[0][facet_1][0][3];
    A[83] = sp[231] * PI0[facet_1][facet_1][0][4] + sp[230] * PI1[facet_1][0][0][2] + sp[224] * PI3[facet_1][facet_1][0][4] + sp[229] * PI2[facet_1][0][0][3] + sp[230] * PI1[facet_1][0][4][0] + sp[222] * PI4[0][facet_1][0][4] + sp[224] * PI3[facet_1][facet_1][4][0] + sp[222] * PI4[0][facet_1][2][0] + sp[223] * PI6[facet_1][facet_1][0][4] + sp[221] * PI5[0][facet_1][3][0] + sp[228] * PI1[facet_1][0][4][0] + sp[220] * PI4[0][facet_1][0][4] + sp[229] * PI2[facet_1][0][4][0] + sp[221] * PI5[0][facet_1][0][4];
    A[84] = sp[216] * PI0[facet_1][facet_0][1][0] + sp[227] * PI1[facet_1][0][1][0] + sp[207] * PI3[facet_1][facet_0][1][0] + sp[225] * PI1[facet_1][0][1][0] + sp[226] * PI2[facet_1][0][1][0] + sp[213] * PI1[facet_0][0][0][1] + sp[203] * PI4[0][facet_0][1][0];
    A[85] = sp[216] * PI0[facet_1][facet_0][1][1] + sp[227] * PI1[facet_1][0][1][1] + sp[213] * PI1[facet_0][0][1][1];
    A[86] = sp[207] * PI3[facet_1][facet_0][1][1] + sp[225] * PI1[facet_1][0][1][1] + sp[203] * PI4[0][facet_0][1][1];
    A[87] = sp[216] * PI0[facet_1][facet_0][1][2] + sp[207] * PI3[facet_1][facet_0][1][2] + sp[226] * PI2[facet_1][0][1][1] + sp[213] * PI1[facet_0][0][2][1] + sp[203] * PI4[0][facet_0][1][2];
    A[88] = sp[216] * PI0[facet_1][facet_0][1][3] + sp[207] * PI3[facet_1][facet_0][1][3] + sp[225] * PI1[facet_1][0][1][2] + sp[226] * PI2[facet_1][0][1][2] + sp[213] * PI1[facet_0][0][3][1] + sp[203] * PI4[0][facet_0][1][3];
    A[89] = sp[216] * PI0[facet_1][facet_0][1][4] + sp[227] * PI1[facet_1][0][1][2] + sp[207] * PI3[facet_1][facet_0][1][4] + sp[226] * PI2[facet_1][0][1][3] + sp[213] * PI1[facet_0][0][4][1] + sp[203] * PI4[0][facet_0][1][4];
    A[90] = sp[231] * PI0[facet_1][facet_1][1][0] + sp[230] * PI1[facet_1][0][1][0] + sp[224] * PI3[facet_1][facet_1][1][0] + sp[228] * PI1[facet_1][0][1][0] + sp[229] * PI2[facet_1][0][1][0] + sp[230] * PI1[facet_1][0][0][1] + sp[222] * PI4[0][facet_1][1][0];
    A[91] = sp[231] * PI0[facet_1][facet_1][1][1] + sp[230] * PI1[facet_1][0][1][1] + sp[230] * PI1[facet_1][0][1][1];
    A[92] = sp[224] * PI3[facet_1][facet_1][1][1] + sp[228] * PI1[facet_1][0][1][1] + sp[222] * PI4[0][facet_1][1][1];
    A[93] = sp[231] * PI0[facet_1][facet_1][1][2] + sp[224] * PI3[facet_1][facet_1][1][2] + sp[229] * PI2[facet_1][0][1][1] + sp[230] * PI1[facet_1][0][2][1] + sp[222] * PI4[0][facet_1][1][2];
    A[94] = sp[231] * PI0[facet_1][facet_1][1][3] + sp[224] * PI3[facet_1][facet_1][1][3] + sp[228] * PI1[facet_1][0][1][2] + sp[229] * PI2[facet_1][0][1][2] + sp[230] * PI1[facet_1][0][3][1] + sp[222] * PI4[0][facet_1][1][3];
    A[95] = sp[231] * PI0[facet_1][facet_1][1][4] + sp[230] * PI1[facet_1][0][1][2] + sp[224] * PI3[facet_1][facet_1][1][4] + sp[229] * PI2[facet_1][0][1][3] + sp[230] * PI1[facet_1][0][4][1] + sp[222] * PI4[0][facet_1][1][4];
    A[96] = sp[215] * PI3[facet_0][facet_1][0][1] + sp[219] * PI4[0][facet_1][0][1] + sp[206] * PI6[facet_1][facet_0][1][0] + sp[217] * PI4[0][facet_1][0][1] + sp[218] * PI5[0][facet_1][0][1] + sp[211] * PI1[facet_0][0][0][1] + sp[201] * PI4[0][facet_0][1][0];
    A[97] = sp[215] * PI3[facet_0][facet_1][1][1] + sp[219] * PI4[0][facet_1][1][1] + sp[211] * PI1[facet_0][0][1][1];
    A[98] = sp[206] * PI6[facet_1][facet_0][1][1] + sp[217] * PI4[0][facet_1][1][1] + sp[201] * PI4[0][facet_0][1][1];
    A[99] = sp[215] * PI3[facet_0][facet_1][2][1] + sp[206] * PI6[facet_1][facet_0][1][2] + sp[218] * PI5[0][facet_1][1][1] + sp[211] * PI1[facet_0][0][2][1] + sp[201] * PI4[0][facet_0][1][2];
    A[100] = sp[215] * PI3[facet_0][facet_1][3][1] + sp[206] * PI6[facet_1][facet_0][1][3] + sp[217] * PI4[0][facet_1][2][1] + sp[218] * PI5[0][facet_1][2][1] + sp[211] * PI1[facet_0][0][3][1] + sp[201] * PI4[0][facet_0][1][3];
    A[101] = sp[215] * PI3[facet_0][facet_1][4][1] + sp[219] * PI4[0][facet_1][2][1] + sp[206] * PI6[facet_1][facet_0][1][4] + sp[218] * PI5[0][facet_1][3][1] + sp[211] * PI1[facet_0][0][4][1] + sp[201] * PI4[0][facet_0][1][4];
    A[102] = sp[224] * PI3[facet_1][facet_1][0][1] + sp[222] * PI4[0][facet_1][0][1] + sp[223] * PI6[facet_1][facet_1][1][0] + sp[220] * PI4[0][facet_1][0][1] + sp[221] * PI5[0][facet_1][0][1] + sp[228] * PI1[facet_1][0][0][1] + sp[220] * PI4[0][facet_1][1][0];
    A[103] = sp[224] * PI3[facet_1][facet_1][1][1] + sp[222] * PI4[0][facet_1][1][1] + sp[228] * PI1[facet_1][0][1][1];
    A[104] = sp[223] * PI6[facet_1][facet_1][1][1] + sp[220] * PI4[0][facet_1][1][1] + sp[220] * PI4[0][facet_1][1][1];
    A[105] = sp[224] * PI3[facet_1][facet_1][2][1] + sp[223] * PI6[facet_1][facet_1][1][2] + sp[221] * PI5[0][facet_1][1][1] + sp[228] * PI1[facet_1][0][2][1] + sp[220] * PI4[0][facet_1][1][2];
    A[106] = sp[224] * PI3[facet_1][facet_1][3][1] + sp[223] * PI6[facet_1][facet_1][1][3] + sp[220] * PI4[0][facet_1][2][1] + sp[221] * PI5[0][facet_1][2][1] + sp[228] * PI1[facet_1][0][3][1] + sp[220] * PI4[0][facet_1][1][3];
    A[107] = sp[224] * PI3[facet_1][facet_1][4][1] + sp[222] * PI4[0][facet_1][2][1] + sp[223] * PI6[facet_1][facet_1][1][4] + sp[221] * PI5[0][facet_1][3][1] + sp[228] * PI1[facet_1][0][4][1] + sp[220] * PI4[0][facet_1][1][4];
    A[108] = sp[216] * PI0[facet_1][facet_0][2][0] + sp[227] * PI1[facet_1][0][2][0] + sp[207] * PI3[facet_1][facet_0][2][0] + sp[225] * PI1[facet_1][0][2][0] + sp[226] * PI2[facet_1][0][2][0] + sp[215] * PI3[facet_0][facet_1][0][2] + sp[219] * PI4[0][facet_1][0][2] + sp[206] * PI6[facet_1][facet_0][2][0] + sp[217] * PI4[0][facet_1][0][2] + sp[218] * PI5[0][facet_1][0][2] + sp[212] * PI2[facet_0][0][0][1] + sp[202] * PI5[0][facet_0][1][0];
    A[109] = sp[216] * PI0[facet_1][facet_0][2][1] + sp[227] * PI1[facet_1][0][2][1] + sp[215] * PI3[facet_0][facet_1][1][2] + sp[219] * PI4[0][facet_1][1][2] + sp[212] * PI2[facet_0][0][1][1];
    A[110] = sp[207] * PI3[facet_1][facet_0][2][1] + sp[225] * PI1[facet_1][0][2][1] + sp[206] * PI6[facet_1][facet_0][2][1] + sp[217] * PI4[0][facet_1][1][2] + sp[202] * PI5[0][facet_0][1][1];
    A[111] = sp[216] * PI0[facet_1][facet_0][2][2] + sp[207] * PI3[facet_1][facet_0][2][2] + sp[226] * PI2[facet_1][0][2][1] + sp[215] * PI3[facet_0][facet_1][2][2] + sp[206] * PI6[facet_1][facet_0][2][2] + sp[218] * PI5[0][facet_1][1][2] + sp[212] * PI2[facet_0][0][2][1] + sp[202] * PI5[0][facet_0][1][2];
    A[112] = sp[216] * PI0[facet_1][facet_0][2][3] + sp[207] * PI3[facet_1][facet_0][2][3] + sp[225] * PI1[facet_1][0][2][2] + sp[226] * PI2[facet_1][0][2][2] + sp[215] * PI3[facet_0][facet_1][3][2] + sp[206] * PI6[facet_1][facet_0][2][3] + sp[217] * PI4[0][facet_1][2][2] + sp[218] * PI5[0][facet_1][2][2] + sp[212] * PI2[facet_0][0][3][1] + sp[202] * PI5[0][facet_0][1][3];
    A[113] = sp[216] * PI0[facet_1][facet_0][2][4] + sp[227] * PI1[facet_1][0][2][2] + sp[207] * PI3[facet_1][facet_0][2][4] + sp[226] * PI2[facet_1][0][2][3] + sp[215] * PI3[facet_0][facet_1][4][2] + sp[219] * PI4[0][facet_1][2][2] + sp[206] * PI6[facet_1][facet_0][2][4] + sp[218] * PI5[0][facet_1][3][2] + sp[212] * PI2[facet_0][0][4][1] + sp[202] * PI5[0][facet_0][1][4];
    A[114] = sp[231] * PI0[facet_1][facet_1][2][0] + sp[230] * PI1[facet_1][0][2][0] + sp[224] * PI3[facet_1][facet_1][2][0] + sp[228] * PI1[facet_1][0][2][0] + sp[229] * PI2[facet_1][0][2][0] + sp[224] * PI3[facet_1][facet_1][0][2] + sp[222] * PI4[0][facet_1][0][2] + sp[223] * PI6[facet_1][facet_1][2][0] + sp[220] * PI4[0][facet_1][0][2] + sp[221] * PI5[0][facet_1][0][2] + sp[229] * PI2[facet_1][0][0][1] + sp[221] * PI5[0][facet_1][1][0];
    A[115] = sp[231] * PI0[facet_1][facet_1][2][1] + sp[230] * PI1[facet_1][0][2][1] + sp[224] * PI3[facet_1][facet_1][1][2] + sp[222] * PI4[0][facet_1][1][2] + sp[229] * PI2[facet_1][0][1][1];
    A[116] = sp[224] * PI3[facet_1][facet_1][2][1] + sp[228] * PI1[facet_1][0][2][1] + sp[223] * PI6[facet_1][facet_1][2][1] + sp[220] * PI4[0][facet_1][1][2] + sp[221] * PI5[0][facet_1][1][1];
    A[117] = sp[231] * PI0[facet_1][facet_1][2][2] + sp[224] * PI3[facet_1][facet_1][2][2] + sp[229] * PI2[facet_1][0][2][1] + sp[224] * PI3[facet_1][facet_1][2][2] + sp[223] * PI6[facet_1][facet_1][2][2] + sp[221] * PI5[0][facet_1][1][2] + sp[229] * PI2[facet_1][0][2][1] + sp[221] * PI5[0][facet_1][1][2];
    A[118] = sp[231] * PI0[facet_1][facet_1][2][3] + sp[224] * PI3[facet_1][facet_1][2][3] + sp[228] * PI1[facet_1][0][2][2] + sp[229] * PI2[facet_1][0][2][2] + sp[224] * PI3[facet_1][facet_1][3][2] + sp[223] * PI6[facet_1][facet_1][2][3] + sp[220] * PI4[0][facet_1][2][2] + sp[221] * PI5[0][facet_1][2][2] + sp[229] * PI2[facet_1][0][3][1] + sp[221] * PI5[0][facet_1][1][3];
    A[119] = sp[231] * PI0[facet_1][facet_1][2][4] + sp[230] * PI1[facet_1][0][2][2] + sp[224] * PI3[facet_1][facet_1][2][4] + sp[229] * PI2[facet_1][0][2][3] + sp[224] * PI3[facet_1][facet_1][4][2] + sp[222] * PI4[0][facet_1][2][2] + sp[223] * PI6[facet_1][facet_1][2][4] + sp[221] * PI5[0][facet_1][3][2] + sp[229] * PI2[facet_1][0][4][1] + sp[221] * PI5[0][facet_1][1][4];
    A[120] = sp[216] * PI0[facet_1][facet_0][3][0] + sp[227] * PI1[facet_1][0][3][0] + sp[207] * PI3[facet_1][facet_0][3][0] + sp[225] * PI1[facet_1][0][3][0] + sp[226] * PI2[facet_1][0][3][0] + sp[215] * PI3[facet_0][facet_1][0][3] + sp[219] * PI4[0][facet_1][0][3] + sp[206] * PI6[facet_1][facet_0][3][0] + sp[217] * PI4[0][facet_1][0][3] + sp[218] * PI5[0][facet_1][0][3] + sp[211] * PI1[facet_0][0][0][2] + sp[201] * PI4[0][facet_0][2][0] + sp[212] * PI2[facet_0][0][0][2] + sp[202] * PI5[0][facet_0][2][0];
    A[121] = sp[216] * PI0[facet_1][facet_0][3][1] + sp[227] * PI1[facet_1][0][3][1] + sp[215] * PI3[facet_0][facet_1][1][3] + sp[219] * PI4[0][facet_1][1][3] + sp[211] * PI1[facet_0][0][1][2] + sp[212] * PI2[facet_0][0][1][2];
    A[122] = sp[207] * PI3[facet_1][facet_0][3][1] + sp[225] * PI1[facet_1][0][3][1] + sp[206] * PI6[facet_1][facet_0][3][1] + sp[217] * PI4[0][facet_1][1][3] + sp[201] * PI4[0][facet_0][2][1] + sp[202] * PI5[0][facet_0][2][1];
    A[123] = sp[216] * PI0[facet_1][facet_0][3][2] + sp[207] * PI3[facet_1][facet_0][3][2] + sp[226] * PI2[facet_1][0][3][1] + sp[215] * PI3[facet_0][facet_1][2][3] + sp[206] * PI6[facet_1][facet_0][3][2] + sp[218] * PI5[0][facet_1][1][3] + sp[211] * PI1[facet_0][0][2][2] + sp[201] * PI4[0][facet_0][2][2] + sp[212] * PI2[facet_0][0][2][2] + sp[202] * PI5[0][facet_0][2][2];
    A[124] = sp[216] * PI0[facet_1][facet_0][3][3] + sp[207] * PI3[facet_1][facet_0][3][3] + sp[225] * PI1[facet_1][0][3][2] + sp[226] * PI2[facet_1][0][3][2] + sp[215] * PI3[facet_0][facet_1][3][3] + sp[206] * PI6[facet_1][facet_0][3][3] + sp[217] * PI4[0][facet_1][2][3] + sp[218] * PI5[0][facet_1][2][3] + sp[211] * PI1[facet_0][0][3][2] + sp[201] * PI4[0][facet_0][2][3] + sp[212] * PI2[facet_0][0][3][2] + sp[202] * PI5[0][facet_0][2][3];
    A[125] = sp[216] * PI0[facet_1][facet_0][3][4] + sp[227] * PI1[facet_1][0][3][2] + sp[207] * PI3[facet_1][facet_0][3][4] + sp[226] * PI2[facet_1][0][3][3] + sp[215] * PI3[facet_0][facet_1][4][3] + sp[219] * PI4[0][facet_1][2][3] + sp[206] * PI6[facet_1][facet_0][3][4] + sp[218] * PI5[0][facet_1][3][3] + sp[211] * PI1[facet_0][0][4][2] + sp[201] * PI4[0][facet_0][2][4] + sp[212] * PI2[facet_0][0][4][2] + sp[202] * PI5[0][facet_0][2][4];
    A[126] = sp[231] * PI0[facet_1][facet_1][3][0] + sp[230] * PI1[facet_1][0][3][0] + sp[224] * PI3[facet_1][facet_1][3][0] + sp[228] * PI1[facet_1][0][3][0] + sp[229] * PI2[facet_1][0][3][0] + sp[224] * PI3[facet_1][facet_1][0][3] + sp[222] * PI4[0][facet_1][0][3] + sp[223] * PI6[facet_1][facet_1][3][0] + sp[220] * PI4[0][facet_1][0][3] + sp[221] * PI5[0][facet_1][0][3] + sp[228] * PI1[facet_1][0][0][2] + sp[220] * PI4[0][facet_1][2][0] + sp[229] * PI2[facet_1][0][0][2] + sp[221] * PI5[0][facet_1][2][0];
    A[127] = sp[231] * PI0[facet_1][facet_1][3][1] + sp[230] * PI1[facet_1][0][3][1] + sp[224] * PI3[facet_1][facet_1][1][3] + sp[222] * PI4[0][facet_1][1][3] + sp[228] * PI1[facet_1][0][1][2] + sp[229] * PI2[facet_1][0][1][2];
    A[128] = sp[224] * PI3[facet_1][facet_1][3][1] + sp[228] * PI1[facet_1][0][3][1] + sp[223] * PI6[facet_1][facet_1][3][1] + sp[220] * PI4[0][facet_1][1][3] + sp[220] * PI4[0][facet_1][2][1] + sp[221] * PI5[0][facet_1][2][1];
    A[129] = sp[231] * PI0[facet_1][facet_1][3][2] + sp[224] * PI3[facet_1][facet_1][3][2] + sp[229] * PI2[facet_1][0][3][1] + sp[224] * PI3[facet_1][facet_1][2][3] + sp[223] * PI6[facet_1][facet_1][3][2] + sp[221] * PI5[0][facet_1][1][3] + sp[228] * PI1[facet_1][0][2][2] + sp[220] * PI4[0][facet_1][2][2] + sp[229] * PI2[facet_1][0][2][2] + sp[221] * PI5[0][facet_1][2][2];
    A[130] = sp[231] * PI0[facet_1][facet_1][3][3] + sp[224] * PI3[facet_1][facet_1][3][3] + sp[228] * PI1[facet_1][0][3][2] + sp[229] * PI2[facet_1][0][3][2] + sp[224] * PI3[facet_1][facet_1][3][3] + sp[223] * PI6[facet_1][facet_1][3][3] + sp[220] * PI4[0][facet_1][2][3] + sp[221] * PI5[0][facet_1][2][3] + sp[228] * PI1[facet_1][0][3][2] + sp[220] * PI4[0][facet_1][2][3] + sp[229] * PI2[facet_1][0][3][2] + sp[221] * PI5[0][facet_1][2][3];
    A[131] = sp[231] * PI0[facet_1][facet_1][3][4] + sp[230] * PI1[facet_1][0][3][2] + sp[224] * PI3[facet_1][facet_1][3][4] + sp[229] * PI2[facet_1][0][3][3] + sp[224] * PI3[facet_1][facet_1][4][3] + sp[222] * PI4[0][facet_1][2][3] + sp[223] * PI6[facet_1][facet_1][3][4] + sp[221] * PI5[0][facet_1][3][3] + sp[228] * PI1[facet_1][0][4][2] + sp[220] * PI4[0][facet_1][2][4] + sp[229] * PI2[facet_1][0][4][2] + sp[221] * PI5[0][facet_1][2][4];
    A[132] = sp[216] * PI0[facet_1][facet_0][4][0] + sp[227] * PI1[facet_1][0][4][0] + sp[207] * PI3[facet_1][facet_0][4][0] + sp[225] * PI1[facet_1][0][4][0] + sp[226] * PI2[facet_1][0][4][0] + sp[213] * PI1[facet_0][0][0][2] + sp[203] * PI4[0][facet_0][2][0] + sp[215] * PI3[facet_0][facet_1][0][4] + sp[219] * PI4[0][facet_1][0][4] + sp[206] * PI6[facet_1][facet_0][4][0] + sp[217] * PI4[0][facet_1][0][4] + sp[218] * PI5[0][facet_1][0][4] + sp[212] * PI2[facet_0][0][0][3] + sp[202] * PI5[0][facet_0][3][0];
    A[133] = sp[216] * PI0[facet_1][facet_0][4][1] + sp[227] * PI1[facet_1][0][4][1] + sp[213] * PI1[facet_0][0][1][2] + sp[215] * PI3[facet_0][facet_1][1][4] + sp[219] * PI4[0][facet_1][1][4] + sp[212] * PI2[facet_0][0][1][3];
    A[134] = sp[207] * PI3[facet_1][facet_0][4][1] + sp[225] * PI1[facet_1][0][4][1] + sp[203] * PI4[0][facet_0][2][1] + sp[206] * PI6[facet_1][facet_0][4][1] + sp[217] * PI4[0][facet_1][1][4] + sp[202] * PI5[0][facet_0][3][1];
    A[135] = sp[216] * PI0[facet_1][facet_0][4][2] + sp[207] * PI3[facet_1][facet_0][4][2] + sp[226] * PI2[facet_1][0][4][1] + sp[213] * PI1[facet_0][0][2][2] + sp[203] * PI4[0][facet_0][2][2] + sp[215] * PI3[facet_0][facet_1][2][4] + sp[206] * PI6[facet_1][facet_0][4][2] + sp[218] * PI5[0][facet_1][1][4] + sp[212] * PI2[facet_0][0][2][3] + sp[202] * PI5[0][facet_0][3][2];
    A[136] = sp[216] * PI0[facet_1][facet_0][4][3] + sp[207] * PI3[facet_1][facet_0][4][3] + sp[225] * PI1[facet_1][0][4][2] + sp[226] * PI2[facet_1][0][4][2] + sp[213] * PI1[facet_0][0][3][2] + sp[203] * PI4[0][facet_0][2][3] + sp[215] * PI3[facet_0][facet_1][3][4] + sp[206] * PI6[facet_1][facet_0][4][3] + sp[217] * PI4[0][facet_1][2][4] + sp[218] * PI5[0][facet_1][2][4] + sp[212] * PI2[facet_0][0][3][3] + sp[202] * PI5[0][facet_0][3][3];
    A[137] = sp[216] * PI0[facet_1][facet_0][4][4] + sp[227] * PI1[facet_1][0][4][2] + sp[207] * PI3[facet_1][facet_0][4][4] + sp[226] * PI2[facet_1][0][4][3] + sp[213] * PI1[facet_0][0][4][2] + sp[203] * PI4[0][facet_0][2][4] + sp[215] * PI3[facet_0][facet_1][4][4] + sp[219] * PI4[0][facet_1][2][4] + sp[206] * PI6[facet_1][facet_0][4][4] + sp[218] * PI5[0][facet_1][3][4] + sp[212] * PI2[facet_0][0][4][3] + sp[202] * PI5[0][facet_0][3][4];
    A[138] = sp[231] * PI0[facet_1][facet_1][4][0] + sp[230] * PI1[facet_1][0][4][0] + sp[224] * PI3[facet_1][facet_1][4][0] + sp[228] * PI1[facet_1][0][4][0] + sp[229] * PI2[facet_1][0][4][0] + sp[230] * PI1[facet_1][0][0][2] + sp[222] * PI4[0][facet_1][2][0] + sp[224] * PI3[facet_1][facet_1][0][4] + sp[222] * PI4[0][facet_1][0][4] + sp[223] * PI6[facet_1][facet_1][4][0] + sp[220] * PI4[0][facet_1][0][4] + sp[221] * PI5[0][facet_1][0][4] + sp[229] * PI2[facet_1][0][0][3] + sp[221] * PI5[0][facet_1][3][0];
    A[139] = sp[231] * PI0[facet_1][facet_1][4][1] + sp[230] * PI1[facet_1][0][4][1] + sp[230] * PI1[facet_1][0][1][2] + sp[224] * PI3[facet_1][facet_1][1][4] + sp[222] * PI4[0][facet_1][1][4] + sp[229] * PI2[facet_1][0][1][3];
    A[140] = sp[224] * PI3[facet_1][facet_1][4][1] + sp[228] * PI1[facet_1][0][4][1] + sp[222] * PI4[0][facet_1][2][1] + sp[223] * PI6[facet_1][facet_1][4][1] + sp[220] * PI4[0][facet_1][1][4] + sp[221] * PI5[0][facet_1][3][1];
    A[141] = sp[231] * PI0[facet_1][facet_1][4][2] + sp[224] * PI3[facet_1][facet_1][4][2] + sp[229] * PI2[facet_1][0][4][1] + sp[230] * PI1[facet_1][0][2][2] + sp[222] * PI4[0][facet_1][2][2] + sp[224] * PI3[facet_1][facet_1][2][4] + sp[223] * PI6[facet_1][facet_1][4][2] + sp[221] * PI5[0][facet_1][1][4] + sp[229] * PI2[facet_1][0][2][3] + sp[221] * PI5[0][facet_1][3][2];
    A[142] = sp[231] * PI0[facet_1][facet_1][4][3] + sp[224] * PI3[facet_1][facet_1][4][3] + sp[228] * PI1[facet_1][0][4][2] + sp[229] * PI2[facet_1][0][4][2] + sp[230] * PI1[facet_1][0][3][2] + sp[222] * PI4[0][facet_1][2][3] + sp[224] * PI3[facet_1][facet_1][3][4] + sp[223] * PI6[facet_1][facet_1][4][3] + sp[220] * PI4[0][facet_1][2][4] + sp[221] * PI5[0][facet_1][2][4] + sp[229] * PI2[facet_1][0][3][3] + sp[221] * PI5[0][facet_1][3][3];
    A[143] = sp[231] * PI0[facet_1][facet_1][4][4] + sp[230] * PI1[facet_1][0][4][2] + sp[224] * PI3[facet_1][facet_1][4][4] + sp[229] * PI2[facet_1][0][4][3] + sp[230] * PI1[facet_1][0][4][2] + sp[222] * PI4[0][facet_1][2][4] + sp[224] * PI3[facet_1][facet_1][4][4] + sp[222] * PI4[0][facet_1][2][4] + sp[223] * PI6[facet_1][facet_1][4][4] + sp[221] * PI5[0][facet_1][3][4] + sp[229] * PI2[facet_1][0][4][3] + sp[221] * PI5[0][facet_1][3][4];
  }

};


class biharmonic_cell_integral_1_otherwise: public ufc::cell_integral
{
public:

  biharmonic_cell_integral_1_otherwise() : ufc::cell_integral()
  {

  }

  ~biharmonic_cell_integral_1_otherwise() override
  {

  }

  const std::vector<bool> & enabled_coefficients() const final override
  {
static const std::vector<bool> enabled({true});
return enabled;
  }

  void tabulate_tensor(double * A,
                       const double * const * w,
                       const double * coordinate_dofs,
                       int cell_orientation) const final override
  {
    // This function was generated using 'uflacs' representation
    // with the following integrals metadata:
    // 
    //   num_cells:         None
    //   optimize:          True
    //   precision:         16
    //   quadrature_degree: 4
    //   quadrature_rule:   'default'
    //   representation:    'uflacs'
    // 
    // and the following integral 0 metadata:
    // 
    //   estimated_polynomial_degree: 4
    //   optimize:                    True
    //   precision:                   16
    //   quadrature_degree:           4
    //   quadrature_rule:             'default'
    //   representation:              'uflacs'
    
    // Quadrature rules
    alignas(32) static const double weights6[6] = { 0.054975871827661, 0.054975871827661, 0.054975871827661, 0.1116907948390055, 0.1116907948390055, 0.1116907948390055 };
    // Precomputed values of basis functions and precomputations
    // FE* dimensions: [entities][points][dofs]
    // PI* dimensions: [entities][dofs][dofs] or [entities][dofs]
    // PM* dimensions: [entities][dofs][dofs]
    alignas(32) static const double FE3_C0_D01_Q6[1][1][2] = { { { -1.0, 1.0 } } };
    alignas(32) static const double FE6_C0_Q6[1][6][6] =
        { { { -0.07480380774819603, 0.5176323419876735, -0.07480380774819664, 0.2992152309927869, 0.03354481152314828, 0.2992152309927838 },
            { -0.07480380774819599, -0.07480380774819662, 0.5176323419876736, 0.2992152309927869, 0.2992152309927838, 0.03354481152314827 },
            { 0.5176323419876714, -0.07480380774819663, -0.07480380774819664, 0.03354481152314866, 0.2992152309927866, 0.2992152309927866 },
            { -0.04820837781551188, -0.08473049309397782, -0.04820837781551189, 0.1928335112620478, 0.795480226200906, 0.1928335112620478 },
            { -0.04820837781551192, -0.04820837781551197, -0.08473049309397784, 0.1928335112620478, 0.1928335112620478, 0.7954802262009062 },
            { -0.08473049309397777, -0.04820837781551201, -0.04820837781551189, 0.795480226200906, 0.1928335112620478, 0.1928335112620478 } } };
    // Unstructured piecewise computations
    const double J_c0 = coordinate_dofs[0] * FE3_C0_D01_Q6[0][0][0] + coordinate_dofs[2] * FE3_C0_D01_Q6[0][0][1];
    const double J_c3 = coordinate_dofs[1] * FE3_C0_D01_Q6[0][0][0] + coordinate_dofs[5] * FE3_C0_D01_Q6[0][0][1];
    const double J_c1 = coordinate_dofs[0] * FE3_C0_D01_Q6[0][0][0] + coordinate_dofs[4] * FE3_C0_D01_Q6[0][0][1];
    const double J_c2 = coordinate_dofs[1] * FE3_C0_D01_Q6[0][0][0] + coordinate_dofs[3] * FE3_C0_D01_Q6[0][0][1];
    alignas(32) double sp[4];
    sp[0] = J_c0 * J_c3;
    sp[1] = J_c1 * J_c2;
    sp[2] = sp[0] + -1 * sp[1];
    sp[3] = std::abs(sp[2]);
    alignas(32) double BF0[6] = {};
    for (int iq = 0; iq < 6; ++iq)
    {
        // Quadrature loop body setup (num_points=6)
        // Unstructured varying computations for num_points=6
        double w0 = 0.0;
        for (int ic = 0; ic < 6; ++ic)
            w0 += w[0][ic] * FE6_C0_Q6[0][iq][ic];
        alignas(32) double sv6[1];
        sv6[0] = sp[3] * w0;
        const double fw0 = sv6[0] * weights6[iq];
        for (int i = 0; i < 6; ++i)
            BF0[i] += fw0 * FE6_C0_Q6[0][iq][i];
    }
    std::fill(&A[0], &A[6], 0.0);
    for (int i = 0; i < 6; ++i)
        A[i] += BF0[i];
  }

};


class biharmonic_form_0: public ufc::form
{
public:

  biharmonic_form_0() : ufc::form()
  {
    // Do nothing
  }

  ~biharmonic_form_0() override
  {
    // Do nothing
  }

  const char * signature() const final override
  {
    return "4ee9dfc5b316b31baa8b5927bfe42a799f038f1cda8e7e23e65137f5e82e43f882d69b90b4b024bcfe06e1a05077850af213d3a05483a093418521eac5250977";
  }

  std::size_t rank() const final override
  {
    return 2;
  }

  std::size_t num_coefficients() const final override
  {
    return 1;
  }

  std::size_t original_coefficient_position(std::size_t i) const final override
  {
    static const std::vector<std::size_t> position({0});
    return position[i];
  }

  ufc::finite_element * create_coordinate_finite_element() const final override
  {
    return new biharmonic_finite_element_1();
  }

  ufc::dofmap * create_coordinate_dofmap() const final override
  {
    return new biharmonic_dofmap_1();
   }

  ufc::coordinate_mapping * create_coordinate_mapping() const final override
  {
    return nullptr;
  }

  ufc::finite_element * create_finite_element(std::size_t i) const final override
  {
    switch (i)
    {
    case 0:
      {
        return new biharmonic_finite_element_3();
        break;
      }
    case 1:
      {
        return new biharmonic_finite_element_3();
        break;
      }
    case 2:
      {
        return new biharmonic_finite_element_2();
        break;
      }
    }
    
    return 0;
  }

  ufc::dofmap * create_dofmap(std::size_t i) const final override
  {
    switch (i)
    {
    case 0:
      {
        return new biharmonic_dofmap_3();
        break;
      }
    case 1:
      {
        return new biharmonic_dofmap_3();
        break;
      }
    case 2:
      {
        return new biharmonic_dofmap_2();
        break;
      }
    }
    
    return 0;
  }

  std::size_t max_cell_subdomain_id() const final override
  {
    return 0;
  }

  std::size_t max_exterior_facet_subdomain_id() const final override
  {
    return 0;
  }

  std::size_t max_interior_facet_subdomain_id() const final override
  {
    return 0;
  }

  std::size_t max_vertex_subdomain_id() const final override
  {
    return 0;
  }

  std::size_t max_custom_subdomain_id() const final override
  {
    return 0;
  }

  std::size_t max_cutcell_subdomain_id() const final override
  {
    return 0;
  }

  std::size_t max_interface_subdomain_id() const final override
  {
    return 0;
  }

  std::size_t max_overlap_subdomain_id() const final override
  {
    return 0;
  }

  bool has_cell_integrals() const final override
  {
    return true;
  }

  bool has_exterior_facet_integrals() const final override
  {
    return false;
  }

  bool has_interior_facet_integrals() const final override
  {
    return true;
  }

  bool has_vertex_integrals() const final override
  {
    return false;
  }

  bool has_custom_integrals() const final override
  {
    return false;
  }

  bool has_cutcell_integrals() const final override
  {
    return false;
  }

  bool has_interface_integrals() const final override
  {
    return false;
  }

  bool has_overlap_integrals() const final override
  {
    return false;
  }

  ufc::cell_integral * create_cell_integral(std::size_t subdomain_id) const final override
  {
    return 0;
  }

  ufc::exterior_facet_integral * create_exterior_facet_integral(std::size_t subdomain_id) const final override
  {
    return 0;
  }

  ufc::interior_facet_integral * create_interior_facet_integral(std::size_t subdomain_id) const final override
  {
    return 0;
  }

  ufc::vertex_integral * create_vertex_integral(std::size_t subdomain_id) const final override
  {
    return 0;
  }

  ufc::custom_integral * create_custom_integral(std::size_t subdomain_id) const final override
  {
    return 0;
  }

  ufc::cutcell_integral * create_cutcell_integral(std::size_t subdomain_id) const final override
  {
    return 0;
  }

  ufc::interface_integral * create_interface_integral(std::size_t subdomain_id) const final override
  {
    return 0;
  }

  ufc::overlap_integral * create_overlap_integral(std::size_t subdomain_id) const final override
  {
    return 0;
  }

  ufc::cell_integral * create_default_cell_integral() const final override
  {
    return new biharmonic_cell_integral_0_otherwise();
  }

  ufc::exterior_facet_integral * create_default_exterior_facet_integral() const final override
  {
    return 0;
  }

  ufc::interior_facet_integral * create_default_interior_facet_integral() const final override
  {
    return new biharmonic_interior_facet_integral_0_otherwise();
  }

  ufc::vertex_integral * create_default_vertex_integral() const final override
  {
    return 0;
  }

  ufc::custom_integral * create_default_custom_integral() const final override
  {
    return 0;
  }

  ufc::cutcell_integral * create_default_cutcell_integral() const final override
  {
    return 0;
  }

  ufc::interface_integral * create_default_interface_integral() const final override
  {
    return 0;
  }

  ufc::overlap_integral * create_default_overlap_integral() const final override
  {
    return 0;
  }

};


class biharmonic_form_1: public ufc::form
{
public:

  biharmonic_form_1() : ufc::form()
  {
    // Do nothing
  }

  ~biharmonic_form_1() override
  {
    // Do nothing
  }

  const char * signature() const final override
  {
    return "9050c89a2c6705ec05da94c4d5c93d1d37302b91b6887ea41c8842c0fbd8b77d1bd95bd6274fd5b43b484e3d801496a90a286f3c61fe7a258147be1208ad6abe";
  }

  std::size_t rank() const final override
  {
    return 1;
  }

  std::size_t num_coefficients() const final override
  {
    return 1;
  }

  std::size_t original_coefficient_position(std::size_t i) const final override
  {
    static const std::vector<std::size_t> position({0});
    return position[i];
  }

  ufc::finite_element * create_coordinate_finite_element() const final override
  {
    return new biharmonic_finite_element_1();
  }

  ufc::dofmap * create_coordinate_dofmap() const final override
  {
    return new biharmonic_dofmap_1();
   }

  ufc::coordinate_mapping * create_coordinate_mapping() const final override
  {
    return nullptr;
  }

  ufc::finite_element * create_finite_element(std::size_t i) const final override
  {
    switch (i)
    {
    case 0:
      {
        return new biharmonic_finite_element_3();
        break;
      }
    case 1:
      {
        return new biharmonic_finite_element_3();
        break;
      }
    }
    
    return 0;
  }

  ufc::dofmap * create_dofmap(std::size_t i) const final override
  {
    switch (i)
    {
    case 0:
      {
        return new biharmonic_dofmap_3();
        break;
      }
    case 1:
      {
        return new biharmonic_dofmap_3();
        break;
      }
    }
    
    return 0;
  }

  std::size_t max_cell_subdomain_id() const final override
  {
    return 0;
  }

  std::size_t max_exterior_facet_subdomain_id() const final override
  {
    return 0;
  }

  std::size_t max_interior_facet_subdomain_id() const final override
  {
    return 0;
  }

  std::size_t max_vertex_subdomain_id() const final override
  {
    return 0;
  }

  std::size_t max_custom_subdomain_id() const final override
  {
    return 0;
  }

  std::size_t max_cutcell_subdomain_id() const final override
  {
    return 0;
  }

  std::size_t max_interface_subdomain_id() const final override
  {
    return 0;
  }

  std::size_t max_overlap_subdomain_id() const final override
  {
    return 0;
  }

  bool has_cell_integrals() const final override
  {
    return true;
  }

  bool has_exterior_facet_integrals() const final override
  {
    return false;
  }

  bool has_interior_facet_integrals() const final override
  {
    return false;
  }

  bool has_vertex_integrals() const final override
  {
    return false;
  }

  bool has_custom_integrals() const final override
  {
    return false;
  }

  bool has_cutcell_integrals() const final override
  {
    return false;
  }

  bool has_interface_integrals() const final override
  {
    return false;
  }

  bool has_overlap_integrals() const final override
  {
    return false;
  }

  ufc::cell_integral * create_cell_integral(std::size_t subdomain_id) const final override
  {
    return 0;
  }

  ufc::exterior_facet_integral * create_exterior_facet_integral(std::size_t subdomain_id) const final override
  {
    return 0;
  }

  ufc::interior_facet_integral * create_interior_facet_integral(std::size_t subdomain_id) const final override
  {
    return 0;
  }

  ufc::vertex_integral * create_vertex_integral(std::size_t subdomain_id) const final override
  {
    return 0;
  }

  ufc::custom_integral * create_custom_integral(std::size_t subdomain_id) const final override
  {
    return 0;
  }

  ufc::cutcell_integral * create_cutcell_integral(std::size_t subdomain_id) const final override
  {
    return 0;
  }

  ufc::interface_integral * create_interface_integral(std::size_t subdomain_id) const final override
  {
    return 0;
  }

  ufc::overlap_integral * create_overlap_integral(std::size_t subdomain_id) const final override
  {
    return 0;
  }

  ufc::cell_integral * create_default_cell_integral() const final override
  {
    return new biharmonic_cell_integral_1_otherwise();
  }

  ufc::exterior_facet_integral * create_default_exterior_facet_integral() const final override
  {
    return 0;
  }

  ufc::interior_facet_integral * create_default_interior_facet_integral() const final override
  {
    return 0;
  }

  ufc::vertex_integral * create_default_vertex_integral() const final override
  {
    return 0;
  }

  ufc::custom_integral * create_default_custom_integral() const final override
  {
    return 0;
  }

  ufc::cutcell_integral * create_default_cutcell_integral() const final override
  {
    return 0;
  }

  ufc::interface_integral * create_default_interface_integral() const final override
  {
    return 0;
  }

  ufc::overlap_integral * create_default_overlap_integral() const final override
  {
    return 0;
  }

};

// DOLFIN wrappers

// Standard library includes
#include <string>

// DOLFIN includes
#include <dolfin/common/NoDeleter.h>
#include <dolfin/mesh/Mesh.h>
#include <dolfin/mesh/MultiMesh.h>
#include <dolfin/fem/FiniteElement.h>
#include <dolfin/fem/DofMap.h>
#include <dolfin/fem/Form.h>
#include <dolfin/fem/MultiMeshForm.h>
#include <dolfin/function/FunctionSpace.h>
#include <dolfin/function/MultiMeshFunctionSpace.h>
#include <dolfin/function/GenericFunction.h>
#include <dolfin/function/CoefficientAssigner.h>
#include <dolfin/function/MultiMeshCoefficientAssigner.h>
#include <dolfin/adaptivity/ErrorControl.h>
#include <dolfin/adaptivity/GoalFunctional.h>
#include <dolfin/la/GenericVector.h>

namespace Biharmonic
{

class CoefficientSpace_alpha: public dolfin::FunctionSpace
{
public:

  // Constructor for standard function space
  CoefficientSpace_alpha(std::shared_ptr<const dolfin::Mesh> mesh):
    dolfin::FunctionSpace(mesh,
                          std::make_shared<const dolfin::FiniteElement>(std::make_shared<biharmonic_finite_element_2>()),
                          std::make_shared<const dolfin::DofMap>(std::make_shared<biharmonic_dofmap_2>(), *mesh))
  {
    // Do nothing
  }

  // Constructor for constrained function space
  CoefficientSpace_alpha(std::shared_ptr<const dolfin::Mesh> mesh, std::shared_ptr<const dolfin::SubDomain> constrained_domain):
    dolfin::FunctionSpace(mesh,
                          std::make_shared<const dolfin::FiniteElement>(std::make_shared<biharmonic_finite_element_2>()),
                          std::make_shared<const dolfin::DofMap>(std::make_shared<biharmonic_dofmap_2>(), *mesh, constrained_domain))
  {
    // Do nothing
  }

};

class CoefficientSpace_f: public dolfin::FunctionSpace
{
public:

  // Constructor for standard function space
  CoefficientSpace_f(std::shared_ptr<const dolfin::Mesh> mesh):
    dolfin::FunctionSpace(mesh,
                          std::make_shared<const dolfin::FiniteElement>(std::make_shared<biharmonic_finite_element_3>()),
                          std::make_shared<const dolfin::DofMap>(std::make_shared<biharmonic_dofmap_3>(), *mesh))
  {
    // Do nothing
  }

  // Constructor for constrained function space
  CoefficientSpace_f(std::shared_ptr<const dolfin::Mesh> mesh, std::shared_ptr<const dolfin::SubDomain> constrained_domain):
    dolfin::FunctionSpace(mesh,
                          std::make_shared<const dolfin::FiniteElement>(std::make_shared<biharmonic_finite_element_3>()),
                          std::make_shared<const dolfin::DofMap>(std::make_shared<biharmonic_dofmap_3>(), *mesh, constrained_domain))
  {
    // Do nothing
  }

};

class Form_a_FunctionSpace_0: public dolfin::FunctionSpace
{
public:

  // Constructor for standard function space
  Form_a_FunctionSpace_0(std::shared_ptr<const dolfin::Mesh> mesh):
    dolfin::FunctionSpace(mesh,
                          std::make_shared<const dolfin::FiniteElement>(std::make_shared<biharmonic_finite_element_3>()),
                          std::make_shared<const dolfin::DofMap>(std::make_shared<biharmonic_dofmap_3>(), *mesh))
  {
    // Do nothing
  }

  // Constructor for constrained function space
  Form_a_FunctionSpace_0(std::shared_ptr<const dolfin::Mesh> mesh, std::shared_ptr<const dolfin::SubDomain> constrained_domain):
    dolfin::FunctionSpace(mesh,
                          std::make_shared<const dolfin::FiniteElement>(std::make_shared<biharmonic_finite_element_3>()),
                          std::make_shared<const dolfin::DofMap>(std::make_shared<biharmonic_dofmap_3>(), *mesh, constrained_domain))
  {
    // Do nothing
  }

};

class Form_a_FunctionSpace_1: public dolfin::FunctionSpace
{
public:

  // Constructor for standard function space
  Form_a_FunctionSpace_1(std::shared_ptr<const dolfin::Mesh> mesh):
    dolfin::FunctionSpace(mesh,
                          std::make_shared<const dolfin::FiniteElement>(std::make_shared<biharmonic_finite_element_3>()),
                          std::make_shared<const dolfin::DofMap>(std::make_shared<biharmonic_dofmap_3>(), *mesh))
  {
    // Do nothing
  }

  // Constructor for constrained function space
  Form_a_FunctionSpace_1(std::shared_ptr<const dolfin::Mesh> mesh, std::shared_ptr<const dolfin::SubDomain> constrained_domain):
    dolfin::FunctionSpace(mesh,
                          std::make_shared<const dolfin::FiniteElement>(std::make_shared<biharmonic_finite_element_3>()),
                          std::make_shared<const dolfin::DofMap>(std::make_shared<biharmonic_dofmap_3>(), *mesh, constrained_domain))
  {
    // Do nothing
  }

};

class Form_a_MultiMeshFunctionSpace_0: public dolfin::MultiMeshFunctionSpace
{
public:

  // Constructor for multimesh function space
  Form_a_MultiMeshFunctionSpace_0(std::shared_ptr<const dolfin::MultiMesh> multimesh): dolfin::MultiMeshFunctionSpace(multimesh)
  {
    // Create and add standard function spaces
    for (std::size_t part = 0; part < multimesh->num_parts(); part++)
    {
      std::shared_ptr<const dolfin::FunctionSpace> V(new Form_a_FunctionSpace_0(multimesh->part(part)));
      add(V);
    }

    // Build multimesh function space
    build();
  }

};

class Form_a_MultiMeshFunctionSpace_1: public dolfin::MultiMeshFunctionSpace
{
public:

  // Constructor for multimesh function space
  Form_a_MultiMeshFunctionSpace_1(std::shared_ptr<const dolfin::MultiMesh> multimesh): dolfin::MultiMeshFunctionSpace(multimesh)
  {
    // Create and add standard function spaces
    for (std::size_t part = 0; part < multimesh->num_parts(); part++)
    {
      std::shared_ptr<const dolfin::FunctionSpace> V(new Form_a_FunctionSpace_1(multimesh->part(part)));
      add(V);
    }

    // Build multimesh function space
    build();
  }

};

typedef CoefficientSpace_alpha Form_a_FunctionSpace_2;

class Form_a: public dolfin::Form
{
public:

  // Constructor
  Form_a(std::shared_ptr<const dolfin::FunctionSpace> V1, std::shared_ptr<const dolfin::FunctionSpace> V0):
    dolfin::Form(2, 1), alpha(*this, 0)
  {
    _function_spaces[0] = V0;
    _function_spaces[1] = V1;

    _ufc_form = std::make_shared<const biharmonic_form_0>();
  }

  // Constructor
  Form_a(std::shared_ptr<const dolfin::FunctionSpace> V1, std::shared_ptr<const dolfin::FunctionSpace> V0, std::shared_ptr<const dolfin::GenericFunction> alpha):
    dolfin::Form(2, 1), alpha(*this, 0)
  {
    _function_spaces[0] = V0;
    _function_spaces[1] = V1;

    this->alpha = alpha;

    _ufc_form = std::make_shared<const biharmonic_form_0>();
  }

  // Destructor
  ~Form_a()
  {}

  /// Return the number of the coefficient with this name
  virtual std::size_t coefficient_number(const std::string& name) const
  {
    if (name == "alpha")
      return 0;

    dolfin::dolfin_error("generated code for class Form",
                         "access coefficient data",
                         "Invalid coefficient");
    return 0;
  }

  /// Return the name of the coefficient with this number
  virtual std::string coefficient_name(std::size_t i) const
  {
    switch (i)
    {
    case 0:
      return "alpha";
    }

    dolfin::dolfin_error("generated code for class Form",
                         "access coefficient data",
                         "Invalid coefficient");
    return "unnamed";
  }

  // Typedefs
  typedef Form_a_FunctionSpace_0 TestSpace;
  typedef Form_a_FunctionSpace_1 TrialSpace;
  typedef Form_a_MultiMeshFunctionSpace_0 MultiMeshTestSpace;
  typedef Form_a_MultiMeshFunctionSpace_1 MultiMeshTrialSpace;
  typedef Form_a_FunctionSpace_2 CoefficientSpace_alpha;

  // Coefficients
  dolfin::CoefficientAssigner alpha;
};

class MultiMeshForm_a: public dolfin::MultiMeshForm
{
public:

  // Constructor
  MultiMeshForm_a(std::shared_ptr<const dolfin::MultiMeshFunctionSpace> V1, std::shared_ptr<const dolfin::MultiMeshFunctionSpace> V0):
    dolfin::MultiMeshForm(V1, V0), alpha(*this, 0)
  {
    // Create and add standard forms
    std::size_t num_parts = V0->num_parts(); // assume all equal and pick first
    for (std::size_t part = 0; part < num_parts; part++)
    {
      std::shared_ptr<dolfin::Form> a(new Form_a(V1->part(part), V0->part(part)));
    add(a);

    }
    // Build multimesh form
    build();

    /// Assign coefficients

  }

  // Constructor
  MultiMeshForm_a(std::shared_ptr<const dolfin::MultiMeshFunctionSpace> V1, std::shared_ptr<const dolfin::MultiMeshFunctionSpace> V0, std::shared_ptr<const dolfin::GenericFunction> alpha):
    dolfin::MultiMeshForm(V1, V0), alpha(*this, 0)
  {
    // Create and add standard forms
    std::size_t num_parts = V0->num_parts(); // assume all equal and pick first
    for (std::size_t part = 0; part < num_parts; part++)
    {
      std::shared_ptr<dolfin::Form> a(new Form_a(V1->part(part), V0->part(part)));
    add(a);

    }
    // Build multimesh form
    build();

    /// Assign coefficients
    this->alpha = alpha;

  }

  // Destructor
  ~MultiMeshForm_a()
  {}

  /// Return the number of the coefficient with this name
  virtual std::size_t coefficient_number(const std::string& name) const
  {
    if (name == "alpha")
      return 0;

    dolfin::dolfin_error("generated code for class Form",
                         "access coefficient data",
                         "Invalid coefficient");
    return 0;
  }

  /// Return the name of the coefficient with this number
  virtual std::string coefficient_name(std::size_t i) const
  {
    switch (i)
    {
    case 0:
      return "alpha";
    }

    dolfin::dolfin_error("generated code for class Form",
                         "access coefficient data",
                         "Invalid coefficient");
    return "unnamed";
  }

  // Typedefs
  typedef Form_a_FunctionSpace_0 TestSpace;
  typedef Form_a_FunctionSpace_1 TrialSpace;
  typedef Form_a_MultiMeshFunctionSpace_0 MultiMeshTestSpace;
  typedef Form_a_MultiMeshFunctionSpace_1 MultiMeshTrialSpace;
  typedef Form_a_FunctionSpace_2 CoefficientSpace_alpha;

  // Coefficients
  dolfin::MultiMeshCoefficientAssigner alpha;
};

class Form_L_FunctionSpace_0: public dolfin::FunctionSpace
{
public:

  // Constructor for standard function space
  Form_L_FunctionSpace_0(std::shared_ptr<const dolfin::Mesh> mesh):
    dolfin::FunctionSpace(mesh,
                          std::make_shared<const dolfin::FiniteElement>(std::make_shared<biharmonic_finite_element_3>()),
                          std::make_shared<const dolfin::DofMap>(std::make_shared<biharmonic_dofmap_3>(), *mesh))
  {
    // Do nothing
  }

  // Constructor for constrained function space
  Form_L_FunctionSpace_0(std::shared_ptr<const dolfin::Mesh> mesh, std::shared_ptr<const dolfin::SubDomain> constrained_domain):
    dolfin::FunctionSpace(mesh,
                          std::make_shared<const dolfin::FiniteElement>(std::make_shared<biharmonic_finite_element_3>()),
                          std::make_shared<const dolfin::DofMap>(std::make_shared<biharmonic_dofmap_3>(), *mesh, constrained_domain))
  {
    // Do nothing
  }

};

class Form_L_MultiMeshFunctionSpace_0: public dolfin::MultiMeshFunctionSpace
{
public:

  // Constructor for multimesh function space
  Form_L_MultiMeshFunctionSpace_0(std::shared_ptr<const dolfin::MultiMesh> multimesh): dolfin::MultiMeshFunctionSpace(multimesh)
  {
    // Create and add standard function spaces
    for (std::size_t part = 0; part < multimesh->num_parts(); part++)
    {
      std::shared_ptr<const dolfin::FunctionSpace> V(new Form_L_FunctionSpace_0(multimesh->part(part)));
      add(V);
    }

    // Build multimesh function space
    build();
  }

};

typedef CoefficientSpace_f Form_L_FunctionSpace_1;

class Form_L: public dolfin::Form
{
public:

  // Constructor
  Form_L(std::shared_ptr<const dolfin::FunctionSpace> V0):
    dolfin::Form(1, 1), f(*this, 0)
  {
    _function_spaces[0] = V0;

    _ufc_form = std::make_shared<const biharmonic_form_1>();
  }

  // Constructor
  Form_L(std::shared_ptr<const dolfin::FunctionSpace> V0, std::shared_ptr<const dolfin::GenericFunction> f):
    dolfin::Form(1, 1), f(*this, 0)
  {
    _function_spaces[0] = V0;

    this->f = f;

    _ufc_form = std::make_shared<const biharmonic_form_1>();
  }

  // Destructor
  ~Form_L()
  {}

  /// Return the number of the coefficient with this name
  virtual std::size_t coefficient_number(const std::string& name) const
  {
    if (name == "f")
      return 0;

    dolfin::dolfin_error("generated code for class Form",
                         "access coefficient data",
                         "Invalid coefficient");
    return 0;
  }

  /// Return the name of the coefficient with this number
  virtual std::string coefficient_name(std::size_t i) const
  {
    switch (i)
    {
    case 0:
      return "f";
    }

    dolfin::dolfin_error("generated code for class Form",
                         "access coefficient data",
                         "Invalid coefficient");
    return "unnamed";
  }

  // Typedefs
  typedef Form_L_FunctionSpace_0 TestSpace;
  typedef Form_L_MultiMeshFunctionSpace_0 MultiMeshTestSpace;
  typedef Form_L_FunctionSpace_1 CoefficientSpace_f;

  // Coefficients
  dolfin::CoefficientAssigner f;
};

class MultiMeshForm_L: public dolfin::MultiMeshForm
{
public:

  // Constructor
  MultiMeshForm_L(std::shared_ptr<const dolfin::MultiMeshFunctionSpace> V0):
    dolfin::MultiMeshForm(V0), f(*this, 0)
  {
    // Create and add standard forms
    std::size_t num_parts = V0->num_parts(); // assume all equal and pick first
    for (std::size_t part = 0; part < num_parts; part++)
    {
      std::shared_ptr<dolfin::Form> a(new Form_L(V0->part(part)));
    add(a);

    }
    // Build multimesh form
    build();

    /// Assign coefficients

  }

  // Constructor
  MultiMeshForm_L(std::shared_ptr<const dolfin::MultiMeshFunctionSpace> V0, std::shared_ptr<const dolfin::GenericFunction> f):
    dolfin::MultiMeshForm(V0), f(*this, 0)
  {
    // Create and add standard forms
    std::size_t num_parts = V0->num_parts(); // assume all equal and pick first
    for (std::size_t part = 0; part < num_parts; part++)
    {
      std::shared_ptr<dolfin::Form> a(new Form_L(V0->part(part)));
    add(a);

    }
    // Build multimesh form
    build();

    /// Assign coefficients
    this->f = f;

  }

  // Destructor
  ~MultiMeshForm_L()
  {}

  /// Return the number of the coefficient with this name
  virtual std::size_t coefficient_number(const std::string& name) const
  {
    if (name == "f")
      return 0;

    dolfin::dolfin_error("generated code for class Form",
                         "access coefficient data",
                         "Invalid coefficient");
    return 0;
  }

  /// Return the name of the coefficient with this number
  virtual std::string coefficient_name(std::size_t i) const
  {
    switch (i)
    {
    case 0:
      return "f";
    }

    dolfin::dolfin_error("generated code for class Form",
                         "access coefficient data",
                         "Invalid coefficient");
    return "unnamed";
  }

  // Typedefs
  typedef Form_L_FunctionSpace_0 TestSpace;
  typedef Form_L_MultiMeshFunctionSpace_0 MultiMeshTestSpace;
  typedef Form_L_FunctionSpace_1 CoefficientSpace_f;

  // Coefficients
  dolfin::MultiMeshCoefficientAssigner f;
};

// Class typedefs
typedef Form_a BilinearForm;
typedef MultiMeshForm_a MultiMeshBilinearForm;
typedef Form_a JacobianForm;
typedef MultiMeshForm_a MultiMeshJacobianForm;
typedef Form_L LinearForm;
typedef MultiMeshForm_L MultiMeshLinearForm;
typedef Form_L ResidualForm;
typedef MultiMeshForm_L MultiMeshResidualForm;
typedef Form_a::TestSpace FunctionSpace;
typedef Form_a::MultiMeshTestSpace MultiMeshFunctionSpace;

}

#endif
